<?php
/**
 * Created by PhpStorm.
 * User: Wangqing
 * Date: 2018/7/12
 * Time: 21:46
 */

namespace App\Common;

/*自定义类
 * **/
use Illuminate\Support\Facades\Request;
use Illuminate\Support\MessageBag;

class Helper
{
    /* @name 引用无限级分类
     * @param Array $items
     * @return Array $tree
     */
    public static function generateTree($items)
    {
        $route = app('request')->route()->action['as'];
        $tree = array();
        foreach ($items as $item) {
            if (isset($items[$item['pid']])) {
                $items[$item['pid']]['child'][] = &$items[$item['id']];
                if ($route==$item['route']){
                    $items[$item['pid']]['active'] = true;
                    if($items[$item['pid']]['pid']){
                        $pid = $items[$item['pid']]['pid'];
                        $items[$pid]['active'] = true;
                    }
                }
            } else {
                if (!$item['pid']){
                    $tree[] = &$items[$item['id']];
                }
            }
        }
        return $tree;
    }

    /* @name 引用无限级分类
     * @param Array $items
     * @return Array $tree
     */
    public static function generateTrees($items)
    {
        $tree = array();
        foreach ($items as $item) {
            if (isset($items[$item['pid']])) {
                $items[$item['pid']]['children'][] = &$items[$item['id']];
            } else {
                $tree[] = &$items[$item['id']];
            }

        }
        return $tree;
    }
    //传递一个父级分类ID返回所有子分类ID
    /**
     *@param $cate 全部分类数组
     *@param $pid 父级ID
     *@param $flag 是否包括父级自己的ID，默认不包括
     **/
    static public function getChildsId($cate, $pid, $flag = 0) {
        $arr = array();
        if ($flag) {
            $arr[] = $pid;
        }
        foreach ($cate as $v) {
            if ($v['pid'] == $pid) {
                $arr[] = $v['id'];
                $arr = array_merge($arr , self::getChildsId($cate, $v['id']));
            }
        }

        return $arr;
    }
    /* @name 递归无限级分类
     * @param Array $cate 全部分类数组
     * @param String $delimiter
     * @param  Integer $parent_id
     * @param  Integer $level
     * @param Array $arr
     */
     public static function toLevel($cate, $delimiter = '———', $parent_id = 0, $level = 0) {
        $arr = array();
        foreach ($cate as $v) {
            if ($v['pid'] == $parent_id) {
                $v['level'] = $level + 1;
                ($level>0)?$v['delimiter'] =$delimiter:  $v['delimiter'] ='';
                $arr[] = $v;
                $arr = array_merge($arr, self::toLevel($cate, $delimiter, $v['id'], $v['level']));
            }
        }
        return $arr;

    }
    /* @name 提交成功提示
     * @param string $message
     * @param  string $redirect
     * @param array $data
     * @return  $this|array
     */
    public static function success($message='',$redirect="",$data=[]){
        if(Request::ajax()){
            Request::session()->flash('message',$message);
            return [
                'status'=>true,
                'message'=>$message,
                'redirect'=>$redirect,
                'data'=>$data,
            ];
        }else
            return \Redirect::to($redirect)->with($message);
    }

    /* @name返回错误信息
     * @param $message
     * @param string $redirect
     * @param array $data
     * @return $this|array
     */
    public static function error($message='',$redirect="",$data=[]){
        if ($message instanceof MessageBag) {
            $message = $message->first();
        }
        if(Request::ajax())
            return [
                'status'=>false,
                'message'=>$message,
                'redirect'=>$redirect,
                'data'=>$data,
            ];
        else
            $message = new MessageBag([$message]);
        if(!empty($redirect)){
            return \Redirect::to($redirect)->withErrors($message);
        }else{
            return \Redirect::back()->withErrors($message);
        }
    }
    public static function str_rand($length = 32, $char = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ') {
        if(!is_int($length) || $length < 0) {
            return false;
        }
        $string = '';
        for($i = $length; $i > 0; $i--) {
            $string .= $char[mt_rand(0, strlen($char) - 1)];
        }
        return $string;
    }

    public static function random($length, $numeric = FALSE) {
        $seed = base_convert(md5(microtime() . $_SERVER['DOCUMENT_ROOT']), 16, $numeric ? 10 : 35);
        $seed = $numeric ? (str_replace('0', '', $seed) . '012340567890') : ($seed . 'zZ' . strtoupper($seed));
        if ($numeric) {
            $hash = '';
        } else {
            $hash = chr(rand(1, 26) + rand(0, 1) * 32 + 64);
            $length--;
        }
        $max = strlen($seed) - 1;
        for ($i = 0; $i < $length; $i++) {
            $hash .= $seed{mt_rand(0, $max)};
        }
        return $hash;
    }

    /**
     * @param $number
     * @return float|string
     * 格式化数字
     */
    public static function formatNumber($number){
        if (empty($number) || !is_numeric($number)) return $number;
        $unit = "";
        if ($number > 10000) {
            $leftNumber = floor($number / 10000);
            $rightNumber = round(($number % 10000) / 10000, 2);
            $number = floatval($leftNumber + $rightNumber);
            $unit = "w";
        } else {
            $decimals = $number > 1 ? 2 : 6;
            $number = (float)number_format($number, $decimals, '.', '');
        }
        return (string)$number . $unit;
    }

    /**
     * @param $end_at
     * @param $start_at
     * @return string
     * 时间差
     */
    public static function timeDiff($end_at,$start_at){
        $timediff = $end_at - $start_at;
        $remain = $timediff%86400;
        $hours = intval($remain/3600);

        $remain = $remain%3600;

        $mins = intval($remain/60);

        $secs = $remain%60;

        if($hours){
            $hours = sprintf('%02d',$hours);
        }else{
            $hours='00';
        }
        if($mins){
            $mins = sprintf('%02d',$mins);
        }else{
            $mins='00';
        }
        if($secs){
            $secs = sprintf('%02d',$secs);
        }else{
            $secs='00';
        }
        $replaytime = $hours.':'.$mins.':'.$secs;
        return $replaytime;
    }
}