<?php

namespace App\Listeners\Commission\Partner;

use App\Events\CommissionOrder;
use App\Events\OrderPaySuccess;
use App\Models\ShimmerLiveshopCashApply;
use App\Models\ShimmerLiveshopCommissionLevel;
use App\Models\ShimmerLiveshopCommissionOrder;
use App\Models\ShimmerLiveshopStoreAsset;
use App\Models\ShimmerLiveshopStoreOrder;
use App\Models\ShimmerLiveshopUser;
use App\Modules\Liveapi\Models\ShimmerLiveshopAgent;
use App\Modules\Liveapi\Models\ShimmerLiveshopUserRelation;
use App\Services\Commission\CashApplyService;
use App\Services\PluginService;
use Shimmer_liveshop_plugin_partner\Models\ShimmerLiveshopPartnerRelation;
use Shimmer_liveshop_plugin_partner\Models\ShimmerLiveshopPartnerUser;
use Shimmer_liveshop_plugin_partner\Models\ShimmerLiveshopPartnerUserLog;
use Shimmer_liveshop_plugin_partner\Models\ShimmerLiveshopPartnerUserPay;
use Shimmer_liveshop_plugin_partner\Services\ConfigService;
use Shimmer_liveshop_plugin_partner\Services\Partner\PartnerService;
use Shimmer_liveshop_plugin_partner\Services\Partner\PlatformReward\PlatformRewardPartnerCommissionService;
use Shimmer_liveshop_plugin_partner\Services\Partner\SameLevel\SameLevelPartnerCommissionService;

class PartnerSettledOrderListener
{
    public $event;
    public $config;
    public $order;
    public $grade;

    /**
     * Handle the event.
     *
     * @param  CommissionOrder  $event
     * @return void
     */
    public function handle(OrderPaySuccess $event)
    {
        if (!PluginService::isEnabled('partner')) {
            return true;
        }
        $this->event = $event;
        $this->getAnchorSettleConfig();
    }

    /**
     * 付费开通主播配置
     */
    public function getAnchorSettleConfig(){
        $this->config = ConfigService::getValueByType(1);
        $this->getOrder();
    }

    /**
     * 分销等级用户
     */
    public function getCommissionRelation(){
        $relation = ShimmerLiveshopUserRelation::where('uid',$this->order->uid)->first();
        if ($relation){
            $this->getMaxCommissionRelation($relation);
        }
    }

    /**
     * @param $relation
     * 获取最大的分销关系
     */
    public function getMaxCommissionRelation($relation){
        $maxPayLevel = ShimmerLiveshopPartnerUserPay::where('uid',$relation->parent_uid)->orderBy('pay_price','desc')->first();
        if (!$maxPayLevel) return true;
        $commission_rate = $maxPayLevel->buy_partner_rate;
        $isGive = 2;
        if ($maxPayLevel->give_partner_nums>0){
            $hasGives = ShimmerLiveshopPartnerRelation::where('parent_uid',$relation->parent_uid)->where('type',1)->count();
            if ($hasGives<$maxPayLevel->give_partner_nums){
                $commission_rate = 100;
                $isGive = 1;
            }
        }

        $commission = bcmul($this->order->pay_price,$commission_rate/100,2);
        $this->storeCommissionOrder($relation,$commission,$commission_rate,$isGive);

        /**********判断赠送名额是否参与分润*****************/
        $commissionConfig = ConfigService::getValueByType(2);
        if ((!$commissionConfig || $commissionConfig['can_commission']==0) && $isGive==1){
            return false;
        }
        /**********判断赠送名额是否参与分润*****************/
        //平级奖
        SameLevelPartnerCommissionService::sameLevelPartnerCommission($this->order,$relation->parent_uid,$maxPayLevel->level_type,'samelevel');
        //PlatformRewardPartnerCommissionService::platformRewardPartnerCommission($this->order,$relation->parent_uid);
    }

    /**
     * @param $type
     * @param $uid
     * @param $commissionData
     * 保存分佣订单
     */
    public function storeCommissionOrder($relation,$commission,$commission_rate,$isGive){
        $commissionOrder = ShimmerLiveshopCommissionOrder::where('order_id',$this->order->id)->where('uid',$this->order->uid)
            ->where('agent_uid',$relation->parent_uid)
            ->first();
        if ($commissionOrder) return false;
        $model = new ShimmerLiveshopCommissionOrder();
        $data = array(
            'uniacid'=>$this->order->uniacid,
            'order_id'=>$this->order->id,
            'uid'=>$this->order->uid,
            'agent_uid'=>$relation->parent_uid,
            'price'=>$this->order->pay_price,
            'rate'=>$commission_rate,
            'level'=>1,
            'commission'=>$commission,
            'status'=>2,
            'title'=>ShimmerLiveshopCommissionOrder::$type[6],
            'has_withdraw'=>0,
            'type'=>6,
            'send_type'=>0,
            'withdraw_time'=>0
        );
        $model->fill($data)->save();
        $this->order->agent_uid = $relation->parent_uid;
        $this->order->save();
        PartnerService::savePartnerRelation($this->order->uid,$relation->parent_uid,$isGive,$this->order->pay_price,$commission,$this->order->id,2);

        $this->sendAmount($relation->parent_uid,$commission,$model->id,$this->order);
    }

    /**
     * @param $uid
     * @param $commission
     * 发送红包
     */
    public function sendAmount($uid,$commission,$commission_id,$order){
        $user = ShimmerLiveshopUser::find($uid);
        if (!$user) return;
        $data =[
            'partner_trade_no' => CashApplyService::createOrderNo(),
            'openid' => $user->wxapp_openid,
            'check_name' => 'FORCE_CHECK',
            're_user_name' => $user->realname,
            'amount' => $commission*100,
            'desc' => '直推合伙人奖励',
        ];
        $cashApplyId = CashApplyService::createCashApply($data,$uid,$order->uniacid,'system-wxapp',6,[$commission_id],2);
        try {
            $app = \App\Modules\Liveapi\Services\MiniProgramService::payment();
            $res = $app->transfer->toBalance($data);
            if ($res['result_code']!='FAIL'){
                ShimmerLiveshopCommissionOrder::where('id', $commission_id)->update([
                    'has_withdraw' => 1,
                    'send_type' => 1,
                    'withdraw_time'=>time()
                ]);
                ShimmerLiveshopCashApply::where('id', $cashApplyId)->update([
                    'status' => 1,
                ]);
                $this->addAssetsLogs($user,$commission,$data['desc']);
            }else{
                ShimmerLiveshopCashApply::where('id', $cashApplyId)->update([
                    'status' => -2,
                    'fail_time' => time(),
                    'fail_msg'=>$res['err_code_des']
                ]);
            }
        } catch (\Exception $e) {
            ShimmerLiveshopCashApply::where('id', $cashApplyId)->update([
                'status' => -2,
                'fail_time' => time(),
                'fail_msg'=>'系统错误'
            ]);
        }

    }

    /**
     * @param $user
     * @param $commission
     * @param $remark
     * 插入资金明细
     */
    public function addAssetsLogs($user,$commission,$remark){
        ShimmerLiveshopStoreAsset::addRecord(ShimmerLiveshopStoreAsset::COMMISSION_WITHDRAW, 0, $user->id, $user->realname, $user->mobile, '', $commission, 2, 1, $remark);
    }


    /**
     * @return mixed
     * 获取默认分销等级
     */
    public function getAgentLevel(){
        $uniacid = $this->order->uniacid;
        $model = ShimmerLiveshopCommissionLevel::where(function($query) use($uniacid){
            $query->where('uniacid',$uniacid)->orWhere('uniacid',0);
        })->where('order_amount',0)->orderBy('id','asc')->first();
        if (!$model) return 0;
        return $model->id;
    }

    /**
     * @param $uid
     * 自动成为分销商
     */
    public function createAgent($uid){
        $agent = ShimmerLiveshopAgent::where('uid',$uid)->first();
        if ($agent){
            if ($agent->status==0){
                ShimmerLiveshopAgent::where('id', $agent->id)->update([
                    'status' => 1
                ]);
            }
        }else{
            $model = new ShimmerLiveshopAgent();
            $model->uniacid = $this->order->uniacid;
            $model->uid = $uid;
            $model->status = 1;
            $model->level = $this->getAgentLevel();
            $model->examine_at=time();
            $model->is_black=0;
            $model->child_count = 0;
            $model->save();
        }
    }

    public function changeOrderStatus(){
        ShimmerLiveshopStoreOrder::where('id', $this->order->id)->update([
            'deliver_time' => time(),
            'finish_time' => time(),
            'status' => 3,
        ]);
    }

    /**
     * 创建合伙人
     */
    public function createPartner(){
        $partnerApplyLog = ShimmerLiveshopPartnerUserLog::where('order_id',$this->order->id)->where('uid',$this->order->uid)->first();
        if (!$partnerApplyLog) return true;

        $model = ShimmerLiveshopPartnerUser::where('uid',$this->order->uid)->first();
        if (!$model){
            $model = new ShimmerLiveshopPartnerUser();
            $model->uniacid = $this->order->uniacid;
            $model->uid = $this->order->uid;
            $model->is_black = 0;
            $model->parent_uid = $this->getPartnerParent($this->order->uid);
            $model->diyform_form_id = $partnerApplyLog->diyform_form_id;
            $model->diyform_data_id = $partnerApplyLog->diyform_data_id;
            $model->status = PartnerService::getApplyStatus();
            $model->anchor_nums = 0;
            $model->partner_nums = 0;
            $model->is_partner = 1;
            $model->upgrade_type = 1;
            $model->is_anchor = 0;
            $model->goods_commission = 0;
            $model->anchor_commission = 0;
            $model->partner_commission = 0;
            $model->save();
        }

        $partnerApplyLog->status = 1;
        $partnerApplyLog->save();

        $partnerPay = PartnerService::savePartnerPayLogs($this->order->uid,$partnerApplyLog->payid,$this->order->id,$model->id);

        $model->level_pay_id = $partnerPay->id;
        $model->save();
    }

    /**
     * @param $uid
     * @return int
     * 获取上级ID
     */
    public function getPartnerParent($uid){
        $relation = \App\Models\ShimmerLiveshopUserRelation::where('uid',$uid)->value('parent_uid');
        return intval($relation);
    }

    /**
     * @param $data
     * @return mixed
     * 创建提现申请
     */
    public function createCashApply($data,$uid){
        $model =(new ShimmerLiveshopCashApply());
        $data = array(
            'uniacid'=>$this->order->uniacid,
            'order_no'=>$data['partner_trade_no'],
            'uid'=>$uid,
            'name'=>$data['desc'],
            'type'=>4,
            'cardnum'=>'',
            'weixin'=>'',
            'bank_address'=>'',
            'bankname'=>'',
            'alipay_code'=>'',
            'openid'=>$data['openid'],
            'remark'=>'',
            'status'=>0,
            'order_ids'=>[],
            'commission'=>$data['amount']/100,
        );
        $model->fill($data)->save();
        return $model->id;
    }

    /**
     * @return bool|\Illuminate\Database\Eloquent\Collection|\Illuminate\Database\Eloquent\Model|null|static|static[]
     * 获取订单信息
     */
    public function getOrder(){
        $this->order = $this->event->order;
        if (!$this->order) return true;
        if ($this->order->paid!=1 || $this->order->refund_status!=0) return true;
        if (($this->order->pay_price-$this->order->pay_postage)<=0) return true;
        if (!in_array($this->order->type,[7])){
            return true;
        }
        $this->createAgent($this->order->uid);
        $this->createPartner();
        $this->changeOrderStatus();
        $this->getCommissionRelation();
    }
}
