<?php
/**
 * 自购奖励
 */
namespace App\Listeners\Commission\SelfBuy;

use App\Events\CommissionOrder;
use App\Events\OrderPaySuccess;
use App\Models\ShimmerLiveshopCommissionOrder;
use App\Models\ShimmerLiveshopCommissionOrderGood;
use App\Models\ShimmerLiveshopGoodsSelfBuyCommission;
use App\Models\ShimmerLiveshopStoreOrderCartInfo;
use App\Models\ShimmerLiveshopUserBill;
use App\Service\AgentService;
use App\Services\ConfigService;
use App\Services\PluginService;
use Shimmer_liveshop_plugin_partner\Services\Partner\PartnerService;

class SelfBuyOrderListener
{
    public $event;
    public $config;
    public $order;
    public $grade;

    /**
     * Handle the event.
     *
     * @param  CommissionOrder  $event
     * @return void
     */
    public function handle(OrderPaySuccess $event)
    {
        $this->event = $event;
        $this->order = $this->event->order;

        if (!PluginService::isEnabled('partner')) return true;
        if (PartnerService::userIsPartner($this->order->uid)) return true;
        if (!$this->order) return true;
        if ($this->order->paid!=1 || $this->order->refund_status!=0) return true;
        if ($this->order->pay_price<=0) return true;
        if (!in_array($this->order->type,[1])){
            return true;
        }
        if (!AgentService::isAgent($this->order->uid)) return true;

        $this->config = ConfigService::getValueByType(11);
        if (!$this->config['open_save_money']) return true;
        $orderGoods = ShimmerLiveshopStoreOrderCartInfo::where('oid',$this->order->id)->get();
        foreach ($orderGoods as $goods){
            $commission = $this->calcuCommission($goods);
            if(!$commission){
                return true;
            }
            $this->addCommissionOrder($commission);
        }
    }

    public function addCommissionOrder($commission){
        $commissionOrder = ShimmerLiveshopCommissionOrder::where('order_id',$this->order->id)->where('uid',$this->order->uid)
            ->where('agent_uid',$this->order->uid)
            ->where('type',7)
            ->first();
        if($commissionOrder) return false;
        if(!$commission || !$commission['commission']) return false;
        $commissionModel = new ShimmerLiveshopCommissionOrder();
        $commissionModel->uniacid = $this->order->uniacid;
        $commissionModel->order_id = $this->order->id;
        $commissionModel->uid = $this->order->uid;
        $commissionModel->agent_uid = $this->order->uid;
        $commissionModel->price = $this->order->pay_price;
        $commissionModel->rate = $commission['commission_rate'];
        $commissionModel->level = 0;
        $commissionModel->commission = $commission['commission'];
        $commissionModel->status = 1;
        $commissionModel->type = 7;
        $commissionModel->has_withdraw = 0;
        $commissionModel->withdraw_time = 0;
        $commissionModel->send_type = 0;
        $commissionModel->title = ShimmerLiveshopCommissionOrder::$type[7];
        $commissionModel->save();

        $this->addCommissionOrderGoods($commission, $commissionModel->id);

        //记录用户账单
        ShimmerLiveshopUserBill::expend(
            '自购佣金',  $commissionModel->uid, 'now_money', 'brokerage', $commissionModel->price, $commissionModel->id, 0, '', 1, 1
        );
    }

    /**
     * @param $commission
     * @param $commissionOrderIds
     */
    public function addCommissionOrderGoods($commission, $commissionOrderIds)
    {
        $orderGoodsData[] = [
            'commission_order_id' => $commissionOrderIds,
            'uniacid' => $this->order->uniacid,
            'name' => $commission['orderGoods']->cart_info['productInfo']['store_name'],
            'thumb' => $commission['orderGoods']->cart_info['productInfo']['image'],
            'total' => $commission['orderGoods']->cart_info['cart_num'],
            'price' => $commission['orderGoods']->cart_info['truePrice'],
            'commission' => $commission['commission'],
            'has_commission' => $commission['orderGoods']['self_buy_commission']['has_commission'],
            'commission_rate' => $commission['orderGoods']['self_buy_commission']['commission_rate'],
            'commission_pay' => $commission['orderGoods']['self_buy_commission']['commission_pay'],
            'created_at'=>time(),
            'updated_at'=>time(),
        ];
        if (!empty($orderGoodsData)) {
            //添加分销订单商品数据
            ShimmerLiveshopCommissionOrderGood::insert($orderGoodsData);
        }
    }

    /**
     * @param $goods
     * @return array|bool
     * 计算自购佣金
     */
    public function calcuCommission($goods){
        if (!$goods) return false;
        $amount = $goods->cart_info['truePrice'];
        $goods_num = $goods->cart_info['cart_num'];
        $totalAmount = bcmul($amount,$goods_num,2);
        $singleRule = ShimmerLiveshopGoodsSelfBuyCommission::where('goods_id',$goods->goods_id)->first();
        if ($singleRule && $singleRule->is_open==1){
            $rule = $singleRule->rule;
            $commission = 0;
            $formula = "+商品独立自购佣金";//分佣计算方式
            if ($rule['first_level_rate']){
                $commissionRate = $rule['first_level_rate'];
                $rate_commission = bcmul($totalAmount,$rule['first_level_rate']/100,2);
            }
            if ($rule['first_level_amount']){
                $commissionPay = $rule['first_level_amount'];
                $amount_commission = bcmul($totalAmount,$rule['first_level_amount'],2);
            }
            $commission = $rate_commission> $amount_commission ? $rate_commission : $amount_commission;
            $goods['self_buy_commission'] = [
                'has_commission' => 1,
                'commission_rate' => $commissionRate ? :0,
                'commission_pay' => $commissionPay?:0,
                'commission' => $commission,
            ];
        }else{
            $commission = bcmul($totalAmount,$this->config['save_money_rate']/100,2);
            $commissionRate = $this->config['save_money_rate'];
            $formula = '商品实际支付金额';

            $goods['self_buy_commission'] = [
                'has_commission' => 0,
                'commission_rate' => $commissionRate,
                'commission_pay' => 0,
                'commission' => $commission,
            ];
        }
        return [
            'commission_amount' => $totalAmount,
            'formula' => $formula,
            'commission_rate' => $commissionRate,
            'commission' => $commission,
            'orderGoods' => $goods
        ];
    }
}
