<?php

namespace App\Listeners;

use App\Events\CommissionAgent;
use App\Models\ShimmerLiveshopAgent;
use App\Models\ShimmerLiveshopStoreOrder;
use App\Models\ShimmerLiveshopUser;
use App\Models\ShimmerLiveshopCommissionLevel;
use App\Services\ConfigService;

class CommissionAgentListener
{
    public $config;
    public $event;
    /**
     * Handle the event.
     *
     * @param  CommissionAgent  $event
     * @return void
     */
    public function handle(CommissionAgent $event)
    {
        $config = ConfigService::getValueByType(11);
        $this->config = $config;
        $this->event = $event;
        $this->becomeAgent();
    }

    /**
     * @return int
     * 是否审核
     */
    public function getStatus(){
        $user =  ShimmerLiveshopUser::find($this->event->uid);
        if ($this->config['become_reg']==1 && (!$user['mobile'] || !$user['realname'])) return 0;
        return $this->config['become_check']==1 ? 0 : 1;
    }

    /**
     * @return bool
     * 判断资料是否填写
     */
    public function judgeInfo(){
        if ($this->config['become_reg']==2) return true;
        return true;
    }

    /**
     * @return array
     * 获取分销满足订单状态
     */
    public function getOrderStatus($model){
        if ($this->config['become']==3){
            if ($this->config['become3_order']==0){
                return $model->where('paid',1);
            }elseif($this->config['become3_order']==1){
                return $model->where('paid',1)->where('refund_status',0);
            }
        }elseif($this->config['become']==4){
            if ($this->config['become4_order']==0){
                return $model->where('paid',1);
            }elseif($this->config['become4_order']==1){
                return $model->where('paid',1)->where('refund_status',0);
            }
        }
        return [];
    }

    /**
     * @return mixed
     * 获取默认分销等级
     */
    public function getAgentLevel(){
        $uniacid = $this->event->uniacid;
        $model = ShimmerLiveshopCommissionLevel::where('uniacid',$uniacid)->where('order_amount',0)->orderBy('id','asc')->first();
        if (!$model) return;
        return $model->id;
    }

    public function becomeAgent(){

        if (!$this->judgeInfo()) return;

        switch ($this->config['become']){
            case 1:
                $this->store();
                break;
            case 2:
                if ($this->event->type=='apply'){
                    $this->store();
                }
                break;
            case 3:
                $this->countOrders();
                break;
            case 4:
                $this->consume();
                break;
            case 5:
                $this->buyGoods();
                break;
        }
    }

    /**
     * 成为分销商
     */
    public function store(){
        $model = ShimmerLiveshopAgent::where('uid',$this->event->uid)->first();
        if ($model){
            if ($model->status==0){
                $model->status = $this->getStatus();
                if ($this->getStatus()==1){
                    $model->examine_at = time();
                }
                $model->save();
            }
            return true;
        }else{
            $level = $this->getAgentLevel();
            if (!$level) return;
            $data = array(
                'uniacid'=>$this->event->uniacid,
                'uid'=>$this->event->uid,
                'status'=>$this->getStatus(),
                'level'=>$this->getAgentLevel(),
                'is_black'=>0,
                'child_count'=>0,
            );
            (new ShimmerLiveshopAgent())->fill($data)->save();
        }
    }

    /**
     * 消费次数
     */
    public function countOrders(){
        $model = ShimmerLiveshopStoreOrder::where('uid',$this->event->uid);
        $nums = $this->getOrderStatus($model)->count();
        if ($nums>=$this->config['become_ordercount']){
            $this->store();
        }
    }

    /**
     * 消费金额
     */
    public function consume(){
        $model = ShimmerLiveshopStoreOrder::where('uid',$this->event->uid);
        $nums = $this->getOrderStatus($model)->sum('pay_price');
        if ($nums>=$this->config['become_moneycount']){
            $this->store();
        }
    }

    /**
     * 购买指定商品
     * todo
     */
    public function buyGoods(){
        $order = ShimmerLiveshopStoreOrder::find($this->event->order_id);
        if ($order->uid != $this->event->uid) return true;
        if (!in_array($order->status,$this->getOrderStatus())) return true;
        //$this->store();
    }
}
