<?php

namespace App\Listeners\Gift;

use App\Events\CommissionOrder;
use App\Events\OrderPaySuccess;
use App\Models\ShimmerLiveshopAnchorApply;
use App\Models\ShimmerLiveshopCashApply;
use App\Models\ShimmerLiveshopCommissionLevel;
use App\Models\ShimmerLiveshopCommissionOrder;
use App\Models\ShimmerLiveshopLive;
use App\Models\ShimmerLiveshopStoreAsset;
use App\Models\ShimmerLiveshopStoreOrder;
use App\Models\ShimmerLiveshopUser;
use App\Modules\Liveapi\Models\ShimmerLiveshopAgent;
use App\Modules\Liveapi\Models\ShimmerLiveshopUserRelation;
use App\Modules\Liveapi\Services\Live\AnchorService;
use App\Modules\Liveapi\Services\MiniProgramService;
use App\Services\Commission\CashApplyService;
use App\Services\ConfigService;

class AnchorGiftSettledOrderListener
{
    public $event;
    public $config;
    public $order;
    public $grade;

    /**
     * Handle the event.
     *
     * @param  CommissionOrder  $event
     * @return void
     */
    public function handle(OrderPaySuccess $event)
    {
        $this->event = $event;
        $this->getAnchorGiftSettleConfig();
    }

    /**
     * 礼物打赏主播分红配置
     */
    public function getAnchorGiftSettleConfig(){
        $this->config = ConfigService::getValueByType(17);
        $this->getOrder();
    }

    /**
     * 修改订单状态
     */
    public function changeOrderStatus(){
        ShimmerLiveshopStoreOrder::where('id', $this->order->id)->update([
            'deliver_time' => time(),
            'finish_time' => time(),
            'status' => 3,
        ]);
    }

    /**
     * @return bool|\Illuminate\Database\Eloquent\Collection|\Illuminate\Database\Eloquent\Model|null|static|static[]
     * 获取订单信息
     */
    public function getOrder(){
        $this->order = $this->event->order;
        if (!$this->order) return true;
        if ($this->order->paid!=1 || $this->order->refund_status!=0) return true;
        if ($this->order->pay_price<=0) return true;
        if ($this->order->relation_type!=1) return true;
        if (!in_array($this->order->type,[3])){
            return true;
        }
        $live = ShimmerLiveshopLive::find($this->order->relation_id);
        if (!$live){
            return true;
        }
        if (!$live->uid) return true;
        $anchor = ShimmerLiveshopAnchorApply::where('uid',$live->uid)->where('status',1)->where('is_black',0)->first();
        if (!$anchor) return true;

        $anchorUserInfo = ShimmerLiveshopUser::where('id',$live->uid)->first();
        if (!$anchorUserInfo) return true;
        $this->changeOrderStatus();
        $this->getCommission($anchorUserInfo);
    }

    /**
     * 分销佣金
     */
    public function getCommission($anchorUserInfo){
        if (!$this->config || $this->config['open_deduct']==0){
            return true;
        }
        $platformCommission = bcmul($this->order->pay_price,$this->config['deduct_ratio']/100,2);
        $commission =bcsub($this->order->pay_price,$platformCommission,2);
        if ($commission<0) return true;
        $this->storeCommissionOrder($anchorUserInfo,$commission);
    }

    /**
     * @param $type
     * @param $uid
     * @param $commissionData
     * 保存分佣订单
     */
    public function storeCommissionOrder($anchorUserInfo,$commission){
        $commissionOrder = ShimmerLiveshopCommissionOrder::where('order_id',$this->order->id)->where('uid',$this->order->uid)
            ->where('agent_uid',$anchorUserInfo->id)
            ->first();
        if ($commissionOrder) return false;
        $model = new ShimmerLiveshopCommissionOrder();
        $data = array(
            'uniacid'=>$this->order->uniacid,
            'order_id'=>$this->order->id,
            'uid'=>$this->order->uid,
            'agent_uid'=>$anchorUserInfo->id,
            'price'=>$this->order->pay_price,
            'rate'=>$this->config['deduct_ratio'],
            'level'=>1,
            'commission'=>$commission,
            'status'=>2,
            'title'=>ShimmerLiveshopCommissionOrder::$type[$this->order->type],
            'has_withdraw'=>0,
            'type'=>3,
            'send_type'=>0,
            'withdraw_time'=>0
        );
        $model->fill($data)->save();
    }
}
