<?php

namespace App\Listeners;

use App\Events\OrderPaySuccess;
use App\Events\OrderRefund;
use App\Models\ShimmerLiveshopStoreAsset;
use App\Models\ShimmerLiveshopStoreOrderCartInfo;
use App\Models\ShimmerLiveshopStoreOrderGroup;
use App\Modules\Liveapi\Models\ShimmerLiveshopOrder;
use App\Modules\Liveapi\Services\GoodsService;

/**
 * 订单退款成功更新
 * Created by PhpStorm.
 * User: lirui
 * Date: 2019/12/03
 * Time: 下午2:37
 */
class OrderRefundUpdateListener
{
    public $order;
    public $refundPrice;
    public $refundType;
    public $event;

    /**
     * Handle the event.
     *
     * @param  OrderPaySuccess $event
     * @return void
     */
    public function handle(OrderRefund $event)
    {
        $this->order = $event->order;
        $this->refundPrice = $event->refundPrice;
        $this->refundType = $event->refundType;
        $this->refundNo = $event->refundNo;

        $this->updateOrder();
        $this->addAssetLog();
        $this->orderLog();
        $this->updateGoods();
    }

    /**
     * 更新商品相关
     */
    public function updateGoods()
    {
        $orderCarts = ShimmerLiveshopStoreOrderCartInfo::where('oid', $this->order->id)->get();
        foreach ($orderCarts as $cart) {
            //增库存减销量
            GoodsService::incGoodsStock($cart['cart_info']['cart_num'], $cart['cart_info']['productInfo']['id'], isset($cart['cart_info']['productInfo']['attrInfo']) ? $cart['cart_info']['productInfo']['attrInfo']['unique'] : '');
        }
    }

    /**
     * 更新订单数据
     */
    public function updateOrder()
    {
        $totalRefundPrice = bcadd($this->order['refund_price'], $this->refundPrice, 2);
        $orderGroupId = $this->order->order_group_id;//分组订单

        if ($orderGroupId) {
            $orderGroup = ShimmerLiveshopStoreOrderGroup::find($orderGroupId);
            if (!$orderGroup) {
                exception('分组订单不存在1255');
            }
        }

        $isOver = bccomp((float)$this->order['pay_price'], (float)$totalRefundPrice, 2);

        if ($isOver < 0) {
            exception('退款金额大于支付金额，请修改退款金额！');
        }

        $this->order->refund_price = $totalRefundPrice;//总退款金额

        if ($this->refundType == 1) {
            $this->order->refund_status = 2;
        } else if ($this->refundType == 2) {
            $this->order->refund_status = 0;
        }

        //更新订单组状态
        if ($orderGroup) {
            $totalRefundPrice = bcadd($orderGroup['refund_price'], $this->refundPrice, 2);
            $orderGroup->refund_price = $totalRefundPrice;//总退款金额
            $orderGroup->save();
        }

        $this->order->save();
    }

    /**
     * 记录订单退款日志
     */
    public function orderLog()
    {
        \App\Services\OrderService::log($this->order->id, 'refund_price', '退款给用户' . $this->refundPrice . '元，单号:' . $this->refundNo);
    }

    /**
     * 插入资产流水表
     */
    public function addAssetLog()
    {
        //插入资产流水表
        return ShimmerLiveshopStoreAsset::addRecord(ShimmerLiveshopStoreAsset::ORDER_REFUND, $this->order['mer_id'], $this->order['uid'], $this->order['real_name'], $this->order['user_phone'], $this->order['id'], $this->order['pay_price'], 2, $this->order->pay_type, $this->order->typeText);
    }

}
