<?php

namespace App\Models;


use Illuminate\Database\Eloquent\SoftDeletes;

class ShimmerLiveshopAnchorApply extends BaseUniacidModel
{
    use SoftDeletes;
    protected $guarded = [];
    public $dateFormat = "U";
    protected $table = 'shimmer_liveshop_anchor_apply';

    public static $apply_type=[
        0=>'免费',
        1=>'付费'
    ];

    public function user()
    {
        return $this->hasOne(ShimmerLiveshopUser::class, 'id', 'uid');
    }

    public function userLevel(){
        return $this->hasOne(ShimmerLiveshopAnchorApplyPay::class,'id','level_pay_id');
    }

    public function userPayLevel(){
        return $this->hasMany(ShimmerLiveshopAnchorApplyPay::class,'uid','uid');
    }

    public function commissionOrder(){
        return $this->hasMany(ShimmerLiveshopCommissionOrder::class,'agent_uid','uid');
    }

    /**
     * @param $query
     * @param $data
     * 搜索
     */
    public function scopeSearchData($query,$data){
        if ($data['query']['uid']){
            $query->where('uid', 'like', "%{$data['query']['uid']}%");
        }
    }

    /**
     * @return mixed
     * 带货总金额
     */
    public function getTotalAmountAttribute(){
        return $this->commissionOrder->sum('price');
    }

    /**
     * @return mixed
     * 带货总佣金
     */
    public function getTotalCommissionAttribute(){
        return $this->commissionOrder->sum('commission');
    }

    /**
     * @return mixed
     * 当前佣金
     */
    public function getNowCommissionAttribute(){
        return $this->commissionOrder->where('status',2)->where('has_withdraw',0)->sum('commission');
    }

    public function live(){
        return $this->hasOne(ShimmerLiveshopLive::class,'uid','uid');
    }

    public function getIncomeGiftAttribute(){
        if ($this->live){
            return $this->live->order->where('type',3)->where('refund_status',0)->where('paid',1)->sum('pay_price');
        }else{
            return 0;
        }
    }

    /**
     * @return int
     * 名下主播数量
     */
    public function getChildAnchorNumsAttribute(){
        $childsUid = ShimmerLiveshopUserRelation::where('parent_uid',$this->uid)->pluck('uid');
        if ($childsUid){
            return ShimmerLiveshopAnchorApply::whereIn('uid',$childsUid)->count();
        }
        return 0;
    }

    /**
     * @return mixed
     * 邀请主播佣金
     */
    public function getIniAnchorCommissionAttribute(){
        return $this->commissionOrder->where('type',2)->sum('commission');
    }

    /**
     * @return int
     * 剩余主播数量
     */
    public function getSurplusAnchorNumsAttribute(){
        $surplus_anchor_nums = 0;
        if ($this->userLevel && $this->userLevel->give_anchor_num>0){
            $hasGives = ShimmerLiveshopAnchorRelation::where('parent_uid',$this->uid)->where('type',1)->count();
            $surplus_anchor_nums = $this->userLevel->give_anchor_num - $hasGives;
            return $surplus_anchor_nums<0 ? 0 :$surplus_anchor_nums;
        }
        return $surplus_anchor_nums;
    }
}