<?php
/**
 * 用户model
 * Created by PhpStorm.
 * User: lirui
 * Date: 2019/7/19
 * Time: 上午11:49
 */

namespace App\Models;


use Illuminate\Database\Eloquent\SoftDeletes;
use Shimmer_liveshop_plugin_merch\Models\ShimmerLiveshopMerchStore;

class ShimmerLiveshopCashApply extends BaseUniacidModel
{
    use SoftDeletes;
    protected $guarded = [];
    public $dateFormat = "U";

    public $casts = ['order_ids' => 'json'];

    public static $category = [
        1 => '佣金提现',
        2 => '余额提现',
        3 => '商户提现',
        4 => '主播付费提成提现',
        5 => '合伙人直推主播奖励提现',
        6 => '直推合伙人奖励提现',
    ];
    public static $status = [
        0 => '待打款',
        1 => '打款成功',
        -1 => '审核拒绝',
        -2 => '打款失败',
    ];
    /**
     * 提现方式
     * @var array
     */
    public static $type = [
        'weixin' => '微信',
        'alipay' => '支付宝',
        'bank' => '银行卡',
        'balance' => '余额',
        'system-wxapp' => '微信-自动打款',
    ];

    /**
     * 支付方式
     * @var array
     */
    public static $payType = [
        1 => '线下打款',
        2 => '线上打款',
    ];

    public function store()
    {
        return $this->belongsTo(ShimmerLiveshopMerchStore::class, 'role_id', 'id');
    }

    public function user()
    {
        return $this->hasOne(ShimmerLiveshopUser::class, 'id', 'uid');
    }

    public function scopeSearchKey($query, $data)
    {
        if ($data['uid']) {
            $query->where('uid', $data['uid']);
        }
    }

    public function scopeStatus($query, $status)
    {
        $query->where('status', $status);
    }

    /**
     * @param $query
     * @param $data
     * 搜索
     */
    public function scopeSearchData($query,$data){
        if ($data['query']['uid']){
            $query->where('uid', $data['query']['uid']);
        }
        if ($data['query']['type']){
            $query->where('type',$data['query']['type']);
        }
        if ($data['query']['pay_type']){
            $query->where('pay_type',  $data['query']['pay_type']);
        }
        if ($data['query']['status']){
            $query->where('status',  $data['query']['status']);
        }
        if ($data['query']['category']){
            $query->where('category',  $data['query']['category']);
        }
        if ($data['query']['start_time']){
            $query->where('created_at',  '>=',strtotime($data['query']['start_time']));
        }
        if ($data['query']['end_time']){
            $query->where('created_at',  '<=',strtotime($data['query']['end_time']));
        }
    }

    /**
     * 提现分类
     * @param $query
     * @param $category
     */
    public function scopeCategory($query, $category)
    {
        if ($category) {
            $query->where('category', $category);
        }
    }

    public function getStatusTextAttribute()
    {
        if ($this->status == 0) {
            return '未审核';
        }
        if ($this->status == 1) {
            return '提现成功';
        }
        if ($this->status == -1) {
            return $this->fail_msg . ',拒绝时间：' . date('Y-m-d H:i:s', $this->fail_time);
        }
    }

    /**
     * 打款方式
     * @return string
     */
    public function getPayTypeTextAttribute()
    {
        return self::$payType[$this->pay_type];
    }

    /**
     * 获取角色
     */
    public function getRoleAttribute()
    {
        return GlobalRoleService::getRole($this->role_type, $this->role_id);
    }
}