<?php

namespace App\Models;

use Illuminate\Database\Eloquent\SoftDeletes;

class ShimmerLiveshopComment extends BaseLiveModel
{
    use SoftDeletes;
    protected $guarded = [];

    public function user()
    {
        return $this->belongsTo(ShimmerLiveshopUser::class, 'user_id', 'id');
    }

    public static $recommendStateText = [
        1 => '推荐',
        0 => '未推荐',
    ];

    public static $checkedText = [
        0 => '待审核',
        1 => '审核通过',
        2 => '审核不通过',
    ];

    /**
     * 通过关键字搜索
     * @param $query
     * @param $content
     */
    public function scopeContent($query, $content)
    {
        if (!$content) {
            return $query;
        }

        return $query->where('content', 'like', "%{$content}%");
    }

    /**
     * 通过用户id
     * @param $query
     * @param $userId
     */
    public function scopeUserId($query, $userId)
    {
        if (!$userId) {
            return $query;
        }

        return $query->where('user_id', $userId);
    }

    /**
     * 通过昵称搜索
     * @param $query
     * @param $nickname
     */
    public function scopeNickname($query, $nickname)
    {
        if (!$nickname) {
            return $query;
        }

        return $query->where('nickname', 'like', "%{$nickname}%");
    }

    /**
     * @param $query
     * @param $recommendState
     */
    public function scopeRecommendState($query, $recommendState)
    {
        if ($recommendState === '') {
            return $query;
        }

        return $query->where('recommend_state', $recommendState);
    }

    /**
     * 评论状态
     * @param $query
     * @param $checked
     */
    public function scopeChecked($query, $checked)
    {
        if ($checked === '') {
            return $query;
        }
        return $query->where('checked', $checked);
    }
}
