<?php

namespace App\Models;

use \App\Modules\Live\Models\BaseModel;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Support\Facades\DB;

class ShimmerLiveshopGoodsCategory extends BaseUniacidModel
{
    use SoftDeletes;

    /**
     * 通过名称搜索
     * @param $query
     * @param $nickname
     */
    public function scopeName($query, $name)
    {
        if (!$name) {
            return $query;
        }

        return $query->where('name', 'like', "%{$name}%");
    }

    /**
     * 格式化图片
     * @param $pic
     */
    public function getPicAttribute($pic)
    {
        if ($pic) {
            $pic = wgTomedia($pic);
        }
        return $pic;
    }

    /**
     * 格式化图片
     * @param $pic
     */
    public function getHomePicAttribute($pic)
    {
        if ($pic) {
            $pic = wgTomedia($pic);
        }
        return $pic;
    }


    /**
     * 获取父类
     */
    public function getParent()
    {
        if ($this->parent_id <= 0) {
            return false;
        } else {
            return self::find($this->parent_id);
        }
    }

    public function getChilds()
    {
        if ($this->parent_id != 0) {
            return collect();
        } else {
            return self::where('parent_id', $this->id)->get();
        }
    }

    /**
     * 获取所有商品分类
     */
    public static function getAllCategorys()
    {
        return self::where('isshow', 1)->get();
    }

    /**
     * 获取所有父级菜单
     */
    public static function getParentCategorys()
    {
        return self::where('parent_id', 0)
            ->where('isshow', 1)
            ->get();
    }

    /**
     * 修改商品分类状态
     * @param $id
     * @param $isShow
     */
    public static function changeIsShow($id, $isShow)
    {
        if (!$id || !$isShow) {
            return false;
        }

        return self::where('id', $id)->update(['isshow' => $isShow]);
    }

    /**
     * 通过分类id获取分类
     * @param $ids 1,2,3
     */
    public static function getCategorysByIds($ids)
    {

        if (!$ids) {
            return collect();
        } else {
            if (!is_array($ids)) {
                $categoryIds = explode(',', $ids);
            } else {
                $categoryIds = $ids;
                $ids = implode(',', $ids);
            }

            return self::whereIn('id', $categoryIds)->orderByRaw(DB::raw("FIELD(id,{$ids})"))->get();
        }
    }

    /**
     * 获取子分类
     * @param $pid
     */
    public static function getChildsByPid($pid)
    {
        if (!$pid) {
            return collect();
        }

        return self::where('parent_id', $pid)
            ->where('isshow', 1)
            ->get();
    }

}
