<?php

namespace App\Modules\Live\Http\Controllers;

/**
 * 资产流水列表
 * Class AssetsController
 * @package App\Modules\Live\Http\Controllers
 */

use App\Events\OrderPaySuccess;
use App\Models\ShimmerLiveshopStoreAsset;
use App\Models\ShimmerLiveshopStoreOrder;
use App\Services\PluginService;

class AssetsController extends Controller
{

    public function index()
    {
        //查询数据
        $queryData = [];
        $queryData['category'] = ShimmerLiveshopStoreAsset::$categorys;
        $queryData['in_come'] = ShimmerLiveshopStoreAsset::$inComes;

        //获取分页数据
        $query = $this->query;
        $startTime = $query['start_time'];
        $endTime = $query['end_time'];

        unset($query['start_time'], $query['end_time']);

        $builder = ShimmerLiveshopStoreAsset::search($query)->with('user');

        if ($startTime) {
            $builder->where('created_at', '>=', $startTime);
        }
        if ($endTime) {
            $builder->where('created_at', '<=', $endTime);
        }

        //排除插件资产
        if (!PluginService::isEnabled('merch')) {
            $merchCategorys = [6, 7, 8, 9, 10];
            foreach ($merchCategorys as $merchCategory) {
                unset($queryData['category'][$merchCategory]);
            }
            $builder->whereNotIn('category', $merchCategorys);
        }


        $totalInBuilder = clone  $builder;
        $totalOutBuilder = clone  $builder;

        $stats['total_in_money'] = (float)$totalInBuilder->where('is_income', 1)->sum('money');

        $stats['total_out_money'] = (float)$totalOutBuilder->where('is_income', 2)->sum('money');

        $builder->orderBy('id', 'desc');
        $pager = $builder->paginate($this->pageNum);

        return view('live::assets.index', [
            'pager' => $pager,
            'queryData' => $queryData,
            'stats' => $stats,
        ]);
    }
}