<?php

namespace App\Modules\Live\Http\Controllers\Commission;
use App\Events\SendCommission;
use App\Models\ShimmerLiveshopCommissionOrder;
use App\Modules\Live\Http\Controllers\Controller;
use App\Modules\Live\Models\ShimmerLiveshopCashApply;
use App\Services\Commission\CashApplyService;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class CashApplyController extends Controller
{
    public function index(Request $request){
        $lists = ShimmerLiveshopCashApply::searchData($request->input())->whereIn('category',[1,4])
            ->paginate($this->pageNum);
        $queryData['type'] = ShimmerLiveshopCashApply::$type;
        $queryData['status'] = ShimmerLiveshopCashApply::$status;
        $queryData['pay_type'] = ShimmerLiveshopCashApply::$payType;
        return view('live::commission.cash-apply.index',compact('lists','queryData'));
    }

    public function edit(Request $request){
        $model = (new ShimmerLiveshopCashApply())->loadModel($request->route('id'));
        return view('live::commission.cash-apply.edit',compact('model'));
    }

    public function update(Request $request){
        $model = (new ShimmerLiveshopCashApply())->loadModel($request->route('id'));
        $model->name = $request->input('name');
        $model->remark = $request->input('remark');
        $model->commission = $request->input('commission');
        if ($model->type=='bank'){
            $model->bankname = $request->input('bankname');
            $model->cardnum = $request->input('cardnum');
            $model->bank_address = $request->input('bank_address');
        }

        if ($model->type=='alipay'){
            $model->alipay_code = $request->input('alipay_code');
        }

        if ($model->type=='weixin'){
            $model->weixin = $request->input('weixin');
        }

        $model->save();
        return $this->result();
    }

    /**
     * @param Request $request
     * @return array|string
     * 审核通过
     */
    public function auditPass(Request $request){
        DB::beginTransaction();
        $model = (new ShimmerLiveshopCashApply())->loadModel($request->route('id'));
        if ($model->status==1){
            return $this->result(1,'已经审核通过了');
        }
        $model->status=1;
        $model->save();

        ShimmerLiveshopCommissionOrder::whereIn('id', $model->order_ids)->update([
            'has_withdraw' => 1,
            'send_type'=>2,
        ]);

        event(new SendCommission($model->id));
        DB::commit();
        return $this->result();
    }

    /**
     * @param Request $request
     * @return array|string
     * 拒绝
     */
    public function cancel(Request $request){
        $model = (new ShimmerLiveshopCashApply())->loadModel($request->route('id'));
        return view('live::commission.cash-apply.cancel',compact('model'));
    }

    /**
     * @param Request $request
     * @return array|string
     * @throws \App\Common\Exceptions\AjaxException
     * 拒绝佣金申请理由
     */
    public function cancelReson(Request $request){
        $model = (new ShimmerLiveshopCashApply())->loadModel($request->route('id'));
        if ($model->status!=0){
            return $this->result(1,'该订单已经操作过了');
        }
        DB::beginTransaction();
        $model->fail_msg = $request->input('fail_msg');
        $model->fail_time = time();
        $model->status = -1;
        $model->save();
        ShimmerLiveshopCommissionOrder::whereIn('id', $model->order_ids)->update([
            'has_withdraw' => 0,
            'withdraw_time'=>0
        ]);
        DB::commit();
        return $this->result();
    }

    public function withdrawSend(Request $request){
        $id = $request->input('id');
        $status = (int)$request->input('status');
        $payType = (int)$request->input('pay_type');
        $failMsg = trim($request->input('fail_msg'));
        $type = trim($request->input('type'));
        $model = ShimmerLiveshopCashApply::with('user')->find($id);
        $model->status = $status;
        try{
            if ($status == -1) {
                $model->remark = $failMsg;
                $model->save();
                ShimmerLiveshopCommissionOrder::whereIn('id', $model->order_ids)->update([
                    'has_withdraw' => 0,
                    'withdraw_time'=>0
                ]);
                return $this->success();
            }
            if ($status == 1) {
                $model->pay_type = $payType;
                $model->status = 1;
                if ($payType == 1) {
                    $model->save();
                    return $this->success();
                } else {
                    if ($type=='balance'){
                        $model->type = 'balance';
                        CashApplyService::commissionToBalance($model);
                        $model->save();
                        return $this->success();
                    }else{
                        try{
                            $app = \App\Modules\Liveapi\Services\MiniProgramService::payment();
                            $data = [
                                'partner_trade_no' => $model->order_no,
                                'openid' => $model->user->wxapp_openid,
                                'check_name' => 'FORCE_CHECK',
                                're_user_name' => $model->user->realname,
                                'amount' => $model->commission * 100,
                                'desc' => $model->name,
                            ];
                            $res = $app->transfer->toBalance($data);
                            if ($res['result_code'] != 'FAIL') {
                                $model->save();
                                return $this->success();
                            } else {
                                $model->status = -2;
                                $model->fail_msg = $res['err_code_des'];
                                $model->fail_time = time();
                                $model->save();
                                return $this->error(-1, '打款失败:' . $res['err_code_des']);
                            }
                        }catch (\Exception $e) {
                            return $this->error(-1, '打款失败:' . $e->getMessage());
                        }
                    }
                }
            }
        }catch (\Exception $e) {
            return $this->error(-1, '操作失败:' . $e->getMessage());
        }

        return $this->error(-1, '系统错误');
    }
}