<?php

namespace App\Modules\Live\Http\Controllers;

use App\Models\BaseUniacidModel;
use App\Services\AppService;
use Illuminate\Http\Request;
use App\Modules\Live\Models\BaseModel;
use Illuminate\Support\Facades\View;
use App\Http\Controllers\Controller as BaseController;

class Controller extends BaseController
{
    protected $uniacid;//公众号id
    protected $liveId;//直播间id
    protected $query;//查询条件
    protected $pageNum;//分页条数

    public function __construct(Request $request)
    {

        $this->setView();

        $this->setW();

        $this->liveId = (int)$request->route('live_id');
        $this->uniacid = (int)$request->route('uniacid');

        View::share('pageTitle', '');

        $this->setQuery();
    }

    private function setW()
    {
        global $_W;
        $_W['uniacid'] = (int)\Illuminate\Support\Facades\Request::route("uniacid");
        $_W['acid'] = (int)\Illuminate\Support\Facades\Request::route("uniacid");
        $_W['siteroot'] = str_replace(config('common.pre_path'), '', $_W['siteroot']);
        View::share('_W', $_W);
    }

    private function setView()
    {
        $request = request();

        $defaultPageNum = config('common.page_nums')[0];
        $this->pageNum = (int)$request->input('page_num', $defaultPageNum);
        $this->pageNum = $this->pageNum <= 0 ? $defaultPageNum : $this->pageNum;

        View::share('pageNums', config('common.page_nums'));
        View::share('currentPageNum', $this->pageNum);

        View::share('liveId', $request->route('live_id'));
        View::share('uniacid', $request->route('uniacid'));
    }

    /**
     * 设置后台列表查询条件
     */
    protected function setQuery()
    {
        //搜索条件
        $this->query = (array)request()->get('query');

        foreach ($this->query as $key => $value) {
            if (is_numeric($value) && !$value) {
                $this->query[$key] = intval($value);
            }
        }
        View::share('query', $this->query);
    }

    /**
     * 设置页面标题
     */
    protected function setPageTitle($title)
    {
        View::share('pageTitle', $title);
    }

    /**
     * $errno 0 成功 其他失败
     * @param $errno
     * @param string $message
     * @param string $data
     * @return array|string
     */
    public function result($errno = 0, $message = 'ok', $data = [])
    {
        $request = app()->make('request');
        if (is_array($data) && isset($data['redirect'])) {
            $redirect = $data['redirect'];
        } else {
            $redirect = $request->server('HTTP_REFERER');
        }
        if ($request->ajax()) {
            return [
                'errno' => $errno,
                'data' => $data,
                'message' => $message,
                'redirect' => $redirect,
            ];
        }
        return $data;
    }

    /**
     * 成功返回
     * @param string $message
     * @param string $data
     * @return array|string
     */
    public function success($data = '', $message = '操作成功')
    {
        $request = app()->make('request');
        if (is_array($data) && isset($data['redirect'])) {
            $redirect = $data['redirect'];
        } else {
            $redirect = $request->server('HTTP_REFERER');
        }
        if ($request->ajax()) {
            response()->json([
                'errno' => 0,
                'data' => $data,
                'message' => $message,
                'redirect' => $redirect,
            ], 200, ['charset' => 'utf-8'])->send();
        }
        return $data;
    }

    /**
     * 失败返回
     * $errno
     * @param $errno
     * @param string $message
     * @param string $data
     * @return array|string
     */
    public function error($errno = -1, $message = '系统错误', $data = '')
    {
        $request = app()->make('request');
        if ($request->ajax()) {
            response()->json([
                'errno' => $errno,
                'data' => $data,
                'message' => $message
            ], 200, ['charset' => 'utf-8'])->send();
        }
        return $data;
    }

    protected function validateScenario($request, $instance, $scenario = 'default')
    {
        $rules = $instance->getRules($request, $scenario);

        $this->validate($request, $rules, $instance->getMessages($scenario));
        $validate = $instance->getScenarios($scenario);
        $data = $request->only($validate);
        if ($instance instanceof BaseUniacidModel
            || $instance instanceof BaseModel
        ) {
            $data['uniacid'] = $request->route('uniacid');
        }

        $data = $instance->validateFilter($data, $scenario);
        return $data;
    }

    /**
     * 验证，只验证配置字段，其他字段放行
     * @param $request
     * @param $instance
     * @param string $scenario
     * @return mixed
     */
    protected function validateScenarioPass($request, $instance, $scenario = 'default')
    {
        $rules = $instance->getRules($request, $scenario);

        $this->validate($request, $rules, $instance->getMessages($scenario));
        $validate = $instance->getScenarios($scenario);
        $data = $request->all($validate);

        if ($instance instanceof BaseModel
            || $instance instanceof BaseUniacidModel
        ) {
            $data['uniacid'] = $request->route('uniacid');
        }

        unset($data['_token'], $data['s'], $data['_method']);
        $data = $instance->validateFilter($data, $scenario);
        return $data;
    }
}
