<?php
/**
 * 商品管理
 * Created by PhpStorm.
 * User: lirui
 * Date: 2019/8/9
 * Time: 上午9:48
 */

namespace App\Modules\Live\Http\Controllers\Live;

use App\Models\ShimmerLiveshopAnchorApply;
use App\Models\ShimmerLiveshopAnchorApplyPay;
use App\Models\ShimmerLiveshopLiveViewer;
use App\Modules\Live\Http\Controllers\Controller;
use App\Modules\Live\Models\ShimmerLiveshopConfig;
use App\Modules\Liveapi\Services\Live\AnchorService;
use App\Services\Anchor\ConfigService;
use App\Services\DiyFormService;
use App\Services\ImageService;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Maatwebsite\Excel\Facades\Excel;

class AnchorController extends Controller
{
    public function getLists(Request $request){
        $model = $this->loadModel();
        $usersModel = ShimmerLiveshopAnchorApply::with(['user','userLevel','commissionOrder','live'=>function($q){
            $q->with('order');
        }])->where(function($query){
            if(Request()->input('uid')){
                $query->where('uid',Request()->input('uid'));
            }
            if(Request()->input('nickname')){
                $query->whereHas('user',function($q){
                    $q->where('nickname', 'like', '%'.Request()->input('nickname').'%');
                });
            }
        })->orderBy('id','desc');

        $submit = $request->input('submit');
        if ($submit == '导出') {
            $list = $usersModel->get();
            $this->export($list);
            return;
        }

        $users = $usersModel->paginate($this->pageNum);
        $formLists = DiyFormService::getAllForms();
        //$payConfig = ConfigService::getPayConfig();
        return view('live::live.anchor.index',compact('users','model','formLists','payConfig'));
    }

    public function store(Request $request){
        $model = $this->loadModel();
        $data = $this->validateScenario($request,ShimmerLiveshopConfig::instance(),'store');
        $data['config']['local_invitation_image'] = ImageService::ossImgToLocal($data['config']['invitation_image']);

        $model->fill($data)->save();
        if ($data['config']['apply_limit_type']==1){
            ConfigService::save($request->input('extends'));
        }
        return $this->result();
    }

    public function loadModel(){
        $model = ShimmerLiveshopConfig::type(14)->first();
        if (!$model){
            $model = new ShimmerLiveshopConfig();
        }
        return $model;
    }

    /**
     * @param Request $request
     * @return array|string
     * 修改拉黑状态
     */
    public function changeBlack(Request $request){
        $id = $request->route('id');
        $users = ShimmerLiveshopAnchorApply::find($id);
        $users->is_black = !$users->is_black;
        $users->save();
        AnchorService::changeBlack($users->uid,$users->is_black);
        return $this->result();
    }

    public function delPayConfig(Request $request){
        $id = $request->input('id');
        ConfigService::delPayConfig($id);
        return $this->result();
    }

    /**
     * @param Request $request
     * @return array|string
     * 修改审核状态
     */
    public function changeStatus(Request $request){
        $id = $request->route('id');
        DB::beginTransaction();
        try {
            $users = ShimmerLiveshopAnchorApply::find($id);
            if ($users) {
                $users->status = !$users->status;
                $users->save();
                if ($users->status==1){
                    AnchorService::createAnchorLive($users);
                }
                AnchorService::changeStatus($users->uid,$users->status);
                DB::commit();
                return $this->result();
            }
        } catch (\Exception $e) {
            DB::rollBack();
            return $this->result(1,$e->getMessage());
        }

    }

    public function destory(Request $request){
        $id = $request->route('id');
        $users = ShimmerLiveshopAnchorApply::find($id);
        $users->delete();
        ShimmerLiveshopAnchorApplyPay::where('uid',$users->uid)->delete();
        return $this->result();
    }

    /**
     * 导出订单
     */
    private function export($list)
    {
        $cellData = [[
            '用户uid',
            '用户昵称',
            '用户手机号',
            '主播等级',
            '带货总金额',
            '带货总佣金',
            '当前佣金',
            '剩余邀请主播名额',
            '状态',
            '拉黑',
            '收到礼物(元)',
            '名下主播数量',
            '邀请主播佣金',
            '申请时间',
        ]];

        foreach ($list as $index => $item) {
            $_cellData = [
                $item->user->nickname,
                $item->user->uid,
                $item->user->mobile,
                $item->userLevel->name ? $item->userLevel->name:'普通',
                $item->total_amount,
                $item->total_commission,
                $item->now_commission,
                $item->surplus_anchor_nums,
                $item->status==1 ? '正常':'待审核',
                $item->is_black==1 ? '已拉黑':'正常',
                $item->income_gift,
                $item->child_anchor_nums,
                $item->ini_anchor_commission,
                $item->created_at,
            ];


            $cellData[] = $_cellData;
        }

        $filename = '主播' . date('YmdHi');

        Excel::create($filename, function ($excel) use ($cellData) {
            $excel->sheet('主播列表', function ($sheet) use ($cellData) {
                $sheet->fromArray($cellData);
            });
        })->download('xlsx');
    }
}