<?php
/**
 * 文章带货
 * Created by PhpStorm.
 * User: lirui
 * Date: 2020/3/10
 * Time: 上午11:50
 */

namespace App\Modules\Live\Http\Controllers\Market;

use App\Models\ShimmerLiveshopArticle;
use App\Models\ShimmerLiveshopGoods;
use App\Modules\Live\Http\Controllers\Controller;
use App\Services\GoodsCategoryService;
use Illuminate\Http\Request;

class ArticleController extends Controller
{
    /**
     * 商品分类列表
     */
    public function index(Request $request)
    {
        //获取分页数据
        $query = $this->query;

        $query['category'] = 1;

        $builder = ShimmerLiveshopArticle::search($query);
        $builder->orderBy('sort', 'desc');
        $builder->orderBy('id', 'desc');
        $pager = $builder->paginate($this->pageNum);

        return view('live::market.article.index', [
            'pager' => $pager,
        ]);
    }

    public function create(Request $request)
    {
        return view('live::market.article.create', [
        ]);
    }

    public function store(Request $request)
    {
        $data = $this->validateScenarioPass($request, ShimmerLiveshopArticle::instance());
        $goodsIds = array_filter(explode(',', $data['goods_ids']));
        $data['goods_ids'] = $goodsIds;

        $model = new ShimmerLiveshopArticle();
        $ret = $model->fill($data)->save();

        if ($ret) {
            return $this->result(0, '保存成功', ['redirect' => ShimmerLiveshopRoute('live.market.article.index')]);
        } else {
            return $this->result(1, '保存失败');
        }
    }

    public function edit($uniacid, $id)
    {
        $model = ShimmerLiveshopArticle::find($id);
        $goodsIds = $model->goods_ids;
        if ($goodsIds) {
            $idsOrdered = implode(',', $goodsIds);
            $goods = ShimmerLiveshopGoods::whereIn('id', $goodsIds)
                ->orderByRaw("FIELD(id, $idsOrdered)")
                ->get();
        }
        $goodsIdStr = '';
        if ($model->goods_ids) {
            $goodsIdStr = implode(',', $model->goods_ids);
        }

        return view('live::market.article.edit', [
            'model' => $model,
            'goods' => $goods,
            'goodsIdStr' => $goodsIdStr,
        ]);
    }

    public function update(Request $request, $uniacid, $id)
    {
        $data = $this->validateScenarioPass($request, ShimmerLiveshopArticle::instance());
        $goodsIds = array_filter(explode(',', $data['goods_ids']));
        $data['goods_ids'] = $goodsIds;

        $model = (new ShimmerLiveshopArticle())->loadModel($id);
        $ret = $model->fill($data)->save();

        if ($ret) {
            return $this->result(0, '保存成功', ['redirect' => ShimmerLiveshopRoute('live.market.article.index')]);
        } else {
            return $this->result(1, '保存失败');
        }
    }

    public function destroy($uniacid, $id)
    {
        if (!$id) {
            return $this->error(-1, '参数错误');
        }

        $model = ShimmerLiveshopArticle::find($id);
        $model->delete();

        return $this->result(0, '删除成功');
    }

    /**
     * 修改商品分类是否显示状态
     */
    public function changeIsShow(Request $request)
    {
        $id = $request->input('id');
        $isshow = $request->input('isshow');

        if (!$id) {
            return $this->error(-1, '参数错误');
        }

        $ret = ShimmerLiveshopArticle::where('id', $id)->update(['isshow' => $isshow]);

        if ($ret) {
            return $this->success();
        } else {
            return $this->error();
        }
    }

}
