<?php
/**
 * 实名认证
 * Created by PhpStorm.
 * User: lirui
 * Date: 2020/1/11
 * Time: 上午8:48
 */

namespace App\Modules\Live\Http\Controllers\Member;

use App\Models\ShimmerLiveshopCertification;
use App\Models\ShimmerLiveshopTemplate;
use App\Models\ShimmerLiveshopUser;
use App\Modules\Live\Http\Controllers\Controller;
use App\Modules\Liveapi\Services\MiniProgramService;
use App\Services\TemplateService;
use Illuminate\Http\Request;

class CertificationController extends Controller
{
    public function index(Request $request)
    {
        //获取分页数据
        $query = $this->query;

        $builder = ShimmerLiveshopCertification::search($query);
        $builder->with('user');
        $builder->orderBy('check_status', 'asc');
        $builder->orderBy('id', 'desc');

        $pager = $builder->paginate($this->pageNum);

        return view('live::member.certification.index', [
            'pager' => $pager,
        ]);
    }

    /**
     * 实名认证审核
     * @param Request $request
     */
    public function check(Request $request)
    {
        $id = $request->input('id');
        $checkStatus = $request->input('check_status');

        if (is_array($id)) {
            foreach ($id as $idItem) {
                $certification = ShimmerLiveshopCertification::find($idItem);
                $certification->check_status = $checkStatus;
                $ret = $certification->save();
                $this->doCheckAfter($certification);
            }
        } else {
            $certification = ShimmerLiveshopCertification::find($id);
            $certification->check_status = $checkStatus;
            $certification->save();
            $this->doCheckAfter($certification);
        }

        return $this->success();
    }

    /**
     * 审核后处理
     * @param $certification
     */
    private function doCheckAfter($certification)
    {
        $request = request();
        $reason = $request->input('reason', '无');

        $user = ShimmerLiveshopUser::find($certification->uid);

        if ($certification->check_status == 1) {
            $user->realname = $certification->realname;
            $user->idcard = $certification->idcard;
            $user->save();

            $page = '/membercenter/certification/index';
            $data = [
                'thing2' => [
                    'value' => '实名认证',
                ],
                'phrase1' => [
                    'value' => '审核通过',
                ],
                'date4' => [
                    'value' => date('Y-m-d H:i:s'),
                ],
                'thing12' => [
                    'value' => $reason
                ]
            ];
            TemplateService::pushQueue('实名认证审核通过', $user->id, 402, $data, $page,1);
        } elseif ($certification->check_status == 2) {
            $data = [
                'thing2' => [
                    'value' => '实名认证',
                ],
                'phrase1' => [
                    'value' => '审核未通过',
                ],
                'date4' => [
                    'value' => date('Y-m-d H:i:s'),
                ],
                'thing12' => [
                    'value' => $reason
                ]
            ];
            $page = '/membercenter/certification/index';
            TemplateService::pushQueue('实名认证审核拒绝', $user->id, 402, $data, $page,1);
        }
    }

    /**
     * 拒绝审核
     * @param Request $request
     */
    public function checkReject(Request $request)
    {
        $id = $request->input('id');

        $model = ShimmerLiveshopCertification::find($id);

        return view('live::member.certification.check_reject', [
            'model' => $model,
        ]);
    }

    public function destroy(Request $request, $uniacid, $id)
    {
        $ids = $request->input('id');

        if (!$ids && !$id) {
            return $this->error(-1, '请选择删除对象');
        }

        if ($ids) {
            $ret = ShimmerLiveshopCertification::whereIn('id', $ids)->delete();
        } else {
            $model = ShimmerLiveshopCertification::find($id);
            $ret = $model->delete();
        }

        if ($ret) {
           return $this->success();
        } else {
           return $this->error(-2, '删除失败');
        }
    }

    /**
     * @param Request $request
     * @param $uniacid
     * @param $id
     */
    public function edit(Request $request, $uniacid, $id)
    {
        $model = ShimmerLiveshopCertification::find($id);
        if (!$model) {
            admin_toastr('实名认证记录不存在', 'error');
            return redirect(wgRoute('live.member.certification.index'));
        }

        return view('live::member.certification.edit', [
            'model' => $model,
        ]);
    }

    /**
     * 编辑保存实名认证
     * @param Request $request
     */
    public function update(Request $request, $uniacid, $id)
    {
        $data = $this->validateScenarioPass($request, ShimmerLiveshopCertification::instance());

        $model = ShimmerLiveshopCertification::find($id);
        $ret = $model->fill($data)->save();

        if ($ret) {
            return $this->success(['redirect' => wgRoute('live.member.certification.index')]);
        } else {
            return $this->error(-2, '保存失败');
        }
    }
}