<?php
/**
 * 插件中心
 * Created by PhpStorm.
 * User: lirui
 * Date: 2019/12/13
 * Time: 上午05:15
 */

namespace App\Modules\Live\Http\Controllers;

use App\Events\Plugin\PluginDisable;
use App\Events\Plugin\PluginEnable;
use App\Exceptions\Exception;
use App\Models\Module;
use App\Models\ShimmerLiveshopPlugin;
use App\Services\PluginService;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class PluginController extends Controller
{
    /**
     * 插件列表
     */
    public function index(Request $request)
    {
        $this->setPageTitle('插件中心');

        $builder = ShimmerLiveshopPlugin::search($this->query);
        $pager = $builder->paginate($this->pageNum);

        foreach ($pager as $key => $item) {
            $slug = "shimmer_liveshop_plugin_{$item->slug}";
            $pager[$key]['module'] = Module::where('name', $slug)->first();
        }

        return view("live::plugin.list", [
            'pager' => $pager,
        ]);
    }

    /**
     * 插件启用禁用
     * @param Request $request
     */
    public function enableSwitch(Request $request)
    {
        $id = $request->input('id');

        $plugin = ShimmerLiveshopPlugin::find($id);

        if (!$plugin) {
            return $this->error(-1, '插件不存在');
        }

        if ($plugin->enable == 0) {
            $plugin->enable = 1;
        } else {
            $plugin->enable = 0;
        }

        DB::beginTransaction();
        try {
            if ($plugin->enable == 0) {
                event(new PluginDisable($plugin));
            } else {
                event(new PluginEnable($plugin));
            }
            $plugin->save();
        } catch (Exception $exception) {
            DB::rollBack();
            return $this->error(-1, $exception->getMessage());
        }

        DB::commit();

        return $this->success('', '操作成功');
    }

    /**
     * 删除插件
     * @param Request $request
     */
    public function delete(Request $request)
    {
        $id = $request->input('id');

        $plugin = ShimmerLiveshopPlugin::find($id);
        $plugin->delete();

        return $this->result(0, '删除成功');
    }

}