<?php

namespace App\Modules\Live\Http\Controllers\Shop;

use App\Models\ShimmerLiveshopCoupon;
use App\Models\ShimmerLiveshopCouponUser;
use App\Modules\Live\Http\Controllers\Controller;
use Illuminate\Http\Request;

class CouponController extends Controller
{
    /**
     * 优惠券列表
     * @return \Illuminate\Contracts\View\Factory|\Illuminate\View\View
     */
    public function index()
    {
        $queryData['type'] = ShimmerLiveshopCoupon::$types;
        $queryData['state'] = ShimmerLiveshopCoupon::$states;
        $queryData['valid_state'] = [
            1 => '未开始',
            2 => '有效期内',
            3 => '已结束',
        ];

        $query = $this->query;
        $validState = $query['valid_state'];
        unset($query['valid_state']);

        $builder = ShimmerLiveshopCoupon::search($query);
        $nowDate = date('Y-m-d H:i:s');

        if ($validState == 1) {
            $builder->where('start_date', '>', $nowDate);
        } else if ($validState == 2) {
            $builder->where('start_date', '<=', $nowDate);
            $builder->where('end_date', '>=', $nowDate);
        } else if ($validState == 3) {
            $builder->where('end_date', '<', $nowDate);
        }

        unset($query['valid_state']);


        $pager = $builder->paginate($this->pageNum);

        return view('live::shop.coupon.index', [
            'pager' => $pager,
            'queryData' => $queryData,
        ]);
    }

    /**
     * 创建优惠券
     */
    public function create()
    {
        $formData['type'] = ShimmerLiveshopCoupon::$types;

        return view('live::shop.coupon.create', [
            'formData' => $formData,
        ]);
    }

    /**
     * 编辑优惠券
     */
    public function edit($uniacid, $id)
    {
        $formData['type'] = ShimmerLiveshopCoupon::$types;
        $model = ShimmerLiveshopCoupon::find($id);

        return view('live::shop.coupon.edit', [
            'model' => $model,
            'formData' => $formData,
        ]);
    }

    /**
     * 新建保存优惠券
     */
    public function store(Request $request)
    {
        $data = $this->validateScenarioPass($request, ShimmerLiveshopCoupon::instance());
        $redirect = wgRoute('liveshopcoupon.index');
        $dataTime = $data['datetime'];
        unset($data['datetime']);

        $data['left_num'] = $data['num'];

        if ($dataTime) {
            $dataTime = explode(' - ', $dataTime);
            $data['start_date'] = trim($dataTime[0]);
            $data['end_date'] = trim($dataTime[1]);
        }

        $model = new ShimmerLiveshopCoupon();
        $ret = $model->fill($data)->save();

        if ($ret) {
            return $this->result(0, '保存成功', ['redirect' => $redirect]);
        } else {
            return $this->result(1, '保存失败');
        }
    }

    /**
     * 编辑保存优惠券
     */
    public function update(Request $request, $uniacid, $id)
    {
        $data = $this->validateScenarioPass($request, ShimmerLiveshopCoupon::instance());
        $redirect = wgRoute('liveshopcoupon.index');
        $dataTime = $data['datetime'];
        unset($data['datetime']);

        $data['left_num'] = $data['num'];

        if ($dataTime) {
            $dataTime = explode(' - ', $dataTime);
            $data['start_date'] = trim($dataTime[0]);
            $data['end_date'] = trim($dataTime[1]);
        }

        $model = (new ShimmerLiveshopCoupon())->loadModel($id);
        $ret = $model->fill($data)->save();

        if ($ret) {
            return $this->result(0, '保存成功', ['redirect' => $redirect]);
        } else {
            return $this->result(1, '保存失败');
        }
    }

    /**
     * 删除优惠券
     */
    public function destroy(Request $request, $uniacid, $id)
    {
        $ret = ShimmerLiveshopCoupon::destroy($id);

        if ($ret) {
            return $this->result(0, '删除成功');
        } else {
            return $this->result(1, '删除失败');
        }
    }

    /**
     * 修改优惠券状态
     */
    public function changeState(Request $request)
    {
        $id = $request->input('id');
        $state = $request->input('state');

        $ret = ShimmerLiveshopCoupon::where('id', $id)->update(['state' => $state]);

        if ($ret) {
            return $this->result(0, '更新成功');
        } else {
            return $this->result(1, '更新失败');
        }
    }

    /**
     * 优惠券数据统计
     */
    public function stat()
    {
        $queryData['status'] = [
            0 => '未使用',
            1 => '已使用',
            2 => '已过期',
        ];

        $query = $this->query;

        $addTime = $query['add_time'];
        $endTime = $query['end_time'];

        unset($query['add_time'], $query['end_time']);

        $builder = ShimmerLiveshopCouponUser::search($query);

        if ($addTime) {
            $builder->where('add_time', '>=', $addTime);
        }
        if ($endTime) {
            $builder->where('end_time', '<=', $endTime);
        }

        $pager = $builder->paginate($this->pageNum);

        return view('live::shop.coupon.stat', [
            'pager' => $pager,
            'queryData' => $queryData,
        ]);
    }
}