<?php
/**
 * 商品管理
 * Created by PhpStorm.
 * User: lirui
 * Date: 2019/8/9
 * Time: 上午9:48
 */

namespace App\Modules\Live\Http\Controllers\Shop;

use App\Models\ShimmerLiveshopAnchorPayLevel;
use App\Models\ShimmerLiveshopGoods;
use App\Models\ShimmerLiveshopGoodsAnchorCommission;
use App\Models\ShimmerLiveshopGoodsCategory;
use App\Models\ShimmerLiveshopGoodsCategoryXref;
use App\Models\ShimmerLiveshopGoodsCommission;
use App\Models\ShimmerLiveshopGoodsSpec;
use App\Models\ShimmerLiveshopGoodsSpecItem;
use App\Models\ShimmerLiveshopGoodsSpecOption;
use App\Models\ShimmerLiveshopTag;
use App\Modules\Live\Http\Controllers\Controller;
use App\Modules\Live\Services\GoodsGrab;
use App\Services\GoodsCategoryService;
use App\Services\GoodsService;
use App\Services\GoodsSpecService;
use App\Services\ImageService;
use App\Services\PluginService;
use App\Services\TagService;
use Illuminate\Http\Request;

class GoodsController extends Controller
{
    /**
     * 商品列表
     */
    public function index()
    {
        $queryData['category'] = GoodsCategoryService::getAllLevelCategorys();
        $queryData['state'] = ShimmerLiveshopGoods::$states;

        //获取分页数据
        $query = $this->query;
        $categoryId = $query['category_id'];
        $title = $query['title'];
        $startTime = $query['start_time'];
        $endTime = $query['end_time'];

        unset($query['start_time'], $query['end_time']);
        $builder = ShimmerLiveshopGoods::select('shimmer_liveshop_goods.*')
            ->leftJoin('shimmer_liveshop_goods_category_xrefs', 'shimmer_liveshop_goods_category_xrefs.goods_id', 'shimmer_liveshop_goods.id')
            ->leftJoin('shimmer_liveshop_goods_categories', 'shimmer_liveshop_goods_categories.id', 'shimmer_liveshop_goods_category_xrefs.category_id')
            ->withoutGlobalScope('uniacid');

        $builder->where('shimmer_liveshop_goods.uniacid', $this->uniacid);

        if ($categoryId) {
            $category = ShimmerLiveshopGoodsCategory::find($categoryId);
            if ($category->parent_id != 0) {
                $builder->where('shimmer_liveshop_goods_categories.id', $categoryId);
            } else {
                $categoryIds = $category->getChilds()->pluck('id')->toArray();
                $builder->whereIn('shimmer_liveshop_goods_categories.id', $categoryIds);
            }
        }
        if ($title) {
            $builder->where('shimmer_liveshop_goods.title', 'like', "%{$title}%");
        }
        if ($startTime) {
            $builder->where('shimmer_liveshop_goods.stateon_at', '>=', $startTime);
        }
        if ($endTime) {
            $builder->where('shimmer_liveshop_goods.stateon_at', '<=', $endTime);
        }
        if (is_numeric($query['state'])) {
            $builder->state($query['state']);
        }

        $builder->orderBy('id', 'desc');
        $builder->groupBy('shimmer_liveshop_goods.id');
        $pager = $builder->paginate($this->pageNum);

        return view('live::shop.goods.index', [
            'queryData' => $queryData,
            'pager' => $pager,
        ]);
    }

    private function getFormCateogry()
    {
        $categorys = GoodsCategoryService::getAllLevelCategorys();
        $result = [];
        $i = 0;
        foreach ($categorys as $key => $row) {
            ++$i;
            $result[$i]['name'] = "|-----{$row['name']}";
            $result[$i]['status'] = 0;
            foreach ((array)$categorys[$key]['child'] as $key1 => $row1) {
                ++$i;
                $result[$i] = $row1;
                $result[$i]['name'] = "|-----|-----{$row1['name']}";
            }
        }

        $result = array_values($result);
        return $result;
    }

    public function create(Request $request)
    {
        $tags = TagService::getTags();
        $tags = $tags[ShimmerLiveshopTag::GOODS_TAG]['list'];
        $ensures = ShimmerLiveshopGoods::$ensures;
        $buyway = $request->input('buyway', 1);

        return view('live::shop.goods.create', [
            'category' => $this->getFormCateogry(),
            'buyway' => $buyway,
            'tags' => $tags,
            'ensures' => $ensures
        ]);
    }

    /**
     * 保存商品标签
     * @param $goodsId
     * @param $data
     */
    private function saveTag($goodsId, $data)
    {
        $tagIds = (array)$data['tag_ids'];
        $goodsTagIds = ShimmerLiveshopTag::getRelationTagIds(1, $goodsId);

        $addIds = array_diff($tagIds, $goodsTagIds);
        $delIds = array_diff($goodsTagIds, $tagIds);

        if ($delIds) {
            ShimmerLiveshopTag::delRelationTags(1, $goodsId, $delIds);
        }
        if ($addIds) {
            ShimmerLiveshopTag::addRelationTags(1, $goodsId, $addIds);
        }
    }

    /**
     * 保存商品规格参数
     * @param $goodsId
     * @param $data
     */
    private function saveOption($goodsId, $data)
    {
        //组装规格数据
        $specs = [];
        //规格列表
        foreach ($data['spec_title'] as $specNum => $specTitle) {
            if (!$specTitle) {
                continue;
            }
            //获取规格项
            $specNum += 1;

            $spec = [
                'index' => $specNum,
                'id' => $data["spec_{$specNum}_id"],
                'title' => $specTitle,
            ];

            foreach ($data['spec_item_title_' . $specNum] as $specItemNum => $specItemTitle) {
                $specItemIndex = $specItemNum + 1;

                $spec['spec_item'][] = [
                    'index' => $specItemIndex,
                    'id' => $data["spec_item_{$specNum}_{$specItemIndex}_id"],
                    'title' => $specItemTitle,
                    'thumb' => $data['spec_item_thumb_' . $specNum][$specItemNum],
                ];
            }

            $specs[] = $spec;
        }

        $specItems = [];
        foreach ($specs as $k => $spec) {
            $specData = [
                'uniacid' => $this->uniacid,
                'title' => $spec['title'],
                'goods_id' => $goodsId,
            ];
            if ($spec['id']) {
                ShimmerLiveshopGoodsSpec::where('id', $spec['id'])->update($specData);
            } else {
                $model = new ShimmerLiveshopGoodsSpec();
                $model->fill($specData)->save();
                $spec['id'] = $model->id;
            }
            //插入规格
            foreach ($spec['spec_item'] as $k2 => $specItem) {
                $specItemData = [
                    'uniacid' => $this->uniacid,
                    'spec_id' => $spec['id'],
                    'title' => $specItem['title'],
                    'thumb' => $specItem['thumb'],
                    'goods_id' => $goodsId,
                ];
                if ($specItem['id']) {
                    ShimmerLiveshopGoodsSpecItem::where('id', $specItem['id'])->update($specItemData);
                } else {
                    $model = new ShimmerLiveshopGoodsSpecItem();
                    $model->fill($specItemData)->save();
                    $specItem['id'] = $model->id;
                }

                $spec['spec_item'][$k2] = $specItem;
            }

            $specItems[] = $spec['spec_item'];
            $specs[$k] = $spec;
        }

        //保存规格价格
        $options = [];
        if (count($specItems) > 1) {
            for ($i = 0; $i < count($specItems) - 1; $i++) {
                $tmp = [];
                if ($i == 0) {
                    $options = $specItems[$i];
                }
                foreach ($options as $option) {
                    foreach ($specItems[$i + 1] as $specItem) {
                        if (isset($option['id'])) {
                            $arr = [$option];
                        } else {
                            $arr = $option;
                        }
                        array_push($arr, $specItem);
                        $tmp[] = $arr;
                    }
                }
                $options = $tmp;
            }
        } else {
            for ($i = 0; $i < count($specItems[0]); $i++) {
                $options[$i][0] = $specItems[0][$i];
            }
        }

        /**
         * 更新价格
         */
        foreach ($options as $option) {
            $specItemIds = array_column($option, 'id');
            sort($specItemIds);
            $specItemNumber = array_column($option, 'index');

            $specItemNumber = implode('_', $specItemNumber);
            if (count($specItemIds) == 1) {
                $specItemNumber = "1_{$specItemNumber}";
            }
            $specItemIds = implode(',', $specItemIds);

            $title = array_column($option, 'title');
            $title = implode(',', $title);

            $goodsOption = ShimmerLiveshopGoodsSpecOption::where('goods_id', $goodsId)->where('spec_item_ids', $specItemIds)->first();

            $saveData = [
                'uniacid' => $this->uniacid,
                'goods_id' => $goodsId,
                'title' => $title,
                'stock' => (int)$data["option_stock_{$specItemNumber}"],
                'marketprice' => (float)$data["option_marketprice_{$specItemNumber}"],
                'costprice' => (float)$data["option_costprice_{$specItemNumber}"],
                'productprice' => (float)$data["option_productprice_{$specItemNumber}"],
                'spec_item_ids' => (string)$specItemIds,
            ];

            if ($goodsOption) {
                ShimmerLiveshopGoodsSpecOption::where('id', $goodsOption['id'])->update($saveData);
            } else {
                $model = new ShimmerLiveshopGoodsSpecOption();
                $model->fill($saveData)->save();
                $option['id'] = $model->id;
            }
        }
    }

    /**
     * 保存商品分类
     * @param $goods
     * @param $data
     */
    private function saveCategory($goods, $data)
    {
        if (!$goods || !$data['category_id']) {
            return false;
        }

        $categoryIds = explode(',', $data['category_id']);
        if (!$categoryIds) {
            return false;
        }

        GoodsService::saveCategorys($goods, $categoryIds);
    }

    public function store(Request $request)
    {
        $data = $this->validateScenarioPass($request, ShimmerLiveshopGoods::instance());
        $redirect = wgRoute('liveshopgoods.index');

        if ($data['hasoption'] == 1) {
        } else {
            unset($data['spec_title']);
        }


        $goodsData = [
            'uniacid' => $this->uniacid,
            'title' => trim($data['title']),
            'category_id' => (string)$data['category_id'],
            'buyway' => (int)$data['buyway'],
            'buymode' => (int)$data['buymode'],
            'hasoption' => (int)$data['hasoption'],
            'stock' => (int)$data['stock'],
            'state' => (int)$data['state'],
            'sort' => (int)$data['sort'],
            'buy_limit' => (int)$data['buy_limit'],
            'subtitle' => trim($data['subtitle']),
            'marketprice' => trim($data['marketprice']),
            'productprice' => trim($data['productprice']),
            'expressprice' => trim($data['expressprice']),
            'description' => trim($data['description']),
            'thumb' => (string)$data['thumb'],
            'video' => (string)$data['video'],
            'is_hot' => (int)$data['is_hot'],
            'is_sales' => (int)$data['is_sales'],
            'is_vip' => (int)$data['is_vip'],
            'is_new' => (int)$data['is_new'],
            'is_free_freight' => (int)$data['is_free_freight'],
            'ficti_num' => (int)$data['ficti_num'],
            'keyword' => trim($data['keyword']),
            'unit' => trim($data['unit']),
            'ensures' => (array)$data['ensures'],
            'thirdurl' => (array)$data['thirdurl'],
        ];

        $goodsData['local_thumb'] = ImageService::ossImgToLocal($goodsData['thumb']);

        if ($goodsData['state'] == 1) {
            $goodsData['stateon_at'] = date('Y-m-d H:i:s');
        }
        if ($data['thumbs']) {
            $goodsData['thumbs'] = implode(',', $data['thumbs']);
        }

        $goodsData = $this->formatGoodsSaveData($goodsData);

        $model = new ShimmerLiveshopGoods();
        $ret = $model->fill($goodsData)->save();
        $goodsId = $model->id;

        if ($data['hasoption'] == 1) {
            $this->saveOption($goodsId, $data);
        }

        $this->saveCategory($model, $data);
        //保存标签
        $this->saveTag($goodsId, $data);

        if ($ret) {
            return $this->result(0, '保存成功', ['redirect' => $redirect]);
        } else {
            return $this->result(1, '保存失败');
        }
    }

    public function edit(Request $request, $uniacid, $id)
    {
        $model = ShimmerLiveshopGoods::find($id);
        $tags = TagService::getTags();
        $tags = $tags[ShimmerLiveshopTag::GOODS_TAG]['list'];
        $ensures = ShimmerLiveshopGoods::$ensures;

        $specs = [];

        if (!$model) {
            admin_toastr('商品不存在', 'error');
            return back();
        }

        if ($model->hasoption == 1) {
            $specs = GoodsSpecService::getGoodsSpec($model->id);
        }

        $goodsCategoryIds = $model->getCategoryIds();
        $model->tag_ids = ShimmerLiveshopGoods::getGoodsTagIds($id);

        $buyway = $request->input('buyway', $model->buyway);

        $extendsData = \App\Services\Goods\GoodsService::getExtendData($id);
        return view('live::shop.goods.edit', [
            'model' => $model,
            'category' => $this->getFormCateogry(),
            'goodsCategoryIds' => $goodsCategoryIds,
            'specs' => $specs,
            'extendsData' => $extendsData,
            'buyway' => $buyway,
            'tags' => $tags,
            'ensures' => $ensures
        ]);
    }

    public function update(Request $request, $uniacid, $id)
    {
        $data = $this->validateScenarioPass($request, ShimmerLiveshopGoods::instance());
        $redirect = wgRoute('liveshopgoods.index');

        if ($data['hasoption'] == 1) {
        } else {
            unset($data['spec_title']);
        }

        $goodsData = [
            'uniacid' => $this->uniacid,
            'category_id' => (string)$data['category_id'],
            'title' => trim($data['title']),
            'buyway' => (int)$data['buyway'],
            'buymode' => (int)$data['buymode'],
            'hasoption' => (int)$data['hasoption'],
            'stock' => (int)$data['stock'],
            'state' => (int)$data['state'],
            'sort' => (int)$data['sort'],
            'buy_limit' => (int)$data['buy_limit'],
            'subtitle' => trim($data['subtitle']),
            'marketprice' => trim($data['marketprice']),
            'productprice' => trim($data['productprice']),
            'expressprice' => trim($data['expressprice']),
            'thumb' => (string)$data['thumb'],
            'video' => (string)$data['video'],
            'description' => trim($data['description']),
            'is_hot' => (int)$data['is_hot'],
            'is_sales' => (int)$data['is_sales'],
            'is_vip' => (int)$data['is_vip'],
            'is_new' => (int)$data['is_new'],
            'is_free_freight' => (int)$data['is_free_freight'],
            'ficti_num' => (int)$data['ficti_num'],
            'keyword' => trim($data['keyword']),
            'unit' => trim($data['unit']),
            'ensures' => (array)$data['ensures'],
            'thirdurl' => (array)$data['thirdurl'],
        ];

        $goodsData['local_thumb'] = ImageService::ossImgToLocal($goodsData['thumb']);

        if ($goodsData['state'] == 1) {
            $goodsData['stateon_at'] = date('Y-m-d H:i:s');
        }
        if ($data['thumbs']) {
            $goodsData['thumbs'] = implode(',', $data['thumbs']);
        }

        $goodsData = $this->formatGoodsSaveData($goodsData);

        $model = (new ShimmerLiveshopGoods())->loadModel($id);
        $ret = $model->fill($goodsData)->save();
        $goodsId = $model->id;

        if ($data['hasoption'] == 1) {
            $this->saveOption($goodsId, $data);
        }

        $this->saveCategory($model, $data);
        //保存标签
        $this->saveTag($goodsId, $data);
        (new \App\Modules\Live\Services\GoodsService())->liveGoodsDestory($goodsId, $goodsData['state']);
        if ($ret) {
            return $this->result(0, '保存成功', ['redirect' => $redirect]);
        } else {
            return $this->result(1, '保存失败');
        }
    }

    public function destroy(Request $request, $uniacid, $id)
    {
        $ids = $request->input('id');
        if (!$ids && !$id) {
            return $this->result(1, '请选择删除对象');
        }

        if ($ids) {
            ShimmerLiveshopGoods::whereIn('id', $ids)->delete();
            (new \App\Modules\Live\Services\GoodsService())->liveGoodsDestory($ids);
        } else {
            $goods = ShimmerLiveshopGoods::find($id);
            $goods->delete();
            (new \App\Modules\Live\Services\GoodsService())->liveGoodsDestory($id);
        }

        return $this->result(0, '删除成功');
    }

    /**
     * 商品保存提交数据格式化
     * @param $goodsData
     */
    private function formatGoodsSaveData($goodsData)
    {
        $goodsData['thumb'] = ImageService::remoteImgToLocal($goodsData['thumb']);
        if ($goodsData['thumbs']) {
            if (!is_array($goodsData['thumbs'])) {
                $goodsData['thumbs'] = explode(',', $goodsData['thumbs']);
            }
            foreach ($goodsData['thumbs'] as $key => $thumb) {
                $goodsData['thumbs'][$key] = ImageService::remoteImgToLocal($thumb);
            }
            $goodsData['thumbs'] = implode(',', $goodsData['thumbs']);
        }
        if ($goodsData['description']) {
            $pattern = "/src=[\"\'](.*?)[\"\']/is";
            preg_match_all($pattern, $goodsData['description'], $match);

            if ($match && is_array($match)) {
                foreach ($match[1] as $img) {
                    $localImg = wgTomedia(ImageService::remoteImgToLocal($img));
                    $goodsData['description'] = str_replace($img, $localImg, $goodsData['description']);
                }
            }
        }

        return $goodsData;
    }

    /**
     * 修改商品状态
     */
    public function changeState(Request $request)
    {
        $id = $request->input('id');
        $state = (int)$request->input('state');

        if (!$id) {
            $this->result(1, '请选择商品');
        }

        $result = GoodsService::changeState($id, $state);
        (new \App\Modules\Live\Services\GoodsService())->liveGoodsDestory($id, $state);
        if ($result) {
            return $this->result(0, '保存成功');
        } else {
            return $this->result(1, '保存失败');
        }
    }

    /**
     * 删除规格
     */
    public function delSpec(Request $request)
    {
        $id = $request->input('id');

        if (!$id) {
            return $this->result(1, '参数错误');
        }

        //删除规格
        $spec = ShimmerLiveshopGoodsSpec::find($id);
        $goodsId = $spec->goodsId;

        $spec->delete();

        //删除规格项
        ShimmerLiveshopGoodsSpecItem::specId($id)->delete();

        //删除商品所有规格值
        ShimmerLiveshopGoodsSpecOption::goodsId($goodsId)->delete();

        return $this->result(0, '删除成功');
    }

    /**
     * 删除规格项
     */
    public function delSpecItem(Request $request)
    {
        $id = $request->input('id');

        if (!$id) {
            return $this->result(1, '参数错误');
        }

        //删除规格项目
        $specItem = ShimmerLiveshopGoodsSpecItem::find($id);
        $goodsId = $specItem->goodsId;

        $specItem->delete();

        //删除商品所有规格值
        ShimmerLiveshopGoodsSpecOption::goodsId($goodsId)->delete();

        return $this->result(0, '删除成功');
    }

    /**
     * 批量修改商品分类
     * @param Request $request
     */
    public function changeCategory(Request $request)
    {
        $id = $request->input('id');

        return view('live::shop.goods.changeCategory', [
            'category' => $this->getFormCateogry(),
            'id' => $id,
        ]);
    }

    /**
     * 批量修改商品分类保存
     * @param Request $request
     */
    public function changeCategorySave(Request $request)
    {
        $id = $request->input('id');
        $data['category_id'] = $request->input('category_id');

        if (!$id) {
            return $this->error(-1, '请选择商品!');
        }

        $ids = explode(',', $id);

        foreach ($ids as $goodsId) {
            $goods = ShimmerLiveshopGoods::find($goodsId);
            $this->saveCategory($goods, $data);
        }

        return $this->success();
    }

    /**
     * 导入商品
     */
    public function import()
    {
        return view('live::shop.goods.import', [
            'category' => $this->getFormCateogry(),
        ]);
    }

    /**
     * 抓取商品
     * @param Request $request
     */
    public function doImport(Request $request)
    {
        set_time_limit(0);

        $link = trim($request->input('link'));

        $link = $this->checkImportLink($link);

        $goods = GoodsGrab::grab($link);

        if (!$goods) {
            return '';
        }

        $goods['buymode'] = 1;
        $goods['state'] = 1;
        $model = new ShimmerLiveshopGoods();
        $model->fill($goods);

        $tags = TagService::getTags();
        $tags = $tags[ShimmerLiveshopTag::GOODS_TAG]['list'];
        $ensures = ShimmerLiveshopGoods::$ensures;
        $buyway = $request->input('buyway', 1);

        return view('live::shop.goods.create_popup', [
            'category' => $this->getFormCateogry(),
            'buyway' => $buyway,
            'tags' => $tags,
            'ensures' => $ensures,
            'model' => $model,
        ]);
    }

    /**
     * 检测抓取链接
     * @param $link
     */
    public function checkImportLink($link)
    {
        $link = strtolower($link);
        if (!$link) {
            die('请输入商品链接');
        }
        if (substr($link, 0, 4) != 'http') {
            die('请输入正确的商品链接');
        }

        $arrLine = explode('?', $link);

        if (!count($arrLine)) {
            die('请输入商品链接');
        }

        if (!isset($arrLine[1])) {
            if (strpos($link, '1688') !== false && strpos($link, 'offer') !== false) {
                return trim($arrLine[0]);
            } else if (strpos($link, 'item.jd') !== false) {
                return trim($arrLine[0]);
            } else {
                die('请输入商品链接');
            }
        }

        if (strpos($link, '1688') !== false && strpos($link, 'offer') !== false) {
            return trim($arrLine[0]);
        }

        if (strpos($link, 'item.jd') !== false) {
            return trim($arrLine[0]);
        }

        $arrLineValue = explode('&', $arrLine[1]);

        if (!is_array($arrLineValue)) {
            die('请输入商品链接');

        }
        if (!strpos(trim($arrLine[0]), 'item.htm')) {
            die('请输入商品链接');
        }
        //链接参数
        $lastStr = '';
        foreach ($arrLineValue as $k => $v) {
            if (substr(strtolower($v), 0, 3) == 'id=') {
                $lastStr = trim($v);
                break;
            }
        }
        if (!$lastStr) {
            die('请输入商品链接');
        }
        return trim($arrLine[0]) . '?' . $lastStr;
    }
}