<?php
/**
 * 小程序底部导航设置
 * Created by PhpStorm.
 * User: lirui
 * Date: 2019/11/24
 * Time: 上午11:36
 */

namespace App\Modules\Live\Http\Controllers\Wxapp;

use App\Modules\Live\Http\Controllers\Controller;
use App\Modules\Live\Models\ShimmerLiveshopConfig;
use App\Modules\Liveapi\Services\MiniProgramService;
use Illuminate\Http\Request;

class TabbarController extends Controller
{
    public function index(Request $request)
    {
        $config = ShimmerLiveshopConfig::getByType(15);

        $list = [];
        $conf = [];
        if ($config) {
            $list = (array)$config['list'];
            if (is_array($list)) {
                $sorts = array_column($list, 'sort');
                array_multisort($sorts, SORT_DESC, $list);
            }
            $conf = $config['config'];
        }

        return view('live::wxapp.tabbar.index', [
            'list' => $list,
            'conf' => $conf,
        ]);
    }

    /**
     * 添加
     * @param Request $request
     */
    public function create(Request $request)
    {
        return view('live::wxapp.tabbar.create_form', [
        ]);
    }

    /**
     * 保存小程序导航
     * @param Request $request
     */
    public function store(Request $request)
    {
        $configValue = ShimmerLiveshopConfig::getByType(15);
        if (!$configValue) {
            $configValue = [];
        }
        $list = (array)$configValue['list'];

        $data = [
            'id' => uniqid(),
            'name' => trim($request->input('name')),
            'icon' => trim($request->input('icon')),
            'select_icon' => trim($request->input('select_icon')),
            'link' => trim($request->input('link')),
            'isshow' => trim($request->input('isshow')),
            'sort' => trim($request->input('sort')),
        ];

        if (!$data['name']) {
            return $this->error(-1, '导航名称不能为空');
        }
        if (!$data['icon']) {
            return $this->error(-1, '导航图标不能为空');
        }
        if (!$data['link']) {
            return $this->error(-1, '导航链接不能为空');
        }
        if (!$data['select_icon']) {
            $data['select_icon'] = $data['icon'];
        }


        $list[$data['id']] = $data;

        $configValue['list'] = $list;

        $config = ShimmerLiveshopConfig::getModelByType(15);
        $config->type = 15;
        $config->uniacid = $this->uniacid;
        $config->config = $configValue;
        $config->save();

        return $this->success();
    }

    /**
     * 删除导航
     * @param Request $request
     */
    public function destroy($uniacid, $id)
    {
        $config = ShimmerLiveshopConfig::getModelByType(15);
        $configValue = $config->config;

        if (!$config || !$configValue['list'] || !$configValue['list'][$id]) {
            return $this->success();
        }


        unset($configValue['list'][$id]);

        $config->config = $configValue;
        $config->save();

        return $this->success();
    }

    public function edit($uniacid, $id)
    {
        $configValue = ShimmerLiveshopConfig::getByType(15);

        $info = $configValue['list'][$id];

        return view('live::wxapp.tabbar.edit_form', [
            'info' => $info,
            'index' => $id,
        ]);
    }

    /**
     * @param $uniacid
     * @param $id
     */
    public function update(Request $request, $uniacid, $id)
    {
        $configValue = ShimmerLiveshopConfig::getByType(15);
        if (!$configValue) {
            $configValue = [];
        }
        $data = [
            'id' => $id,
            'name' => trim($request->input('name')),
            'icon' => trim($request->input('icon')),
            'select_icon' => trim($request->input('select_icon')),
            'link' => trim($request->input('link')),
            'sort' => trim($request->input('sort')),
            'isshow' => trim($request->input('isshow')),
        ];

        if (!$data['name']) {
            return $this->error(-1, '导航名称不能为空');
        }
        if (!$data['icon']) {
            return $this->error(-1, '导航图标不能为空');
        }
        if (!$data['link']) {
            return $this->error(-1, '导航链接不能为空');
        }
        if (!$data['select_icon']) {
            $data['select_icon'] = $data['icon'];
        }

        $configValue['list'][$id] = $data;
        $config = ShimmerLiveshopConfig::getModelByType(15);
        $config->config = $configValue;
        $config->save();

        return $this->success();
    }

    /**
     * 底部导航启用状态
     */
    public function changeIsShow(Request $request)
    {
        $id = $request->input('id');
        $isshow = $request->input('isshow');

        $configValue = ShimmerLiveshopConfig::getByType(15);
        if (!$configValue) {
            $configValue = [];
        }
        $data = $configValue['list'][$id];

        $data['isshow'] = $isshow;

        $configValue['list'][$id] = $data;

        $config = ShimmerLiveshopConfig::getModelByType(15);
        $config->config = $configValue;
        $config->save();

        return $this->success();
    }

    /**
     * 批量修改导航排序
     * @param Request $request
     */
    public function changesorts(Request $request)
    {
        $sorts = $request->input('sorts');

        $configValue = ShimmerLiveshopConfig::getByType(15);
        if (!$configValue) {
            $configValue = [];
        }
        foreach ($configValue['list'] as $k => $value) {
            $configValue['list'][$k]['sort'] = (int)$sorts[$k];
        }

        $config = ShimmerLiveshopConfig::getModelByType(15);
        $config->config = $configValue;
        $config->save();

        return $this->success();
    }

    /**
     * 导航设置
     * @param Request $request
     */
    public function config(Request $request)
    {
        $configValue = ShimmerLiveshopConfig::getByType(15);
        $config = [];

        if ($configValue) {
            $config = $configValue['config'];
        }

        return view('live::wxapp.tabbar.config_form', [
            'config' => $config,
        ]);
    }

    /**
     * 导航保存设置
     */
    public function configSave(Request $request)
    {
        $configValue = ShimmerLiveshopConfig::getByType(15);

        if (!$configValue) {
            $configValue = [];
        }

        $data = [
            'color' => $request->input('color'),
            'select_color' => $request->input('select_color'),
        ];

        $configValue['config'] = $data;

        $config = ShimmerLiveshopConfig::getModelByType(15);
        $config->config = $configValue;
        $config->save();

        return $this->success();
    }

    /**
     * 重置导航
     * @return array|string
     */
    public function reset()
    {
        $type = 15;

        $config = ShimmerLiveshopConfig::getModelByType($type);
        $configDefault = MiniProgramService::getDefaultTabbarConfig();

        $config->uniacid = $this->uniacid;
        $config->type = $type;
        $config->config = $configDefault;
        $config->save();

        return $this->success();
    }

}