@extends('live::layouts.master')
@section('title')
    资产明细
@endsection
@section('content')
    <div class="row">
        <div class="col-lg-12">
            <div class="ibox">
                <div class="ibox-content">
                    <form class="layui-form" action="">
                        <div class="layui-inline">
                            <label class="layui-form-label">明细类型</label>
                            <div class="layui-input-inline">
                                <select name="query[category]" lay-verify="">
                                    <option value="">全部</option>
                                    @foreach($queryData['category'] as $index=>$item)
                                        <option value="{{$index}}" {{ isNotNullAndNotEmptyString($query['category']) && $query['category']==$index ? 'selected': ''}}>{{$item}}</option>
                                    @endforeach
                                </select>
                            </div>
                        </div>
                        <div class="layui-inline">
                            <label class="layui-form-label">支出类型</label>
                            <div class="layui-input-inline">
                                <select name="query[is_income]" lay-verify="">
                                    <option value="">全部</option>
                                    @foreach($queryData['in_come'] as $index=>$item)
                                        <option value="{{$index}}" {{ isNotNullAndNotEmptyString($query['is_income']) && $query['is_income']==$index ? 'selected': ''}}>{{$item}}</option>
                                    @endforeach
                                </select>
                            </div>
                        </div>
                        <div class="layui-inline">
                            <label class="layui-form-label">时间:</label>
                            <div class="layui-input-inline" style="width: 150px;">
                                <input type="text" class="layui-input  start_time" value="{{$query['start_time']}}"
                                       name="query[start_time]"
                                       readonly>
                            </div>
                            -
                            <div class="layui-input-inline" style="width: 150px;">
                                <input type="text" class="layui-input end_time" value="{{$query['end_time']}}"
                                       name="query[end_time]"
                                       readonly>
                            </div>
                        </div>
                        <div class="layui-inline" style="padding-bottom: 5px">
                            <div class="layui-input-inline">
                                <input type="hidden" name="page_num" value="{{$currentPageNum}}">
                                <button name="submit" class="btn btn-success" type="submit">搜索</button>
                            </div>
                        </div>
                    </form>
                    @section('toolbar')
                        <span class="left">
                        <span class="static-left-text">总支出金额: {{$stats['total_out_money']}}</span>
                             <span class="static-left-text">总收入金额: {{$stats['total_in_money']}}</span>
                        </span>
                    @endsection
                    @include('live::common.toolbar', ['pager' => $pager])
                    <table class="layui-table">
                        <thead>
                        <tr>
                            <th>ID</th>
                            @if(\App\Services\PluginService::isEnabled('merch'))
                                <th>商户</th>
                            @endif
                            <th>订单ID</th>
                            <th>粉丝</th>
                            <th>会员</th>
                            <th>类型</th>
                            <th>支出类型</th>
                            <th>支付方式</th>
                            <th>金额</th>
                            <th>备注</th>
                            <th>时间</th>
                        </tr>
                        </thead>
                        <tbody>
                        @foreach($pager as $item)
                            <tr class="">
                                <td>
                                    {{$item->id}}
                                </td>
                                @if(\App\Services\PluginService::isEnabled('merch'))
                                    <td>
                                        @if($item->store)
                                            {{$item->store->id}}<br/>
                                            {{$item->store->name}}
                                        @else
                                            平台
                                        @endif
                                    </td>
                                @endif
                                <td>
                                    {{$item->oid}}
                                </td>
                                <td>
                                    @if($item->user)
                                        <img src="{{$item->user->avatar}}"
                                             class="rounded-circle img-sm img_pop_show"/><br/>
                                        {{$item->user->nickname}}
                                    @else
                                        未绑定
                                    @endif
                                </td>
                                <td>
                                    @if($item->user)
                                        {{$item->user->realname}}<br/>
                                        {{$item->user->mobile}}
                                    @else
                                        未绑定
                                    @endif
                                </td>
                                <td>
                                    {{$item->category_text}}
                                </td>
                                <td>
                                    {{$item->is_income_text}}
                                </td>
                                <td>
                                    {{$item->pay_type_text}}
                                </td>
                                <td>
                                    {{$item->money}}
                                </td>
                                <td>
                                    {{$item->remark}}
                                </td>
                                <td>
                                    {{$item->created_at}}
                                </td>
                            </tr>
                        @endforeach
                        </tbody>
                    </table>
                    @include('live::common.pagination', ['pager' => $pager])
                </div>
            </div>
        </div>
    </div>
@endsection
@section('script')
    <script>
        layui.use(['laydate', 'element'], function () {
            var laydate = layui.laydate, element = layui.element;
            //日期时间选择器
            laydate.render({
                elem: '.start_time'
                , type: 'datetime'
            });
            laydate.render({
                elem: '.end_time'
                , type: 'datetime'
            });
        });
    </script>
@endsection