<?php

namespace App\Modules\Liveapi\Http\Controllers;

use App\Models\ShimmerLiveshopComment;
use App\Models\ShimmerLiveshopUser;
use function EasyWeChat\Kernel\Support\get_client_ip;
use Illuminate\Http\Request;
use Illuminate\Pagination\LengthAwarePaginator;

class CommentController extends Controller
{

    /**
     * 添加评论
     */
    public function add(Request $request)
    {
        $contentType = (int)$request->input('content_type');
        $liveId = (int)$request->input('live_id');
        $content = $request->input('content');
        $uniacid = $request->route('uniacid');
        $giftlogid = (int)$request->route('giftlogid');

        $ip = $request->getClientIp();

        if (!$contentType) {
            return $this->result(1, '聊天类型参数错误');
        }
        if (!$content) {
            return $this->result(1, '评论内容不能为空');
        }

        $comment = new ShimmerLiveshopComment();
        $comment->live_id = $liveId;
        $comment->user_id = $request->userInfo->id;
        $comment->nickname = $request->userInfo->nickname;
        $comment->avatar = $request->userInfo->avatar;
        $comment->ip = $ip;
        $comment->uniacid = $uniacid;
        $comment->content_type = $contentType;
        $comment->content = $content;

        //回复
        if ($comment->is_reply) {
            $comment->is_reply = 1;
            $comment->to_id = (int)$request->input('to_id');;
            $comment->to_user_id = (int)$request->input('to_user_id');
            $toUser = ShimmerLiveshopUser::find($comment->to_user_id);
            $comment->to_nickname = $toUser->nickname;
            $comment->to_avatar = $toUser->avatar;
        }

        $ret = $comment->save();

        if ($ret) {
            $this->result(0, '评论成功');
        } else {
            $this->result(1, '评论失败');
        }
    }

    /**
     * 评论列表
     */
    public function list(Request $request)
    {
        $result = [];

        $liveId = (int)$request->input('live_id');

        if (!$liveId) {
            $this->result(1, '参数错误');
        }

        $pageNum = (int)$request->input('page_num', 10);
        $paginateList = ShimmerLiveshopComment::where('live_id', $liveId)->orderBy('id', 'desc')->paginate($pageNum);

        $list = $paginateList->sortBy('id')->values()->all();

        $list = new LengthAwarePaginator($list, $paginateList->total(), $paginateList->perPage());

        $this->result(1, '', $list);
    }
}