<?php
namespace App\Modules\Liveapi\Http\Controllers\Live;

use App\Events\OrderCreated;
use App\Models\ShimmerLiveshopGift;
use App\Models\ShimmerLiveshopGiftLog;
use App\Models\ShimmerLiveshopUser;
use App\Modules\Liveapi\Http\Controllers\Controller;
use App\Modules\Liveapi\Services\Gift\GiftService;
use App\Modules\Liveapi\Services\OrderService;
use App\Modules\Liveapi\Services\Wechat\WechatService;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class GiftController extends Controller {

    public function index(Request $request){
        $live_id = $request->route('live_id');
        $gifts = ShimmerLiveshopGift::where('status',1)->where('live_id',$live_id)->get();
        foreach($gifts as &$gift){
            $gift['image'] = wgTomedia($gift['image']);
        }
        return $this->result([
            'gifts'=>$gifts
        ],'ok');
    }

    /**
     * @param Request $request
     * 单个礼物信息
     */
    public function getGiftInfo(Request $request){
        $live_id = $request->route('live_id');
        $gift_id = $request->route('gift_id');
        $gift = ShimmerLiveshopGift::where('live_id',$live_id)
            ->where('id',$gift_id)
            ->first();
        if (!$gift){
            return $this->result(1,'礼物不存在');
        }
        $orderKey = OrderService::cacheOrderInfo($request->userInfo['id'], [
            'id'=>$gift->id,
            'title'=>'赠送礼物:'.$gift->name
        ], [
            'totalPrice'=>$gift->price,
            'costPrice'=>$gift->price,
            'vipPrice'=>$gift->price,
        ]);
        return $this->result(0,'ok',[
            'gift'=>$gift,
            'orderKey'=>$orderKey
        ]);
    }

    /**
     * @param Request $request
     * @throws \Throwable
     * 礼物支付
     */
    public function payGift(Request $request){
        $live_id = $request->route('live_id');
        $gift_id = $request->input('gift_id');
        $gift_num = $request->input('gift_num');

        $gift = ShimmerLiveshopGift::where('live_id',$live_id)
            ->where('id',$gift_id)
            ->first();
        if (!$gift){
            return $this->result(1,'礼物不存在');
        }
        $amount = bcmul($gift->price,$gift_num,2);

        $key = OrderService::cacheOrderInfo($request->userInfo['id'], [
            'id'=>time(),
            'title'=>'礼物支付'
        ], [
            'totalPrice'=>$amount,
            'costPrice'=>$amount,
            'vipPrice'=>$amount,
        ]);

        $formId = $request->route('formId');
        WechatService::collectFormId($request->userInfo['id'], $formId);
        $payType = strtolower($request->input('payType'));


        try {
            $order = OrderService::createdFictitiousOrder($request->userInfo['id'], $key,$payType,$amount,3,1,$live_id);

            $giftLogs = new ShimmerLiveshopGiftLog();
            $giftLogs->uniacid = $request->route('uniacid');
            $giftLogs->live_id = $live_id;
            $giftLogs->gift_id = $gift->id;
            $giftLogs->gift_name = $gift->name;
            $giftLogs->gift_image = $gift->image;
            $giftLogs->gift_num = $gift_num;
            $giftLogs->amount = $amount;
            $giftLogs->status = 0;
            $giftLogs->pay_time = 0;
            $giftLogs->order_id = $order->id;
            $giftLogs->uid = $request->userInfo['id'];
            $giftLogs->save();

        } catch (\Exception $e) {
            DB::rollBack();
            return $this->error(-1, '系统错误:' . $e->getMessage());
        }

        if (is_error($order)) {
            DB::rollBack();
            return $this->error(-1, $order['message']);
        }

        $orderId = $order['order_id'];

        $info = compact('orderId', 'key');

        if ($orderId) {
            //订单创建成功事件
            event(new OrderCreated($order));
            //订单事务提交
            DB::commit();
            //开启支付事务
            DB::beginTransaction();
            try {
                switch ($payType) {
                    case 'yue':
                        $this->yuePay($order, $info,$giftLogs);
                        break;
                }
            } catch (\Exception $e) {
                DB::rollBack();
                $data = [
                    'status' => 'PAY_ERROR',
                    'result' => [],
                ];
                return $this->success($data, $e->getMessage());
            }
        } else {
            DB::rollBack();
            return $this->error('订单生成失败!');
        }
    }

    /**
     * 余额支付
     * @param $orderInfo
     * @param $info
     */
    private function yuePay($orderInfo, $info,$giftLogs){
        $user = $this->userInfo;
        $totalBalance = $user['balance'] + $user['give_balance'];

        if ($totalBalance < $orderInfo['pay_price']) {
            return $this->error(-1, '余额不足');
        }
        $ret = OrderService::yuePay($orderInfo,'gift_pay');
        if (is_error($ret)) {
            DB::rollBack();
            return $this->error(-1, $ret['message']);
        }

        $newUser = ShimmerLiveshopUser::find($user->id);
        $data = [
            'status' => 'SUCCESS',
            'result' => $info,
            'total_balance' => bcadd($newUser->balance,$newUser->give_balance,2),
        ];

        $giftLogs->status=1;
        $giftLogs->pay_time=time();
        $giftLogs->save();
        DB::commit();
        return $this->success($data, '订单支付成功');
    }

    /**
     * @param Request $request
     * 排名
     */
    public function giftRank(Request $request){
        $type = $request->route('rank_type');
        $live_id = $request->route('live_id');
        switch ($type){
            case 'week':
                $rankLists = GiftService::weekRank($live_id);
                break;
            case 'total':
                $rankLists = GiftService::totalRank($live_id);
                break;
        }
        return $this->result([
            'rankLists'=>$rankLists
        ]);
    }
}