<?php
namespace App\Modules\Liveapi\Http\Controllers\Live;

use App\Models\ShimmerLiveshopAnchorApply;
use App\Models\ShimmerLiveshopGoodsAnchorCommission;
use App\Models\ShimmerLiveshopLiveGoods;
use App\Modules\Liveapi\Http\Controllers\Controller;
use App\Modules\Liveapi\Models\ShimmerLiveshopGoods;
use App\Modules\Liveapi\Models\ShimmerLiveshopLive;
use App\Modules\Liveapi\Services\Live\AnchorService;
use Illuminate\Http\Request;

class GoodsController extends Controller {

    /**
     * @param Request $request
     * 主播段 搜索商品
     */
    public function searchGoods(Request $request){
        $keywords = trim($request->input('keywords'));
        $goods = ShimmerLiveshopGoods::where('title','like','%'.$keywords.'%')->select('id','title','thumb','stock','marketprice')->get();
        return $this->result($goods);
    }

    /**
     * @param Request $request
     * 佣金商品展示
     */
    public function showCommissionGoods(Request $request){
        $keywords = trim($request->input('keywords'));
        $live_id = intval($request->route('live_id'));
        $userinfo = $this->userInfo;
        AnchorService::isAnchor($userinfo);
        $anchor = ShimmerLiveshopAnchorApply::where('uid',$userinfo->id)->first();
        $userLevel = $anchor->userLevel;

        $liveGoods = ShimmerLiveshopLiveGoods::where('live_id',$live_id)->pluck('goods_id')->toArray();

        if ($userLevel){
            $model =(new ShimmerLiveshopGoods())->setTable('goods')->from('shimmer_liveshop_goods as goods')
                ->withoutGlobalScope('uniacid')
                ->join('shimmer_liveshop_goods_anchor_commissions as anchor_commission','goods.id','=','anchor_commission.goods_id')
                ->where('goods.uniacid',Request()->route('uniacid'))
                ->where('goods.state',1)
                ->where('anchor_commission.is_commission',1)
                ->select('goods.id as id','goods.title as title','goods.thumb as thumb','goods.stock as stock','goods.marketprice as marketprice','anchor_commission.rule as rule')
                ->orderBy('goods.id','desc');

            if ($keywords){
                $model = $model->where('goods.title','like', "%{$keywords}%");
            }
            $goods = $model->paginate(ShimmerLiveshopGoods::PAGE_SIZE);
            foreach ($goods as $key=>&$good){
                $rule = json_decode($good->rule,true);
                $userRule = $rule[$userLevel->level_id];
                $good->commission = 0;
                if ($userRule && $userRule['first_level_rate']){
                    $good->commission = bcmul($good->marketprice,$userRule['first_level_rate']/100,2);
                }
                if ($userRule['first_level_amount'] && $userRule['first_level_amount']>$good->commission){
                    $good->commission = $userRule['first_level_amount'];
                }
                if (in_array($good->id,$liveGoods)){
                    $good->has_add= true;
                }else{
                    $good->has_add= false;
                }
            }
        }else{
            $goods = ShimmerLiveshopGoods::title($keywords)->where('state',1)->select('id','title','thumb','stock','marketprice')->paginate(ShimmerLiveshopGoods::PAGE_SIZE);
            foreach ($goods as $key=>&$good){
                if (in_array($good->id,$liveGoods)){
                    $good->has_add= true;
                }else{
                    $good->has_add= false;
                }
            }
        }
        return $this->result([
            'goods'=>$goods->items(),
            'limit'=>ShimmerLiveshopGoods::PAGE_SIZE
        ]);
    }

    /**
     * @param Request $request
     * 主播添加直播间商品
     */
    public function addLiveGoods(Request $request){
        $live_id = intval($request->route('live_id'));
        $goods_id = intval($request->input('goods_id'));
        $live = ShimmerLiveshopLive::find($live_id);
        if (!$live || $live->uid != $this->userInfo->id){
            return $this->result('','暂无操作权限',400);
        }

        $userinfo = $this->userInfo;
        AnchorService::isAnchor($userinfo);
        $anchor = ShimmerLiveshopAnchorApply::where('uid',$userinfo->id)->first();
        $userLevel = $anchor->userLevel;

        $liveGoods = ShimmerLiveshopLiveGoods::where('live_id',$live_id)->where('goods_id',$goods_id)->first();
        if ($liveGoods){
            return $this->result('','商品已经存在，无需重复添加',400);
        }
        $goods = ShimmerLiveshopGoods::find($goods_id);
        if (!$goods){
            return $this->result('','商品不存在',400);
        }
        $commission = 0;
        $anchorCommission = ShimmerLiveshopGoodsAnchorCommission::where('goods_id',$goods_id)->first();
        if ($anchorCommission && $anchorCommission->is_commission==1){
            $rule = $anchorCommission->rule;
            $userRule = $rule[$userLevel->level_id];

            if ($userRule && $userRule['first_level_rate']){
                $commission = bcmul($goods->marketprice,$userRule['first_level_rate']/100,2);
            }
            if ($userRule['first_level_amount'] && $userRule['first_level_amount']>$commission){
                $commission = $userRule['first_level_amount'];
            }
        }

        $model = new ShimmerLiveshopLiveGoods();
        $model->uniacid = $request->route('uniacid');
        $model->live_id = $live_id;
        $model->goods_id = $goods_id;
        $model->sort = 0;
        $model->status = 0;
        $model->save();
        return $this->result([
            'id'=>$goods->id,
            'title'=>$goods->title,
            'stock'=>$goods->stock,
            'commission'=>$commission==0 ? '暂无' : $commission,
            'status'=>0,
            'sort'=>0,
            'marketprice'=>$goods->marketprice,
            'thumb'=>wgTomedia($goods->thumb),
        ]);
    }

    /**
     * @param Request $request
     * @throws \Exception
     * 移除直播间商品
     */
    public function delLiveGoods(Request $request){
        $live_id = intval($request->route('live_id'));
        $goods_id = intval($request->input('goods_id'));
        $live = ShimmerLiveshopLive::find($live_id);
        if (!$live || $live->uid != $this->userInfo->id){
            return $this->result('','暂无操作权限',400);
        }
        $liveGoods = ShimmerLiveshopLiveGoods::where('live_id',$live_id)->where('goods_id',$goods_id)->first();
        if (!$liveGoods){
            return $this->result('','商品不存在',400);
        }
        $liveGoods->delete();
        return $this->result();
    }

    /**
     * @param Request $request
     * 置顶商品
     */
    public function changTop(Request $request){
        $live_id = intval($request->route('live_id'));
        $goods_id = intval($request->input('goods_id'));
        $sort = intval($request->input('sort'));
        $live = ShimmerLiveshopLive::find($live_id);
        if (!$live || $live->uid != $this->userInfo->id){
            return $this->result('','暂无操作权限',400);
        }
        $liveGoods = ShimmerLiveshopLiveGoods::where('live_id',$live_id)->where('goods_id',$goods_id)->first();
        if (!$liveGoods){
            return $this->result('','直播间不存在该商品',400);
        }
        $goods = ShimmerLiveshopGoods::find($goods_id);
        if (!$goods){
            return $this->result('','商品不存在',400);
        }
        $liveGoods->sort=$sort;
        $liveGoods->save();
        return $this->result();
    }

    /**
     * @param Request $request
     * 曝光商品
     */
    public function exposure(Request $request){
        $live_id = intval($request->route('live_id'));
        $goods_id = intval($request->input('goods_id'));
        $status = intval($request->input('status'));
        $live = ShimmerLiveshopLive::find($live_id);
        if (!$live || $live->uid != $this->userInfo->id){
            return $this->result('','暂无操作权限',400);
        }
        $liveGoods = ShimmerLiveshopLiveGoods::where('live_id',$live_id)->where('goods_id',$goods_id)->first();
        if (!$liveGoods){
            return $this->result('','直播间不存在该商品',400);
        }
        $goods = ShimmerLiveshopGoods::find($goods_id);
        if (!$goods){
            return $this->result('','商品不存在',400);
        }
        $liveGoods->status=$status;
        $liveGoods->save();
        return $this->result();
    }

    /**
     * @param Request $request
     * 直播间商品
     */
    public function getAnchorLiveGoods(Request $request){
        $live_id = $request->route('live_id');
        $userinfo = $this->userInfo;
        $anchor = ShimmerLiveshopAnchorApply::where('uid',$userinfo->id)->first();
        $userLevel = $anchor->userLevel;
        $lists =(new ShimmerLiveshopLiveGoods())->setTable('lg')->from('shimmer_liveshop_live_goods as lg')
            ->withoutGlobalScope('uniacid')
            ->leftJoin('shimmer_liveshop_goods as g','lg.goods_id','=','g.id')
            ->leftJoin('shimmer_liveshop_goods_anchor_commissions as anchor_commission','g.id','=','anchor_commission.goods_id')
            ->where('lg.uniacid',Request()->route('uniacid'))
            ->where('lg.live_id',$live_id)
            ->where('g.deleted_at',null)
            ->select('g.id as id','lg.sort as sort','lg.status as status','g.thumb as thumb','g.stock as stock','g.title as title','g.marketprice as marketprice','anchor_commission.rule as rule')
            ->orderBy('lg.sort','desc')
            ->orderBy('lg.id','desc')
            ->paginate(ShimmerLiveshopLiveGoods::PAGE_SIZE);
        foreach ($lists as &$list){
            $rule = json_decode($list->rule,true);
            $userRule = $rule[$userLevel->level_id];
            $list->commission = 0;
            if ($userRule && $userRule['first_level_rate']){
                $list->commission = bcmul($list->marketprice,$userRule['first_level_rate']/100,2);
            }
            if ($userRule['first_level_amount'] && $userRule['first_level_amount']>$list->commission){
                $list->commission = $userRule['first_level_amount'];
            }
            if (!$list->commission){
                $list->commission = '暂无';
            }
            $list->thumb = wgTomedia($list->thumb);
        }
        return $this->result([
            'goods'=>$lists->items(),
            'limit'=>ShimmerLiveshopLiveGoods::PAGE_SIZE
        ]);
    }

    /**
     * @param Request $request
     * 直播间商品
     */
    public function getLiveGoods(Request $request){
        $live_id = $request->route('live_id');
        $userinfo = $this->userInfo;
        $anchor = ShimmerLiveshopAnchorApply::where('uid',$userinfo->id)->first();
        $userLevel = $anchor->userLevel;
        $lists =(new ShimmerLiveshopLiveGoods())->setTable('lg')->from('shimmer_liveshop_live_goods as lg')
            ->withoutGlobalScope('uniacid')
            ->leftJoin('shimmer_liveshop_goods as g','lg.goods_id','=','g.id')
            ->where('lg.uniacid',Request()->route('uniacid'))
            ->where('lg.live_id',$live_id)
            ->where('g.deleted_at',null)
            ->select('g.id as id','lg.sort as sort','lg.status as status','g.thumb as thumb','g.stock as stock','g.title as title','g.marketprice as marketprice')
            ->orderBy('lg.sort','desc')
            ->orderBy('lg.id','desc')
            ->paginate(ShimmerLiveshopLiveGoods::PAGE_SIZE);
        foreach ($lists as &$list){
            $list->commission = \App\Services\Goods\GoodsService::getGoodsCommission($list['id'],$list['marketprice']);
            $list->thumb = wgTomedia($list->thumb);
        }
        return $this->result([
            'goods'=>$lists->items(),
            'limit'=>ShimmerLiveshopLiveGoods::PAGE_SIZE
        ]);
    }

    /**
     * @param Request $request
     * 主播中心商品列表（用户端）
     */
    public function getAnchorCenterGoods(Request $request){
        $live_id = $request->route('live_id');
        $goods_ids = ShimmerLiveshopLiveGoods::where('live_id',$live_id)->pluck('goods_id')->toArray();
        $goods = ShimmerLiveshopGoods::whereIn('id',$goods_ids)->select('id','title','thumb','sale_num','marketprice','productprice')->paginate(ShimmerLiveshopGoods::PAGE_SIZE);
        return $this->result([
            'goods'=>$goods->items(),
            'limit'=>ShimmerLiveshopGoods::PAGE_SIZE
        ]);
    }
}