<?php

/**
 * 用户余额/积分
 * Created by PhpStorm.
 * User: lirui
 * Date: 2020/2/07
 * Time: 下午8:42
 */

namespace App\Modules\Liveapi\Http\Controllers\Member;


use App\Events\OrderCreated;
use App\Models\ShimmerLiveshopCreditLog;
use App\Modules\Liveapi\Models\ShimmerLiveshopConfig;
use App\Modules\Liveapi\Http\Controllers\Controller;
use App\Modules\Liveapi\Models\ShimmerLiveshopCashApply;
use App\Modules\Liveapi\Models\ShimmerLiveshopStoreOrder;
use App\Modules\Liveapi\Services\Wechat\WechatService;
use App\Services\AppService;
use App\Services\BalanceService;
use App\Services\Commission\CashApplyService;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class CreditController extends Controller
{
    /**
     * 获取用户积分/余额账单
     * @param Request $request
     */
    public function bill(Request $request)
    {
        $logType = (int)$request->input('log_type');
        $type = (int)$request->input('type');
        $type = $type ?: 1;
        $pm = (int)$request->input('pm');

        $limit = (int)$request->input('limit');
        $builder = ShimmerLiveshopCreditLog::
        where('uid', $this->uid)
            ->where('type', $type);

        if ($logType) {
            $builder->where('log_type', $logType);
        }
        if ($pm == 1) {
            $builder->where('num', '>', 0);
        }
        if ($pm == 2) {
            $builder->where('num', '<', 0);
        }

        $builder->orderBy('id', 'desc');
        $list = $builder->paginate($limit)->items();

        foreach ($list as $key => $item) {
            $createdAt = $item['created_at']->format('Y-m-d H:i');
            $list[$key]['created'] = $createdAt;
            $list[$key]['title'] = $item->logTypeText;
        }

        return $this->success($list);
    }

    /**
     * 用户充值余额
     * @param Request $request
     */
    public function rechargeBalance(Request $request)
    {
        DB::beginTransaction();

        $money = (float)$request->input('money');
        $type = (float)$request->input('type');

        $config = ShimmerLiveshopConfig::getByType(13);

        $balanceConfig = $config['balance'];

        if (!$balanceConfig['recharge']) {
            return $this->error(-1, '系统暂未开启线上充值');
        }
        if ($money <= 0) {
            return $this->error(-1, '充值金额错误！');
        }
        if ($balanceConfig['min_recharge'] && $money < $balanceConfig['min_recharge']) {
            return $this->error(-1, "最小充值{$balanceConfig['min_recharge']}元");
        }

        if ($type == 1) {
            $order = BalanceService::createRechargeOrder($this->userInfo, $money);
        }

        if (is_error($order)) {
            DB::rollBack();
            return $this->error(-1, $order['message']);
        }

        $orderId = $order['order_id'];

        $info = compact('orderId', 'key');

        if ($orderId) {
            //订单创建成功事件
            event(new OrderCreated($order));
            try {
                $orderInfo = ShimmerLiveshopStoreOrder::getByOrderId($orderId);
                if (!$orderInfo) {
                    DB::rollBack();
                    return $this->error(-1, '支付订单不存在!');
                }
                if ($orderInfo['paid']) {
                    DB::rollBack();
                    return $this->error(-1, '订单已支付!');
                }

                $jsConfig = WechatService::jsPay($orderId, '余额充值', 'recharge_balance'); //创建订单jspay
                $info['jsConfig'] = $jsConfig;
                $data = $info;
                DB::commit();
                return $this->success($data, '订单创建成功');
            } catch (\Exception $e) {
                DB::rollBack();
                return $this->error(-1, $e->getMessage());
            }
        } else {
            DB::rollBack();
            return $this->error(-1, '订单生成失败2!');
        }
    }

    /**
     * 用户余额提现申请
     * @param Request $request
     */
    public function extractBalance(Request $request)
    {
        DB::beginTransaction();

        $userInfo = $this->userInfo;
        $extractType = $request->input('extract_type');//提现方式
        $money = $request->input('money');
        $config = ShimmerLiveshopConfig::getByType(13);
        $configBalance = $config['balance'];
        $name = trim($request->input('name'));

        $exists = ShimmerLiveshopCashApply::where('uid', $userInfo['id'])->category(2)->status(0)->first();

        if ($exists) {
            return $this->error(-1, "提现申请审核中，请勿重复提交");
        }

        //参数判断
        $commission = max(0, $configBalance['withdraw_commission']);

        if ($configBalance['withdraw_limit'] > 0
            && $money < $configBalance['withdraw_limit']
        ) {
            return $this->error(-1, "最低提现{$configBalance['withdraw_limit']}元");
        }

        if ($userInfo['balance'] < $money) {
            return $this->error(-1, "余额不足");
        }

        if ($configBalance['withdraw_commission'] > 0) {
            $commission = $money * ($configBalance['withdraw_commission'] / 100);
        }

        //是否手续费在免额区间
        if ($configBalance['withdraw_commission_free_max'] > 0
            && $money >= $configBalance['withdraw_commission_free_min']
            && $money <= $configBalance['withdraw_commission_free_max']
        ) {
            $commission = 0;
        }

        if ($commission > 0 && ($userInfo['balance'] - $money) <= $commission) {
            $money = $money - $commission;
        }

        if ($money <= 0) {
            return $this->error(-1, "提现金额错误");
        }

        $model = new ShimmerLiveshopCashApply();
        $model->uniacid = AppService::$uniacid;
        $model->type = $extractType;
        $model->category = 2;
        $model->commission = $commission;
        $model->uid = $userInfo['id'];
        $model->openid = $userInfo['wxapp_openid'];
        $model->balance = $money;

        //表单判断
        if ($extractType == 'bank') {
            $cardnum = trim($request->input('cardnum'));
            $bankaddress = trim($request->input('bankaddress'));
            $bankname = trim($request->input('bankname'));
            if (!$name) {
                return $this->error(-1, "请填写持卡人姓名");
            }
            if (!$cardnum) {
                return $this->error(-1, "请填写银行卡号");
            }
            if (!$bankname) {
                return $this->error(-1, "请选择银行");
            }
            $model->name = $name;
            $model->cardnum = $cardnum;
            $model->bankname = $bankname;
            $model->bank_address = $bankaddress;
        } else if ($extractType == 'weixin') {
            $weixin = trim($request->input('weixin'));
            if (!$weixin) {
                return $this->error(-1, "请填写微信号");
            }
            $model->weixin = $weixin;
        } else if ($extractType == 'alipay') {
            $alipayCode = trim($request->input('alipay_code'));
            if (!$alipayCode) {
                return $this->error(-1, "请填写支付宝账号");
            }
            $model->alipay_code = $alipayCode;
        }

        $ret = $model->saveOrFail();

        if ($ret) {
            CashApplyService::extractBalanceSuccess($model);
            DB::commit();
            return $this->success('', '申请成功');
        } else {
            DB::rollBack();
            return $this->error(-1, "申请失败，系统错误");
        }
    }
}