<?php

/**
 * 商品
 * Created by PhpStorm.
 * User: lirui
 * Date: 2019/9/9
 * Time: 下午4:41
 */

namespace App\Modules\Liveapi\Http\Controllers\Member;

use App\Models\ShimmerLiveshopAnchorApply;
use App\Models\ShimmerLiveshopConfig;
use App\Models\ShimmerLiveshopShop;
use App\Modules\Liveapi\Http\Controllers\Controller;
use App\Modules\Liveapi\Models\ShimmerLiveshopAgent;
use App\Modules\Liveapi\Services\Util\ExamineService;
use App\Services\ConfigService;
use App\Services\PluginService;
use Illuminate\Http\Request;
use Shimmer_liveshop_plugin_merch\Models\ShimmerLiveshopMerchConfig;
use Shimmer_liveshop_plugin_merch\Models\ShimmerLiveshopMerchStore;
use Shimmer_liveshop_plugin_merch\Models\ShimmerLiveshopMerchStoreApply;

class MenuController extends Controller
{
    public function userMenus()
    {
        $menus = [
            '我的服务' => [
                [
                    'id' => '139',
                    'name' => '地址信息',
                    'pic' => wgUrl('/static/wxapp/user/icon/location.png'),
                    'url' => '/membercenter/user_address_list/index',
                    'wap_url' => '/user/add_manage',
                ],
                [
                    'id' => '140',
                    'name' => '我的收藏',
                    'pic' => wgUrl('/static/wxapp/user/icon/collect.png'),
                    'url' => '/membercenter/user_goods_collection/index',
                    'wap_url' => '/collection',
                ],
                [
                    'id' => '142',
                    'name' => '联系客服',
                    'pic' => wgUrl('/static/wxapp/user/icon/service.png'),
                    'url' => '/pages/service/index',
                    'wap_url' => '/customer/list',
                ],
                [
                    'id' => '143',
                    'name' => '实名认证',
                    'pic' => wgUrl('/static/wxapp/user/icon/certification.png'),
                    'url' => '/membercenter/certification/index',
                    'wap_url' => '/customer/list',
                ]
            ]
        ];

        //多商户
        $menus = $this->getMerchMenus($menus);

        //分销商菜单
        $agentMenus = $this->getAgentMenus();
        if ($agentMenus) {
            $menus['分销商'] = $agentMenus;
        }

        //主播菜单
        $examine = ExamineService::getConfig();

        $anchorMenus = $this->getAnchorMenus();
        if ($anchorMenus && !empty($anchorMenus)) {
            $menus[$examine['keywords_live_room']] = $anchorMenus;
        }

        $menus = $this->formatMenu($menus);

        return $this->result(['routine_my_menus' => $menus]);
    }

    /**
     * 获取店铺菜单
     */
    private function getMerchMenus($menus)
    {
        if (!PluginService::isEnabled('merch')) {
            return $menus;
        }

        $baseConfig = ShimmerLiveshopMerchConfig::getByType('base');

        if ($baseConfig && $baseConfig['enable'] == 0) {
            return $menus;
        }

        //查看是否有店铺
        $store = ShimmerLiveshopMerchStore::uid($this->uid)
            ->first();

        if ($store) {
            $menus['我的服务'][] = [
                'name' => '我的店铺',
                'pic' => wgUrl('/static/wxapp/user/icon/tool4.png'),
                'url' => '/merch/pages/store/center/index',
                'wap_url' => '/customer/list',
            ];
        } else {
            //是否开启前台申请
            $intoConfig = ShimmerLiveshopMerchConfig::getByType('into');
            if ($intoConfig && $intoConfig['open_apply'] == 0) {
                return $menus;
            }

            $menus['我的服务'][] = [
                'name' => '我要开店',
                'pic' => wgUrl('/static/wxapp/user/icon/tool4.png'),
                'url' => '/merch/pages/apply/index',
                'wap_url' => '/customer/list',
            ];
        }

        return $menus;
    }

    /**
     * 格式化菜单数据
     * @param $menus
     */
    private function formatMenu($menus)
    {
        if (!$menus) {
            return $menus;
        }

        foreach ($menus as $key => $menu) {
            $menus[$key] = array_chunk($menu, 4);
        }

        return $menus;
    }

    /**
     * @return array
     * 主播菜单
     */
    private function getAnchorMenus()
    {
        $result = [];

        $examine = ExamineService::getConfig();

        $anchor = ShimmerLiveshopAnchorApply::where('uid', Request()->userInfo['id'])->where('status', 1)->first();
        if (!$anchor) {
            $config = ShimmerLiveshopConfig::type(14)->where('uniacid', Request()->route('uniacid'))->first();
            if ($config->config['apply_open'] == 1) {
                $result[] = [
                    'id' => '143',
                    'name' => "{$examine['keywords_live_anchor']}套餐",
                    'pic' => wgUrl('/static/wxapp/user/icon/live.png'),
                    'url' => '/live/pages/anchor-apply/index',
                ];
            }
        } else {
            $result[] = [
                'id' => '144',
                'name' => "开始{$examine['keywords_live']}",
                'pic' => wgUrl('/static/wxapp/user/icon/live.png'),
                'url' => '/live/pages/live_add/index',
            ];
            $result[] = [
                'id' => '145',
                'name' => "邀请{$examine['keywords_live_anchor']}",
                'pic' => wgUrl('/static/wxapp/user/icon/live.png'),
                'url' => '/live/pages/anchor-inviation/index',
            ];
            $result[] = [
                'id' => '145',
                'name' => "{$examine['keywords_live']}预告",
                'pic' => wgUrl('/static/wxapp/user/icon/live.png'),
                'url' => '/live/pages/anchor-advance-notice/index',
            ];
            $result[] = [
                'id' => '145',
                'name' => "{$examine['keywords_live']}商品",
                'pic' => wgUrl('/static/wxapp/user/icon/live.png'),
                'url' => '/live/pages/anchor-goods/index',
            ];
            $result[] = [
                'id' => '143',
                'name' => "{$examine['keywords_live_anchor']}套餐",
                'pic' => wgUrl('/static/wxapp/user/icon/live.png'),
                'url' => '/live/pages/anchor-apply/index',
            ];
        }
        return $result;
    }

    /**
     * @return array
     * 分销商菜单
     */
    private function getAgentMenus()
    {
        $result = [];
        $agent = ShimmerLiveshopAgent::where('uid', Request()->userInfo['id'])->where('status', 1)->first();
        if (!$agent) {
            $config = ConfigService::getValueByType(11);
            if ($config['level'] > 0) {
                $result[] = [
                    'id' => '137',
                    'name' => '分销商申请',
                    'pic' => wgUrl('/static/wxapp/user/icon/commission.png'),
                    'url' => '/membercenter/commission-apply/index',
                ];
            }
        } else {
            $result[] = [
                'id' => '138',
                'name' => '我的推广',
                'pic' => wgUrl('/static/wxapp/user/icon/commission.png'),
                'url' => '/membercenter/user_spread_user/index',
            ];
        }

        return $result;
    }
}