<?php

/**
 * 商品相关接口
 * Created by PhpStorm.
 * User: lirui
 * Date: 2019/11/11
 * Time: 下午4:41
 */

namespace App\Modules\Liveapi\Http\Controllers\Shop;

use App\Lib\Bemrc\UtilService;
use App\Models\ShimmerLiveshopGoods;
use App\Models\ShimmerLiveshopGoodsCategory;
use App\Models\ShimmerLiveshopGoodsRelation;
use App\Models\ShimmerLiveshopGoodsReply;
use App\Models\ShimmerLiveshopTag;
use App\Modules\Liveapi\Http\Controllers\Controller;
use App\Modules\Liveapi\Services\AttachmentService;
use App\Modules\Liveapi\Services\GoodsService;
use App\Modules\Liveapi\Services\PageService;
use App\Modules\Liveapi\Services\Spread\PosterService;
use App\Modules\Liveapi\Services\Wechat\WechatService;
use App\Services\GlobalRoleService;
use App\Services\LideshowsService;
use App\Services\PluginService;
use App\Services\TagService;
use Illuminate\Http\Request;
use We7\Table\Store\Goods;

class GoodsController extends Controller
{
    /**
     * 活动商品列表. 精选商品 | 促销商品
     */
    public function activityList(Request $request)
    {
        $data = [];

        $type = $request->route('atype');
        $slideType = $type + 1;

        $data['banners'] = LideshowsService::getPageBanners($slideType);

        $data['list'] = GoodsService::getActiviyGoodsList($type);

        return $this->success($data);
    }

    /**
     * 标签商品
     */
    public function tagList(Request $request)
    {
        $data = [];
        $tagId = $request->route('tag');

        $data['banners'] = TagService::getTagBanners($tagId);
        $data['tag_info'] = ShimmerLiveshopTag::find($tagId);
        $data['list'] = TagService::getTagRelations(1, $tagId);
        $data['list'] = PageService::formatGoodsData($data['list']);

        return $this->success($data);
    }

    /**
     * 搜索商品列表
     */
    public function getList(Request $request)
    {
        $page = $request->input('page');
        $limit = $request->input('limit');
        $keyword = $request->input('keyword');
        $sid = (int)$request->input('sid');
        $storeId = (int)$request->input('store_id');
        $sort = $request->input('sort');
        $news = $request->input('news');
        $priceOrder = $request->input('priceOrder');
        $salesOrder = $request->input('salesOrder');

        $query = [];
        $sort = [];
        if ($keyword) {
            $query['keyword'] = $keyword;
        }
        if ($sid) {
            $query['sid'] = $sid;
        }
        if ($storeId) {
            $query['store_id'] = $storeId;
        }
        if ($news) {
            $query['news'] = $news;
        }
        if ($priceOrder) {
            $sort['priceOrder'] = $priceOrder;
        }
        if ($salesOrder) {
            $sort['salesOrder'] = $salesOrder;
        }

        $data = GoodsService::getGoodsList($query, $sort, $page, $limit);

        return $this->success($data);
    }

    /**
     * 热门商品列表
     */
    public function hotList(Request $request)
    {
        $page = $request->input('page');
        $limit = $request->input('limit');

        $data = GoodsService::getHotGoodsList($page, $limit);

        return $this->success($data);
    }

    /**
     * 商品分类列表
     * @param Request $request
     */
    public function category(Request $request)
    {
        $pid = (int)$request->input('pid', 0);

        $data = GoodsService::getCategoryList($pid);

        return $this->success($data);
    }

    /**
     * 获取商品详情
     * @param Request $request
     */
    public function detail(Request $request)
    {
        $data = [];
        $id = $request->route('id');

        $goodsInfo = ShimmerLiveshopGoods::getValidGoodsById($id);
        $user = $request->userInfo;
        $uid = $data['uid'] = (int)$user['id'];

        if (!$goodsInfo) {
            return $this->error(-1, '商品不存在或已下架');
        }

        //增加浏览量
        $goodsInfo->increment('browse_num');

        $goodsInfo = GoodsService::formatToCrmeb($goodsInfo);

        $goodsInfo['userCollect'] = ShimmerLiveshopGoodsRelation::hasRelation($id, $uid, 'collect');
        $goodsInfo['userLike'] = ShimmerLiveshopGoodsRelation::hasRelation($id, $uid, 'like');

        //更新商品访问量
        GoodsService::setView($uid, $id, $goodsInfo['category_id'], 'view');

        //商品规格属性
        list($goodsAttr, $goodsValue) = GoodsService::getAttrDetail($id);
        $data['productAttr'] = $goodsAttr;
        $data['productValue'] = $goodsValue;

        //获取评论
        $data['reply'] = ShimmerLiveshopGoodsReply::getRecGoodsReply($id);
        $data['replyCount'] = ShimmerLiveshopGoodsReply::getGoodsReplyNum($id);
        $data['priceName'] = 0;

        //获取好评率
        if ($data['replyCount']) {
            $goodReply = ShimmerLiveshopGoodsReply::getGoodsScoreNum($id, 5);
            $data['replyChance'] = $goodReply;
            if ($goodReply) {
                $data['replyChance'] = bcdiv($goodReply, $data['replyCount'], 2);
                $data['replyChance'] = bcmul($data['replyChance'], 100, 2);
            }
        } else {
            $data['replyChance'] = 0;
        }

        $data['mer_id'] = $goodsInfo['mer_id'];
        $data['storeInfo'] = $goodsInfo;

        if (PluginService::isEnabled('merch')) {
            $role = GlobalRoleService::getRole($goodsInfo['role_type'], $goodsInfo['role_id']);
            if ($role) {
                $data['shopInfo'] = $role->formatData();
            }
        }

        return $this->success($data);
    }

    /**
     * 获取产品分销二维码
     */
    public function qrcode(Request $request)
    {
        $id = $request->route('id');
        $uid = (int)$request->userInfo['id'];

        $name = $id . '_' . $uid . '_product.jpg';
        if (!$id || !$goodsInfo = ShimmerLiveshopGoods::getValidGoodsById($id)
        ) {
            return $this->error(-1, '商品不存在或已下架');
        }

        $wxappQrcode = AttachmentService::getInfo($name);
        $path = 'goods/pages/goods_details/index';
        $params = [
            'id' => $id,
            'spid'=>$request->userInfo['id'],
            'pid'=>$request->userInfo['id']
        ];
        if (!$wxappQrcode || !file_exists($wxappQrcode->compress)) {
            $res = WechatService::getCode($params, $name, $path);
            $qrcode = $res->compress;
        } else {
            $qrcode = $wxappQrcode->compress;
        }

        $data['code'] = $qrcode;

        return $this->success($data);
    }

    /**
     * 商品收藏
     */
    public function addCollect(Request $request)
    {
        list($id, $category) = UtilService::postMore([['id', 0], ['category', 'goods']], $request, true);

        if (!$id || !is_numeric($id)) {
            return $this->error('参数错误');
        }

        $uid = $request->userInfo['id'];

        $res = ShimmerLiveshopGoodsRelation::goodsRelation($id, $uid, 'collect', $category);

        return $this->success();
    }

    /**
     * 商品取消收藏
     */
    public function delCollect(Request $request)
    {
        list($id, $category) = UtilService::postMore([['id', 0], ['category', 'goods']], $request, true);

        if (!$id || !is_numeric($id)) {
            return $this->error('参数错误');
        }

        $uid = $request->userInfo['id'];

        $res = ShimmerLiveshopGoodsRelation::ungoodsRelation($id, $uid, 'collect', $category);

        return $this->success();
    }

    /**
     * 商品批量收藏
     */
    public function collectAll(Request $request)
    {
        $goodsIds = $request->input('id');
        $category = $request->input('category', 'goods');

        if (!$goodsIds) {
            return $this->error(-1, '参数错误');
        }

        $res = ShimmerLiveshopGoodsRelation::goodsRelationAll($goodsIds, $this->uid, 'collect', $category);

        if (is_error($res)) {
            return $this->error('-1', $res['message']);
        }

        return $this->success('', '收藏成功');
    }

    /**
     * 商品评论列表
     */
    public function replyList(Request $request)
    {
        $goodsId = $request->route('goods_id');

        list($page, $limit, $type) = UtilService::getMore([
            ['page', 0], ['limit', 0], ['type', 0]
        ], $request, true);

        if (!$goodsId) {
            return $this->error(-1, '参数错误');
        }

        $data = GoodsService::getGoodsReplyList($goodsId, (int)$type, $page, $limit);

        return $this->success($data);
    }

    /**
     * 商品评论统计
     */
    public function replyConfig(Request $request)
    {
        $goodsId = $request->route('goods_id');

        if (!$goodsId) {
            return $this->error(-1, '参数错误');
        }

        $data = GoodsService::goodsReplyStat($goodsId);

        return $this->success($data);
    }

}