<?php

/**
 * 商品
 * Created by PhpStorm.
 * User: lirui
 * Date: 2019/9/9
 * Time: 下午4:41
 */

namespace App\Modules\Liveapi\Http\Controllers\Spread;
use App\Events\CommissionAgent;
use App\Models\ShimmerLiveshopCommissionOrder;
use App\Modules\Live\Models\ShimmerLiveshopSpreadPoster;
use App\Modules\Liveapi\Http\Controllers\Controller;
use App\Modules\Liveapi\Models\ShimmerLiveshopAgent;
use App\Modules\Liveapi\Models\ShimmerLiveshopAgentApply;
use App\Modules\Liveapi\Models\ShimmerLiveshopStoreOrder;
use App\Modules\Liveapi\Models\ShimmerLiveshopUser;
use App\Modules\Liveapi\Models\ShimmerLiveshopUserRelation;
use App\Modules\Liveapi\Services\AttachmentService;
use App\Modules\Liveapi\Services\Spread\PosterService;
use App\Modules\Liveapi\Services\User\UserRelationService;
use App\Modules\Liveapi\Services\Wechat\WechatService;
use App\Services\ConfigService;
use Illuminate\Http\Request;

class UserinfoController extends Controller
{
    public function apply(Request $request){
        $config = ConfigService::getValueByType(11);
        if ($config['become_reg']==1){
            if (!$this->userInfo->mobile || !$this->userInfo->realname){
                return $this->result('','请完善资料',400);
            }
        }
        event(new CommissionAgent(['uid'=>$request->userInfo['id'],'type'=>'apply']));
        return $this->result('','申请成功');
    }

    /**
     * @param Request $request
     * 获取分销商申请界面信息
     */
    public function getInfo(Request $request){
        $siteinfo = ConfigService::getValueByType(1);
        $config = ConfigService::getValueByType(11);

        $already= 0;
        $need= 0;
        if ($config['become']==3){
            $model = ShimmerLiveshopStoreOrder::where('uid',$this->userInfo->id)->where('paid',1);
            if ($config['become3_order']==0){
                $already = $model->count();
            }else{
                $already = $model->where('refund_status',0)->count();
            }
        }

        if ($config['become']==4){
            $model = ShimmerLiveshopStoreOrder::where('uid',$this->userInfo->id)->where('paid',1);
            if ($config['become4_order']==0){
                $already = $model->sum('pay_price');
            }else{
                $already = $model->where('refund_status',0)->sum('pay_price');
            }
        }
        $banner = wgTomedia($config['poster']);
        $parent = "总店";
        if ($request->input('spid')){
            $shareUser = ShimmerLiveshopUser::find(intval($request->input('spid')));
            if ($shareUser){
                $parent = $shareUser->nickname;
            }
        }
        $agent = ShimmerLiveshopAgent::where('uid',$request->userInfo['id'])->first();
        return $this->result([
            'banner'=>$banner,
            'parent'=>$parent,
            'agent'=>$agent,
            'already'=>$already,
            'need'=>$need,
            'config'=>$config,
            'userInfo'=>$this->userInfo,
            'shopname'=>$siteinfo['site_name'],
        ]);
    }
    /**
     * @param Request $request
     * 我的推广
     */
    public function commission(Request $request){
        $user = $request->userInfo;

        //佣金总额
        $commission['total'] = ShimmerLiveshopCommissionOrder::where('agent_uid',$user->id)->whereIn('status',[0,2,1])->sum('commission');
        //可提现佣金
        $commission['can_cash'] = ShimmerLiveshopCommissionOrder::where('agent_uid',$user->id)->where('status',2)->where('has_withdraw',0)->sum('commission');

        //累计订单数
        $commission['order_count'] = ShimmerLiveshopCommissionOrder::where('agent_uid',$user->id)->whereIn('status',[2,1])->count();

        //待结算佣金
        $commission['wait_commission'] = ShimmerLiveshopCommissionOrder::where('agent_uid',$user->id)->where('status',1)->sum('commission');

        //累计客户数
        $child_ids = UserRelationService::getChildIds($this->userInfo);

        $commission['user_count'] = count((array)$child_ids['first_child_ids']) + intval(count((array)$child_ids['sec_child_ids'])) + intval(count((array)$child_ids['thr_child_ids']));


        $todayCondition = ShimmerLiveshopCommissionOrder::where('agent_uid',$user->id)->where('created_at','>=',strtotime(date('Y-m-d'.'00:00:00',time())));

        //今日收益
        $commission['today_commission'] = $todayCondition->whereIn('status',[2,1])->sum('commission');
        //今日订单
        $commission['today_order_count'] = $todayCondition->whereIn('status',[2,1])->count();
        //今日新增客户
        $commission['today_user_count'] = $todayCondition->whereIn('status',[2,1])->distinct('uid')->count('uid');

        return $this->result([
            'user'=>$user,
            'commission'=>$commission
        ],'ok');
    }

    /**
     * @param Request $request
     * 分销海报
     */
    public function picture(Request $request){
        $user = $request->userInfo;
        $pics = ShimmerLiveshopSpreadPoster::where('status',1)->get();
        if (!$pics){
            return $this->result([],'暂无海报',400);
        }

        $name = $user->id.'_'.'_user_share_code.png';
        $wxappQrcode = AttachmentService::getInfo($name);

        if (!$wxappQrcode || !file_exists($wxappQrcode->compress)){
            $res = WechatService::getCode(['spid'=>$user->id],$name);
            $qrcode = $res->compress;
        }else{
            $qrcode = $wxappQrcode->compress;
        }

        $filepath ='shimmer_liveshop/shareCode/poster';

        if (!is_dir(ATTACHMENT_ROOT.$filepath)) {
            mkdir(ATTACHMENT_ROOT.$filepath, 0777, true);
        }

        foreach ($pics as $key=>&$item){
            if ($item['local_poster']){
                $item['poster'] = PosterService::getSharePoster($item['local_poster'],$qrcode,$filepath);
            }else{
                $item['poster'] = PosterService::getSharePoster($item['poster'],$qrcode,$filepath);
            }

        }
        return $this->result($pics,'ok');
    }
}