<?php
namespace App\Modules\Liveapi\Services\Live;

use App\Models\ShimmerLiveshopAnchorApply;
use App\Models\ShimmerLiveshopCommentConfig;
use App\Models\ShimmerLiveshopLiveViewer;
use App\Modules\Liveapi\Models\ShimmerLiveshopComment;
use App\Modules\Liveapi\Services\BaseService;
use App\Services\Project\Wxapp\HandleService;
use App\Services\ConfigService;

class CommentService extends BaseService {

    public static $commentConfig = [];

    /**
     * @param $live_id
     * @return \Illuminate\Database\Eloquent\Model|null|static
     * 评论配置
     */
    public static function getCommentConfig($live_id)
    {
        static $commentConfig;
        if (is_null($commentConfig)) {
            $commentConfig = ShimmerLiveshopCommentConfig::where('live_id',$live_id)->first();
        }
        return $commentConfig;
    }

    public static function getLiveAllComments($live_id){
        $comments = ShimmerLiveshopComment::where('live_id',$live_id)->where('checked',1)->select('id','user_id','nickname','avatar','content','content_type')->get();
        return $comments;
    }

    /**
     * @param $live_id
     * @return \Illuminate\Database\Eloquent\Collection|static[]
     * 直播间评论前几条
     */
    public static function getLiveComments($live_id){
        $config = ConfigService::getValueByType(17);
        if (intval($config['chat_nums'])==0) return [];
        $comments = ShimmerLiveshopComment::where('live_id',$live_id)->where('checked',1)->select('id','user_id','nickname','avatar','content','content_type')->get(intval($config['chat_nums']));
        return $comments;
    }

    /**
     * @param $live_id
     * 判断是否能评论
     */
    public static function canSpeaking($live_id,$content_type,$content){
        if (!$live_id){
            return ajaxResult('','无效的直播间',400);
        }
        $commentConfig = self::getCommentConfig($live_id);
        if ($commentConfig->is_notalk==1){
            return ajaxResult('','直播间禁止评论',400);
        }
        if ($commentConfig->is_no_sendimg==1 && $content_type==2){
            return ajaxResult('','直播间禁止发送图片',400);
        }
        $viewer = ShimmerLiveshopLiveViewer::where('live_id',$live_id)->where('uid',Request()->userInfo['id'])->first();
        if (!$viewer){
            $anchor = ShimmerLiveshopAnchorApply::where('uid',Request()->userInfo['id'])->where('status',1)->where('is_black',0)->first();
            if (!$anchor){
                return ajaxResult('','你还未进入直播间，无法评论',400);
            }
        }
        if ($viewer->is_shut==1){
            return ajaxResult('','你已被禁言',400);
        }
        if ($viewer->is_black==1){
            return ajaxResult('','你已被拉黑',400);
        }
        $syConfig = ConfigService::getValueByType(17);
        if ($syConfig['open_keyword_filter']!=1){
            $res = HandleService::smsCheck($content);
            if ($res['errcode']!=0){
                return ajaxResult('','内容不合法',400);
            }
        }
    }

    /**
     * @param $live_id
     * @return int
     * 获取评论状态
     */
    public static function getCommentStatus($live_id){
        $commentConfig = self::getCommentConfig($live_id);
        return $commentConfig->is_check==0 ? 1 : 0;
    }

    /**
     * @param $content
     * @return array
     * 评论信息过滤
     */
    public static function getContent($content){
        return $content;
    }
}