<?php
/**
 * 小程序service
 * Created by PhpStorm.
 * User: lirui
 * Date: 2019/11/16
 * Time: 下午7:27
 */

namespace App\Modules\Liveapi\Services;

use Illuminate\Support\Facades\Cache;
use App\Models\ShimmerLiveshopStoreOrderGroup;
use App\Modules\Liveapi\Models\ShimmerLiveshopConfig;
use App\Models\ShimmerLiveshopPayResponseData;
use App\Models\ShimmerLiveshopStoreOrder;
use App\Services\AppService;
use EasyWeChat\Factory;
use EasyWeChat\Kernel\Support\XML;
use Illuminate\Support\Facades\Log;
use App\Models\AccountWxapp;

class MiniProgramService extends \App\Services\MiniProgramService
{
    private static $instances = null;

    /**
     * 获取实例
     * @param $appType
     */
    public static function getApp($appType)
    {
        if (isset(self::$instances[$appType])) {
            return self::$instances[$appType];
        }

        $config = self::getConfig($appType);
        $app = Factory::$appType($config);
        self::$instances[$appType] = $app;

        return $app;
    }

    /**
     * 获取参数配置
     * @param $appType
     * @return array
     */
    private static function getConfig($appType)
    {
        $uniacid = AppService::$uniacid;
        $wxapp = AccountWxapp::where('uniacid', $uniacid)->first()->toArray();
        $config = [
            'response_type' => 'array',
//            'log' => [
//                'level' => 'debug',
//                'file' => storage_path('logs/wechat.log'),
//            ],
        ];

        switch ($appType) {
            case 'miniProgram':
                $config['app_id'] = $wxapp['key'];
                $config['secret'] = $wxapp['secret'];
                break;
            case 'payment':
                $payment = ShimmerLiveshopConfig::getByType(9);
                $accountType = $payment['account_type'];
                $notifyUrl = route('liveapiwechat.notify');
                $config['mch_id'] = trim($payment['mch_id']);

                if ($accountType == 1) {
                    $config['sub_mch_id'] = trim($payment['sub_mch_id']);
                    $config['sub_appid'] = $wxapp['key'];
                    $config['app_id'] = trim($payment['appid']);
                } else {
                    $config['app_id'] = $wxapp['key'];
                }

                $config['key'] = trim($payment['apikey']);
                $config['cert_path'] = storage_path('/app/' . $payment['cert_file']);
                $config['key_path'] = storage_path('/app/' . $payment['key_file']);
                $config['notify_url'] = $notifyUrl;

                break;
        }

        return $config;
    }

    /**
     * 小程序实例
     * @return \EasyWeChat\MiniProgram\MiniProgram
     */
    public static function miniprogram()
    {
        return self::getApp('miniProgram');
    }

    /**
     * 支付实例
     * @return mixed
     */
    public static function payment()
    {
        return self::getApp('payment');
    }

    /**
     * @return mixed
     * 获取access_token
     */
    public static function getAccessToken($new = false)
    {
        $wxappAccessToken = 'wxapp_access_token_' . AppService::$uniacid;
        if ($new || !Cache::has($wxappAccessToken)) {
            $app = self::miniprogram();
            $accessToken = $app->access_token;
            $token = $accessToken->getToken();
            Cache::put($wxappAccessToken, $token['access_token'], $token['expires_in'] - 200);
            return $token['access_token'];
        }
        return Cache::get($wxappAccessToken);
    }

    /**
     * 小程序支付回调
     */
    public static function handleNotify()
    {
        $request = request();
        $response = $request->getContent();
        $response = XML::parse($response);

//        $response = '{"appid":"wxd056d92701e08bb4","attach":"product","bank_type":"OTHERS","cash_fee":"10","fee_type":"CNY","is_subscribe":"N","mch_id":"1493717222","nonce_str":"5deb9df1b51bc","openid":"oavRH4_8A3WWNeqRMm1k52BmJpzU","out_trade_no":"wx157572248171111461","result_code":"SUCCESS","return_code":"SUCCESS","sign":"69C8CBC6E6FC685D732FF554420EC35E","time_end":"20191207204136","total_fee":"10","trade_type":"JSAPI","transaction_id":"4200000447201912075733470948"}';
//        $response = json_decode($response,true);

        $orderId = $response['out_trade_no'];

        if (strpos($response['attach'], 'group') !== false) {
            $order = ShimmerLiveshopStoreOrderGroup::getByOrderId($orderId);
        } else {
            $order = ShimmerLiveshopStoreOrder::getByOrderId($orderId);
        }


        if (!$order) {
            Log::error("回调订单数据异常:{$response}");
        }

        $uniacid = $order->uniacid;
        AppService::setUniacid($uniacid);
        $request->route()->setParameter('uniacid', $uniacid);
        AppService::setUniacid($uniacid);

        //回调日志记录
        ShimmerLiveshopPayResponseData::record($orderId, $response['attach'], $response, $uniacid);

        $app = MiniProgramService::payment();

        $response = $app->handlePaidNotify(function ($message, $fail) use ($order) {
            // 使用通知里的 "微信支付订单号" 或者 "商户订单号" 去自己的数据库找到订单
            if (!$order || $order->paid) { // 如果订单不存在 或者 订单已经支付过了
                return true; // 告诉微信，我已经处理完了，订单没找到，别再通知我了
            }

            ///////////// <- 建议在这里调用微信的【订单查询】接口查一下该笔订单的情况，确认是已经支付 /////////////

            if ($message['return_code'] === 'SUCCESS') { // return_code 表示通信状态，不代表支付状态
                // 用户是否支付成功
                if (array_get($message, 'result_code') === 'SUCCESS') {

                    $isGroup = is_array($order->order_ids);

                    $transactionId = (string)$message['transaction_id'];
                    OrderService::paySuccess($order->order_id, 'weixin', $message['attach'], $transactionId, $isGroup);

                    // 用户支付失败
                } elseif (array_get($message, 'result_code') === 'FAIL') {
                    $order->notify_fail = array_get($message, 'return_msg');
                    $order->save(); // 保存订单
                }
            } else {
                return $fail('通信失败，请稍后再通知我');
            }

            return true; // 返回处理完成
        });

        $response->send(); // return $response;
    }

    /**
     * 获取小程序默认导航
     * @return array
     */
    public static function getDefaultTabbarConfig()
    {
        $baseUrl = wgUrl('/static/wxapp/common/');
        $home = rand(100000, 999999) . 'bar';
        $person = rand(100000, 999999) . 'bar';
        $live = rand(100000, 999999) . 'bar';
        $cart = rand(100000, 999999) . 'bar';
        $near = rand(100000, 999999) . 'bar';

        $config = [
            'config' => [
                'color' => '#434343',
                'select_color' => '#ff0077',
            ],
            'list' => [
                $home => [
                    'id' => $home,
                    'name' => '首页',
                    'icon' => $baseUrl . 'home.png',
                    'select_icon' => $baseUrl . 'select_home.png',
                    'link' => '/common/pages/index/index',
                    'isshow' => 1,
                    'sort' => 6,
                ],
                $live => [
                    'id' => $live,
                    'name' => '直播',
                    'icon' => $baseUrl . 'live.png',
                    'select_icon' => $baseUrl . 'select_live.png',
                    'link' => '/live/pages/live_list1/index',
                    'isshow' => 1,
                    'sort' => 3,
                ],
                $near => [
                    'id' => $near,
                    'name' => '附近',
                    'icon' => $baseUrl . 'live.png',
                    'select_icon' => $baseUrl . 'select_live.png',
                    'link' => '/live/pages/live_list2/index',
                    'isshow' => 1,
                    'sort' => 5,
                ],
                $cart => [
                    'id' => $cart,
                    'name' => '购物车',
                    'icon' => $baseUrl . 'cart.png',
                    'select_icon' => $baseUrl . 'select_cart.png',
                    'link' => '/order/pages/order_addcart/order_addcart',
                    'isshow' => 1,
                    'sort' => 4,
                ],
                $person => [
                    'id' => $person,
                    'name' => '个人中心',
                    'icon' => $baseUrl . 'person.png',
                    'select_icon' => $baseUrl . 'select_person.png',
                    'link' => '/common/pages/user/user',
                    'isshow' => 1,
                    'sort' => 1,
                ],
            ],
        ];
        return $config;
    }
}