<?php

/**
 * 订单
 * Created by PhpStorm.
 * User: lirui
 * Date: 2019/11/15
 * Time: 下午5:43
 */

namespace App\Modules\Liveapi\Services;


use App\Events\CommissionAgent;
use App\Events\OrderCommented;
use App\Events\OrderCreated;
use App\Events\OrderDelivery;
use App\Events\OrderPaySuccess;
use App\Events\OrderRemoved;
use App\Events\OrderTakeed;
use App\Models\ShimmerLiveshopGoodsReply;
use App\Models\ShimmerLiveshopLiveCame;
use App\Models\ShimmerLiveshopStoreOrder;
use App\Models\ShimmerLiveshopStoreOrderCartInfo;
use App\Models\ShimmerLiveshopStoreOrderChangeLog;
use App\Models\ShimmerLiveshopStoreOrderGroup;
use App\Models\ShimmerLiveshopUserBill;
use App\Modules\Live\Services\LimitService;
use App\Modules\Liveapi\Models\ShimmerLiveshopConfig;
use App\Modules\Liveapi\Models\ShimmerLiveshopGoods;
use App\Modules\Liveapi\Models\ShimmerLiveshopOrder;
use App\Modules\Liveapi\Models\ShimmerLiveshopOrderGood;
use App\Modules\Liveapi\Models\ShimmerLiveshopUser;
use App\Services\AppService;
use App\Services\BalanceService;
use App\Services\GlobalRoleService;
use App\Services\Live\LiveCameService;
use App\Services\PluginService;
use App\Services\TemplateCustomerService;
use App\Services\TemplateService;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Request;

class OrderService extends \App\Services\OrderService
{
    protected static $payType = ['weixin' => '微信支付', 'yue' => '余额支付', 'offline' => '线下支付'];

    protected static $deliveryType = ['send' => '商家配送', 'express' => '快递配送'];

    /**
     * 获取订单组商品价格
     * @param $cartInfo
     */
    public static function getOrderGroupPrice($cartInfo)
    {
        $expressConfig = ShimmerLiveshopConfig::getByType(10);

        $storePostage = floatval($expressConfig['store_postage']) ?: 0;//邮费基础价
        $storeFreePostage = floatval($expressConfig['store_free_postage']) ?: 0;//满额包邮
        $totalPrice = self::getOrderSumPrice($cartInfo, 'truePrice');//获取订单总金额
        $costPrice = self::getOrderSumPrice($cartInfo, 'costPrice');//获取订单成本价
        $vipPrice = self::getOrderSumPrice($cartInfo, 'vip_truePrice');//获取订单会员优惠金额

        //如果满额包邮等于0
        if ($storeFreePostage && $totalPrice >= $storeFreePostage
            && $cartInfo[0]['role_id'] == 0
        ) {
            $storePostage = 0;
        } else {
            foreach ($cartInfo as $cart) {
                if (!$cart['productInfo']['is_postage']) {
                    //若果产品不包邮
                    $storePostage = bcadd($storePostage, $cart['productInfo']['postage'], 2);
                }
            }
        }

        return compact('storePostage', 'storeFreePostage', 'totalPrice', 'costPrice', 'vipPrice');
    }

    /**
     *
     * 设置多商户价格
     * @param $cartInfo
     */
    public static function getMerchOrderGroupPrice($cartInfo)
    {
        $totalPrice = 0;
        $storePostage = 0;
        $costPrice = 0;
        foreach ($cartInfo as $storeId => $group) {
            $cartInfo[$storeId]['priceGroup'] = self::getOrderGroupPrice($group['goods']);

            foreach ($group['goods'] as $cart) {

            }

            if ($storeId == 0) {
                //如果满额包邮等于0
                if ($cartInfo[$storeId]['priceGroup']['storeFreePostage'] && $cartInfo[$storeId]['priceGroup']['totalPrice'] >= $cartInfo[$storeId]['priceGroup']['storeFreePostage']) {
                    $cartInfo[$storeId]['priceGroup']['storePostage'] = 0;
                }
            }

            $totalPrice += $cartInfo[$storeId]['priceGroup']['totalPrice'];
            $storePostage += $cartInfo[$storeId]['priceGroup']['storePostage'];
            $costPrice += $cartInfo[$storeId]['priceGroup']['costPrice'];
        }

        return compact('cartInfo', 'totalPrice', 'storePostage', 'costPrice', 'vipPrice');
    }

    /**
     * 计算购物车商品总金额
     * @param $cartInfo
     * @param $key 键名
     * @return int|string
     */
    public static function getOrderSumPrice($cartInfo, $key = 'truePrice')
    {
        $result = 0;
        foreach ($cartInfo as $cart) {
            $result = bcadd($result, bcmul($cart['cart_num'], $cart[$key], 2), 2);
        }
        return $result;
    }

    /**
     * 缓存用户订单信息
     * @param $uid
     * @param $cartInfo
     * @param $priceGroup
     * @param array $other
     * @param int $cacheTime
     * @return string
     */
    public static function cacheOrderInfo($uid, $cartInfo, $priceGroup, $other = [], $cacheTime = 600)
    {
        $key = md5(time() . mt_rand(1, 1000000));
        Cache::put('user_order_' . $uid . '_' . $key, compact('cartInfo', 'priceGroup', 'other'), $cacheTime);
        return $key;
    }

    /**
     * 获取缓存的用户订单信息
     * @param $uid
     * @param $key
     * @return null
     */
    public static function getCacheOrderInfo($uid, $key)
    {
        $cacheName = 'user_order_' . $uid . '_' . $key;
        if (!Cache::has($cacheName)) {
            return null;
        }
        return Cache::get($cacheName);
    }

    /**
     * 清理订单缓存信息
     * @param $uid
     * @param $key
     */
    public static function clearCacheOrderInfo($uid, $key)
    {
        Cache::forget('user_order_' . $uid . $key);
    }

    /**
     * @return ShimmerLiveshopStoreOrder|mixed
     * @throws \Throwable
     * 创建主播申请订单
     */
    public static function createdFictitiousOrder($uid, $key, $payType, $price, $type, $relation_type = 0, $relation_id = 0)
    {
        if (!array_key_exists($payType, self::$payType)) {
            return self::error(-1, '支付方式有误!');
        }

        if (!array_key_exists($type, ShimmerLiveshopStoreOrder::$type)) {
            return self::error(-1, '订单类型错误!');
        }

        $exisit = self::userOrderExists($key, $uid);

        if ($exisit) {
            return self::error(-1, '请勿重复提交订单!');
        }

        $userInfo = ShimmerLiveshopUser::find($uid);
        if (!$userInfo) {
            return self::error(-1, '用户不存在!');
        }

        $orderInfo = [
            'uniacid' => Request::route('uniacid'),
            'uid' => $uid,
            'type' => $type,
            'order_id' => self::getNewOrderId(),
            'real_name' => $userInfo->realname,
            'user_phone' => $userInfo->mobile,
            'user_address' => '-',
            'cart_id' => 0,
            'total_num' => 1,
            'total_price' => $price,
            'total_postage' => 0,
            'coupon_id' => 0,
            'coupon_price' => 0,
            'pay_price' => $price,
            'pay_postage' => 0,
            'deduction_price' => 0,
            'paid' => 0,
            'pay_type' => $payType,
            'use_integral' => 0,
            'gain_integral' => 0,
            'mark' => '',
            'combination_id' => 0,
            'pink_id' => 0,
            'seckill_id' => 0,
            'bargain_id' => 0,
            'cost' => $price,
            'is_channel' => 0,
            'add_time' => time(),
            'unique' => $key,
            'relation_id' => $relation_id,
            'relation_type' => $relation_type,
        ];

        $order = new ShimmerLiveshopStoreOrder();
        $saveRet = $order->fill($orderInfo)->saveOrFail();

        if (!$saveRet) {
            return self::error(-1, '订单生成失败!');
        }

        OrderService::log($order['id'], 'cache_key_create_order', '订单生成');

        return $order;
    }


    /**
     * 生成订单
     * @param $uid
     * @param $key
     * @param $addressId
     * @param $payType
     * @param bool $useIntegral
     * @param int $couponId
     * @param string $mark
     * @param int $combinationId
     * @param int $pinkId
     * @param int $seckill_id
     * @param int $bargain_id
     * @param int $isChannel
     */

    public static function cacheKeyCreateOrder($uid, $key, $addressId, $payType, $useIntegral = false, $couponId = 0, $mark = '', $combinationId = 0, $pinkId = 0, $seckill_id = 0, $bargain_id = 0, $isChannel = 0)
    {
        //检测创建订单
        $checkData = self::checkCacheKeyCreateOrder($uid, $key, $addressId, $payType);
        if (is_error($checkData)) {
            return self::error($checkData['errno'], $checkData['message']);
        }

        //根据店铺商品拆单
        if (PluginService::isEnabled('merch')) {
            if (count($checkData['cartGroup']['cartInfo']) > 1) {
                //生成店铺订单
                return self::cacheKeyCreateOrderMerch($uid, $key, $checkData['address'], $payType, $checkData['cartGroup'], $mark, $isChannel);
            } else {
                //生成普通订单
                $cartInfo = $checkData['cartGroup']['cartInfo'];
                foreach ($cartInfo as $storeId => $group) {
                    $mark = isset($mark[$storeId]) ? trim($mark[$storeId]) : '';
                    $cartGroup = [
                        'cartInfo' => $group['goods'],
                        'priceGroup' => $group['priceGroup'],
                    ];
                    $roleType = 0;
                    if ($storeId > 0) {
                        $roleType = GlobalRoleService::MERCH;
                    }
                    return self::cacheKeyCreateOrderNomal($uid, $key, $checkData['address'], $payType, $cartGroup, $mark, $isChannel, $roleType, $storeId);
                }
            }
        } else {
            //生成普通订单
            return self::cacheKeyCreateOrderNomal($uid, $key, $checkData['address'], $payType, $checkData['cartGroup'], $mark, $isChannel);
        }
    }

    /**
     * 创建多商户订单
     * @param $uid
     * @param $key
     * @param $address
     * @param $payType
     * @param $cartGroup
     * @param $mark
     * @param $isChannel
     * @return ShimmerLiveshopStoreOrder|mixed
     * @throws \Throwable
     */
    public static function cacheKeyCreateOrderMerch($uid, $key, $address, $payType, $cartGroup, $mark, $isChannel)
    {
        $cartInfo = $cartGroup['cartInfo'];
        $priceGroup = $cartGroup['priceGroup'];
        $payPrice = (float)$priceGroup['totalPrice'];
        $payPostage = (float)$priceGroup['storePostage'];

        $orders = collect();
        foreach ($cartInfo as $storeId => $group) {
            $mark = isset($mark[$storeId]) ? trim($mark[$storeId]) : '';
            $cartGroup = [
                'cartInfo' => $group['goods'],
                'priceGroup' => $group['priceGroup'],
            ];

            //生成新子订单号
            $orderKey = md5($uid . time() . mt_rand(1, 1000000));

            $roleType = 0;
            if ($storeId > 0) {
                $roleType = GlobalRoleService::MERCH;
            }

            $order = self::cacheKeyCreateOrderNomal($uid, $orderKey, $address, $payType, $cartGroup, $mark, $isChannel, $roleType, $storeId);

            if (is_error($order)) {
                return self::error($order['errno'], $order['message']);
            }
            $orders->push($order);
        }

        //生成订单组
        $orderIds = $orders->pluck('id')->toArray();
        $payPrice = (float)bcadd($payPrice, $payPostage, 2);
        $payPrice = max(0, $payPrice);

        $orderGroup = new ShimmerLiveshopStoreOrderGroup();
        $orderGroupData = [
            'uniacid' => AppService::$uniacid,
            'uid' => $uid,
            'unique' => $key,
            'order_id' => self::getNewOrderId(),
            'order_ids' => $orderIds,
            'pay_price' => $payPrice,
            'pay_type' => $payType,
        ];
        $ret = $orderGroup->fill($orderGroupData)->saveOrFail();

        if (!$ret) {
            return self::error(-1, '订单组创建失败');
        }

        self::clearCacheOrderInfo($uid, $key);

        $orderGroup->orders = $orders;
        return $orderGroup;
    }

    /**
     * 创建普通商品订单
     * @param $uid
     * @param $key
     * @param $address
     * @param $payType
     * @param $cartGroup
     * @param $mark
     * @param $isChannel
     * @param $roleType
     * @param $roleId
     * @return ShimmerLiveshopStoreOrder|mixed
     * @throws \Throwable
     */
    public static function cacheKeyCreateOrderNomal($uid, $key, $address, $payType, $cartGroup, $mark, $isChannel, $roleType = 0, $roleId = 0)
    {
        $cartInfo = $cartGroup['cartInfo'];
        $priceGroup = $cartGroup['priceGroup'];
        $payPrice = (float)$priceGroup['totalPrice'];
        $payPostage = $priceGroup['storePostage'];
        $cartIds = [];
        $totalNum = 0;
        $gainIntegral = 0;
        $couponPrice = 0;
        $couponId = 0;
        $deductionPrice = 0;
        $usedIntegral = 0;
        $combinationId = 0;
        $pinkId = 0;
        $seckillId = 0;
        $bargainId = 0;

        foreach ($cartInfo as $cart) {
            $cartIds[] = $cart['id'];
            $totalNum += $cart['cart_num'];
        }

        //是否包邮
        $payPrice = (float)bcadd($payPrice, $payPostage, 2);
        $payPrice = max(0, $payPrice);

        $orderInfo = [
            'uniacid' => AppService::$uniacid,
            'uid' => $uid,
            'type' => 1,
            'order_id' => self::getNewOrderId(),
            'real_name' => $address['realname'],
            'user_phone' => $address['mobile'],
            'user_address' => $address['province'] . ' ' . $address['city'] . ' ' . $address['district'] . ' ' . $address['address_info'],
            'cart_id' => $cartIds,
            'total_num' => $totalNum,
            'total_price' => $priceGroup['totalPrice'],
            'total_postage' => $priceGroup['storePostage'],
            'coupon_id' => $couponId,
            'coupon_price' => $couponPrice,
            'pay_price' => $payPrice,
            'pay_postage' => $payPostage,
            'deduction_price' => $deductionPrice,
            'paid' => 0,
            'pay_type' => $payType,
            'use_integral' => $usedIntegral,
            'gain_integral' => $gainIntegral,
            'mark' => htmlspecialchars($mark),
            'combination_id' => $combinationId,
            'pink_id' => $pinkId,
            'seckill_id' => $seckillId,
            'bargain_id' => $bargainId,
            'cost' => $priceGroup['costPrice'],
            'is_channel' => $isChannel,
            'add_time' => time(),
            'unique' => $key,
            'role_type' => $roleType,
            'role_id' => $roleId,
        ];

        $order = new ShimmerLiveshopStoreOrder();
        $saveRet = $order->fill($orderInfo)->saveOrFail();

        if (!$saveRet) {
            return self::error(-1, '订单生成失败!');
        }


        foreach ($cartInfo as $cart) {
            //减库存加销量
            $res5 = GoodsService::decGoodsStock($cart['cart_num'], $cart['productInfo']['id'], isset($cart['productInfo']['attrInfo']) ? $cart['productInfo']['attrInfo']['unique'] : '');

            if (!$res5) {
                return self::error(-1, '减库存失败!');
            }

        }

        //保存购物车商品信息
        $res4 = false !== self::saveOrderCartInfo($order['id'], $cartInfo);
        if (!$res4) {
            return self::error(-1, '添加订单商品失败!');
        }

        //购物车状态修改
        $res6 = false !== CartService::updateIsPay($cartIds);
        if (!$res6) {
            return self::error(-1, '修改购物车状态失败!');
        }

        //设置默认地址
        (new AddressService())->setDefalutAddress($address->id);

        self::clearCacheOrderInfo($uid, $key);

        OrderService::log($order['id'], 'cache_key_create_order', '订单生成');

        event(new OrderCreated($order));
        return $order;
    }

    /**
     * 检测创建订单
     * @param $uid
     * @param $key
     * @param $addressId
     * @param $payType
     * @return array|mixed
     */
    public static function checkCacheKeyCreateOrder($uid, $key, $addressId, $payType)
    {
        $offlinePayStatus = 2;
        if ($offlinePayStatus == 2) {
            unset(self::$payType['offline']);
        }

        if (!array_key_exists($payType, self::$payType)) {
            return self::error(-1, '支付方式有误!');
        }

        $exisit = self::userOrderExists($key, $uid);

        if ($exisit) {
            return self::error(-1, '请勿重复提交订单!');
        }

        $userInfo = ShimmerLiveshopUser::find($uid);
        if (!$userInfo) {
            return self::error(-1, '用户不存在!');
        }

        $cartGroup = self::getCacheOrderInfo($uid, $key);

        if (!$cartGroup) {
            return self::error(-1, '订单已过期,请重新下单！');
        }

        $address = AddressService::getUserAddressById($addressId, $uid);

        if (!$address) {
            return self::error(-1, '收货地址有误!');
        }

        return compact('userInfo', 'cartGroup', 'address');
    }

    /**
     * 判断用户订单是否生成
     */
    public static function userOrderExists($key, $uid)
    {
        return ShimmerLiveshopStoreOrder::where('unique', $key)
                ->orWhere('order_id', $key)
                ->where('uid', $uid)
                ->count() > 0;
    }

    /**
     * 保存订单商品表
     * @param $orderId
     * @param $cartInfo
     */
    public static function saveOrderCartInfo($orderId, $cartInfo)
    {
        $data = [];
        foreach ($cartInfo as $cart) {
            $data[] = [
                'uniacid' => AppService::$uniacid,
                'oid' => $orderId,
                'cart_id' => $cart['id'],
                'relation_type' => $cart['relation_type'],
                'relation_id' => $cart['relation_id'],
                'goods_id' => $cart['productInfo']['id'],
                'cart_info' => json_encode($cart),
                'unique' => md5($cart['id'] . '' . $orderId)
            ];
        }

        return ShimmerLiveshopStoreOrderCartInfo::insert($data);
    }

    /**
     * 微信支付 为 0元时
     * @param $orderId
     * @param $uid
     * @param $isGroup 是否是组合订单
     * @return bool
     */
    public static function jsPayPrice($orderId, $uid, $attach = 'product', $isGroup = false)
    {
        if ($isGroup) {
            $orderInfo = ShimmerLiveshopStoreOrderGroup::getByOrderId($orderId);
        } else {
            $orderInfo = ShimmerLiveshopStoreOrder::getByOrderId($orderId);
        }

        if (!$orderInfo) {
            return self::error(-1, '订单不存在!');
        }
        if ($orderInfo['uid'] != $uid) {
            return self::error(-1, '非法订单!');
        }
        if ($orderInfo['paid']) {
            return self::error(-1, '订单已支付!');
        }

        $userInfo = ShimmerLiveshopUser::find($uid);
        if (!$userInfo) {
            return self::error(-1, '购买用户已被删除!');
        }

        $res = self::paySuccess($orderId, 'weixin', $attach, '', $isGroup);

        return $res;
    }

    /**
     * 支付成功后处理
     * @param $orderId 订单id
     * @param $paytype 支付类型
     * @param $attach 支付附属参数
     * @param $transactionId 支付交易id
     * @param $isGroup 是否是组合订单
     * @return bool
     */
    public static function paySuccess($orderId, $paytype = 'weixin', $attach = 'product', $transactionId = '', $isGroup = false)
    {
        if ($isGroup) {
            $order = ShimmerLiveshopStoreOrderGroup::getByOrderId($orderId);
        } else {
            $order = ShimmerLiveshopStoreOrder::getByOrderId($orderId);
        }

        if (!$order) {
            exception('订单号不存在');
        }
        if ($order['paid']) {
            exception('订单已支付');
        }

        if ($isGroup) {
            $orders = $order->orders();
            foreach ($orders as $orderItem) {
                $attach = str_replace('_group', '', $attach);
                self::paySuccessOrder($orderItem, $paytype, $attach, $transactionId);
                //组合订单
                $orderItem->order_group_id = $order->id;
                $orderItem->save();
            }
            //更新订单状态
            $data = [
                'paid' => 1,
                'transaction_id' => (string)$transactionId,
                'pay_type' => $paytype,
                'pay_time' => time()
            ];
            $ret = $order->fill($data)->save();
            if (!$ret) {
                exception('组合订单支付状态更新失败' + $order['id']);
            }
        } else {
            self::paySuccessOrder($order, $paytype, $attach, $transactionId);
        }
    }

    /**
     * 订单支付成功调用
     * @param $order
     * @param string $paytype
     * @param string $attach
     * @param string $transactionId
     * @throws \Exception
     */
    public static function paySuccessOrder($order, $paytype = 'weixin', $attach = 'product', $transactionId = '')
    {
        //更新订单状态
        $data = [
            'paid' => 1,
            'transaction_id' => (string)$transactionId,
            'pay_type' => $paytype,
            'pay_time' => time()
        ];

        $ret = $order->fill($data)->save();

        if (!$ret) {
            exception('订单支付状态更新失败' + $order['id']);
        }

        //支付成功事件
        event(new OrderPaySuccess($order, $attach, $paytype));
        event(new CommissionAgent(['uid' => $order->uid]));
    }

    /**
     * 订单详情数据格式化
     * @param $order
     * @param bool $detail 是否包含商品数据
     * @param bool $isPic 是否获取产品状态图片
     * @return mixed
     */
    public static function tidyOrder($order, $detail = false, $isPic = false)
    {
        if ($detail && $order['id']) {
            $cartInfos = self::getCartInfosByOid($order['id']);

            $carts = [];
            foreach ($cartInfos as $k => $cartInfo) {
                $cart = $cartInfo->cart_info;
                $cart['unique'] = $cartInfo->unique;
                //是否评价字段
                $cart['is_reply'] = ShimmerLiveshopGoodsReply::getOrderGoodsReplayCount($order['id'], $cartInfo['unique']);
                $carts[] = $cart;
            }
            $order['cartInfo'] = $carts;
        }

        $status = [];
        $shopConfig = ShimmerLiveshopConfig::getByType(13);
        $baseConfig = ShimmerLiveshopConfig::getByType(1);
        $createTime = $order['created_at']->timestamp;

        if (!$order['paid'] && $order['pay_type'] == 'offline' && !$order['status'] >= 2) {
            $status['_type'] = 9;
            $status['_title'] = '线下付款';
            $status['_msg'] = '等待等待商家处理';
            $status['_class'] = 'nobuy';
        } else if (!$order['paid']) {
            $status['_type'] = 0;
            $status['_title'] = '等待买家付款';
            //系统预设取消订单时间段
            //获取配置
            $time = bcadd($createTime, (int)$shopConfig['order_cancel_time'] * 3600, 0);
            $status['_msg'] = '请在' . date('m-d H:i:s', $time) . '前付款，超时订单将自动关闭';
            $status['_class'] = 'nobuy';
        } else if ($order['refund_status'] == 1) {
            $status['_type'] = -1;
            $status['_title'] = '申请退款中';
            $status['_msg'] = '商家正在努力审核，请耐心等待';
            $status['_class'] = 'state-sqtk';
        } else if ($order['refund_status'] == 2) {
            $status['_type'] = -2;
            $status['_title'] = '商家已退款';
            $status['_msg'] = '系统已为您退款，给您带来不便非常抱歉';
            $status['_class'] = 'state-sqtk';
        } else if (!$order['status']) {
            $status['_type'] = 1;
            $status['_title'] = '等待商家发货';
            $status['_msg'] = '';
            $status['_class'] = 'state-nfh';
        } else if ($order['status'] == 1) {
            if ($order['delivery_type'] == 'send') {//TODO 送货
                $status['_type'] = 2;
                $status['_title'] = '待收货';
                $status['_msg'] = date('m月d日H时i分', ShimmerLiveshopStoreOrderChangeLog::getTime($order['id'], 'delivery')) . '服务商已送货';
                $status['_class'] = 'state-ysh';
            } else {//TODO  发货
                $status['_type'] = 2;
                $status['_title'] = '待收货';
                $status['_msg'] = date('m月d日H时i分', ShimmerLiveshopStoreOrderChangeLog::getTime($order['id'], 'delivery_goods')) . '商家已发货，请耐心等待';
                $status['_class'] = 'state-ysh';
            }
        } else if ($order['status'] == 2) {
            $status['_type'] = 3;
            $status['_title'] = '待评价';
            $status['_msg'] = '已收货,快去评价一下吧';
            $status['_class'] = 'state-ypj';
        } else if ($order['status'] == 3) {
            $status['_type'] = 4;
            $status['_title'] = '交易完成';
            $status['_msg'] = '您的订单已完成,感谢您的支持';
            $status['_class'] = 'state-ytk';
        }

        if ($order['is_del'] == 1) {
            $status['_type'] = -99;
            $status['_title'] = '交易关闭';
            $status['_msg'] = '';
        }

        if (isset($order['pay_type'])) {
            $status['_payType'] = isset(self::$payType[$order['pay_type']]) ? self::$payType[$order['pay_type']] : '其他方式';
        }

        if (isset($order['delivery_type'])) {
            $status['_deliveryType'] = isset(self::$deliveryType[$order['delivery_type']]) ? self::$deliveryType[$order['delivery_type']] : '其他方式';
        }

        $order['_status'] = $status;

        if (isset($order['pay_time']) && $order['pay_time']) {
            $order['_pay_time'] = date('Y-m-d H:i:s', $order['pay_time']);
        } else {
            $order['_pay_time'] = $order['created_at']->format('Y-m-d H:i:s');
        }

        $order['_add_time'] = $order['created_at']->format('Y-m-d H:i:s');
        $order['status_pic'] = '';
        //获取产品状态图片
        if ($isPic) {
            $order['status_pic'] = '';
        }
        $order['offlinePayStatus'] = 2;


        //快递方式
        $deliveryType = ShimmerLiveshopStoreOrder::$deliveryTypes[$order['delivery_type']];

        if (!$deliveryType) {
            $deliveryType = '快递发货';
        }

        $order['_delivery_type'] = $deliveryType;
        $order['consumer_hotline'] = (string)$baseConfig['consumer_hotline'];

        $order['goods_price'] = $order['pay_price'] - $order['pay_postage'];

        //订单店铺
        if (PluginService::isEnabled('merch')
            && $order['role_id'] > 0
        ) {
            $order['store'] = GlobalRoleService::getRole($order['role_type'], $order['role_id']);
        }
        return $order;
    }

    /*
    * 个人中心获取个人订单列表和订单搜索
    * @param int $uid 用户uid
    * @param int | string 查找订单类型
    * @param int $first 分页
    * @param int 每页显示多少条
    * @param string $search 订单号
    * @return array
    * */
    public static function getUserOrderSearchList($uid, $type, $page, $limit, $search)
    {
        if ($search) {
            $order = \App\Modules\Liveapi\Models\ShimmerLiveshopStoreOrder::searchUserOrder($uid, $search) ?: [];
            $list = $order == false ? [] : [$order];
        } else {
            $list = \App\Modules\Liveapi\Models\ShimmerLiveshopStoreOrder::getUserOrderList($uid, $type, $page, $limit);
        }

        foreach ($list as $k => $order) {
            if ($list[$k]['_status']['_type'] == 3) {
                foreach ($order['cartInfo'] ?: [] as $key => $product) {
                    $list[$k]['cartInfo'][$key]['is_reply'] = ShimmerLiveshopGoodsReply::isReply($product['unique'], 'product');
                    $list[$k]['cartInfo'][$key]['add_time'] = isset($product['add_time']) ? date('Y-m-d H:i', $product['add_time']) : '时间错误';
                }
            }
            if (PluginService::isEnabled('merch')
                && $order->role_id > 0
            ) {
                $order->store = GlobalRoleService::getRole($order->role_type, $order->role_id);
            }
        }
        return $list;
    }

    /**
     * 取消订单
     * @param $orderId
     * @param $uid
     * @param $isSystem 是否是系统取消
     */
    public static function cancelOrder($orderId, $uid = 0, $isSystem = 0)
    {
        $order = ShimmerLiveshopStoreOrder::getByOrderId($orderId);

        if (!$order) {
            return self::error(-1, '没有查到此订单！');
        }

        $isConsole = \Illuminate\Support\Facades\App::runningInConsole();

        if ($order->uid != $uid && !$isConsole) {
            return self::error(-1, '没有查到此订单！');
        }

        DB::beginTransaction();
        try {
            $res = self::regressionStock($order);
            if ($res) {
                $order->is_del = 1;
                $order->is_system = $isSystem;
                $order->save();
                DB::commit();
                return true;
            }
        } catch (\Exception $e) {
            DB::rollBack();
            return self::error(-1, $e->getMessage());
        }
    }

    /**
     * 回退库存和销量
     * @param $order
     * @return bool
     */
    public static function regressionStock($order)
    {
        if ($order['paid'] || $order['status'] == -2) {
            return true;
        }

        $res5 = true;
        $cartInfo = ShimmerLiveshopStoreOrderCartInfo::whereIn('cart_id', $order['cart_id'])->get();

        foreach ($cartInfo as $cart) {
            //增库存减销量
            GoodsService::incGoodsStock($cart['cart_info']['cart_num'], $cart['cart_info']['productInfo']['id'], isset($cart['cart_info']['productInfo']['attrInfo']) ? $cart['cart_info']['productInfo']['attrInfo']['unique'] : '');
        }
        return $res5;
    }

    /**
     * 用户申请退款
     * @param $orderId
     * @param $uid
     * @param $type
     * @param string $refundReasonWap
     * @return bool
     */
    public static function orderApplyRefund($orderId, $uid, $type, $refundReasonWap = '', $refundReasonWapExplain = '', $refundReasonWapImg = array())
    {
        $order = ShimmerLiveshopStoreOrder::getByOrderId($orderId);

        if (!$order || $order->uid != $uid) {
            return self::error(-1, '支付订单不存在');
        }
        if ($order['refund_status'] == 2) {
            return self::error(-1, '订单已退款');
        }
        if ($order['refund_status'] == 1) {
            return self::error(-1, '该订单已在申请退款审核中');
        }
        if (!$order['paid']) {
            return self::error(-1, '订单未支付,不能申请退款!');
        }
        if (!$type) {
            return self::error(-1, '请选择退款方式');
        }
        if ($order['status'] == 1) {
        }

        DB::beginTransaction();

        $applyRefund = '用户申请退款，原因：' . $refundReasonWap;
        $res1 = self::log($order['id'], 'apply_refund', $applyRefund);

        $order->refund_type = $type;
        $order->refund_status = 1;
        $order->refund_reason_time = time();
        $order->refund_reason_wap = $refundReasonWap;
        $order->refund_reason_wap_explain = $refundReasonWapExplain;
        $order->refund_reason_wap_img = $refundReasonWapImg;
        $res2 = $order->save();

        $res = $res1 && $res2;

        if ($res) {
            DB::commit();
        } else {
            DB::rollBack();
            return self::error(-1, '申请退款失败！');
        }
        $adminIds = UserService::getServiceUids();
        if ($adminIds) {
            TemplateService::sendOrderRefundStatus($order, $refundReasonWap, $adminIds);
        }
        return true;
    }

    /**
     * 订单确认收货
     * @param $orderId
     * @param $uid
     * @param $isSystem
     */
    public static function takeOrder($orderId, $uid = 0, $isSystem = 0)
    {
        $order = ShimmerLiveshopStoreOrder::getByOrderId($orderId);

        if (!$order) {
            return self::error(-1, '订单不存在!');
        }

        $isConsole = \Illuminate\Support\Facades\App::runningInConsole();

        if ($order->uid != $uid && !$isSystem) {
            return self::error(-1, '没有查到此订单！');
        }

        $order = self::tidyOrder($order);

        if ($order['_status']['_type'] != 2) {
            return self::error(-1, '订单状态错误!');
        }

        DB::beginTransaction();

        $update = ['status' => 2, 'is_system' => $isSystem, 'finish_time' => time()];
        $ret = ShimmerLiveshopStoreOrder::where('id', $order->id)
            ->update($update);

        if ($ret) {
            self::log($order->id, 'user_take_delivery', '用户确认收货');
            try {
                event(new OrderTakeed($order));
                event(new CommissionAgent(['uid' => $order->uid]));
                DB::commit();

            } catch (\Exception $e) {
                DB::rollBack();
                return self::error(-1, '确认收货失败:' . $e->getMessage());
            }

        }
    }

    /**
     * 订单商品评价
     * @param $replyData
     */
    public static function comment($replyData)
    {
        $reply = new ShimmerLiveshopGoodsReply();
        $ret = $reply->fill($replyData)->save();

        if ($ret) {
            $uniqueList = ShimmerLiveshopStoreOrderCartInfo::select('unique')
                ->where('oid', $replyData['oid'])
                ->get();

            $uniqueList = $uniqueList->pluck('unique');

            $countReplay = ShimmerLiveshopGoodsReply::whereIn('unique', $uniqueList)->count('unique');

            //商品全部评价完成
            if ($countReplay >= $uniqueList) {
                $order = ShimmerLiveshopStoreOrder::find($replyData['oid']);
                $order->status = 3;
                $res = $order->save();

                if (!$res) {
                    return self::error(-1, '订单评价失败');
                }

                event(new OrderCommented($order));
                self::log($order->id, 'check_order_over', '用户完成评价');
            }
        } else {
            return self::error(-1, '商品评价失败！');
        }
    }

    /**
     * 用户删除订单
     * @param $orderId
     * @param $uid
     */
    public static function removeOrder($orderId, $uid)
    {
        $order = ShimmerLiveshopStoreOrder::getByOrderId($orderId);

        if (!$order || $order->uid != $uid) {
            return self::error(-1, '订单不存在!');
        }
        if ($order->is_del == 1) {
            return self::error(-1, '订单已删除!');
        }

        $order = self::tidyOrder($order);

        if ($order['_status']['_type'] != 0 && $order['_status']['_type'] != -2 && $order['_status']['_type'] != 4) {
            return self::error(-1, '该订单无法删除!');
        }

        $update = [
            'is_del' => 1,
        ];

        $ret = ShimmerLiveshopStoreOrder::where('id', $order->id)->update($update);

        if (!$ret) {
            return self::error(-1, '删除失败!');
        }

        self::log($order->id, 'remove_order', '用户删除订单');

        event(new OrderRemoved($order));
    }

    /**
     * 余额支付
     * @param $orderInfo
     * @param string $attach
     * @return mixed
     * @throws \Throwable
     */
    public static function yuePay($orderInfo, $attach = 'product')
    {
        $user = ShimmerLiveshopUser::find($orderInfo['uid']);
        $isGroup = is_array($orderInfo->order_ids);

        if (!$user) {
            return self::error(-1, '订单用户不存在');
        }

        $totalBalance = $user->balance + $user->give_balance;

        if ($totalBalance < $orderInfo['pay_price']) {
            return self::error(-1, '余额不足');
        }
        if ($orderInfo['paid']) {
            return self::error(-1, '该订单已支付');
        }

        //优先扣除赠送余额
        $useBalance = 0;
        $useGiveBalance = 0;
        if ($user->balance >= $orderInfo['pay_price']) {
            $user->balance = $user->balance - $orderInfo['pay_price'];
            $useBalance = $orderInfo['pay_price'];
        } else {
            $user->give_balance = $user->give_balance - ($orderInfo['pay_price'] - $user->balance);
            $useBalance = $user->balance;
            $useGiveBalance = $orderInfo['pay_price'] - $user->balance;
            $user->balance = 0;
        }

        $ret = $user->saveOrFail();

        if (!$ret) {
            DB::rollBack();
            return self::error(-1, '用户余额扣除失败');
        }

        //余额明细
        BalanceService::addCreditLog(50, $user->id, 1, -$useBalance, $user->balance, '', -$useGiveBalance, 0, 0, $user->give_balance, $orderInfo->id);

        try {
            OrderService::paySuccess($orderInfo['order_id'], 'yue', $attach, '', $isGroup);
        } catch (\Exception $exception) {
            DB::rollBack();
            return self::error(-1, $exception->getMessage());
        }

    }

    /**
     * 卡密商品购买成功处理
     * @param $order
     */
    public static function buyCameGoodsSuccess($order)
    {
        //如果是卡密商品直接,自动发货。自动确认收货
        $orderGoods = OrderService::getCartInfosByOid($order['id']);
        if (count($orderGoods) == 1
            && $orderGoods[0]['cart_info']['productInfo']['buymode'] == 3
        ) {
            $order->status = 1;
            $order->save();
            event(new OrderDelivery($order));

            $ret = OrderService::takeOrder($order['order_id'], $order['uid']);
            if (is_error($ret)) {
                Log::error($ret['message']);
                apiException($ret['message']);
            }

            //发放卡密
            LiveCameService::sendCame($order->uid, $order->id);
        }
    }

}