<?php

namespace App\Modules\Liveapi\Services;

use App\Models\ShimmerLiveshopGoodsCategory;
use App\Models\ShimmerLiveshopPageConfig;
use App\Modules\Liveapi\Models\ShimmerLiveshopConfig;
use App\Modules\Liveapi\Models\ShimmerLiveshopGoods;
use App\Modules\Liveapi\Services\Util\ExamineService;
use App\Services\TagService;

/**
 * 页面
 * Class PageService
 * @package App\Modules\Liveapi\Services
 */
class PageService extends BaseService
{

    /**
     * 获取首页展示分类列表
     */
    public static function getHomeConfigs()
    {
        $result = [];
        $pageType = 'home';
        $config = ShimmerLiveshopPageConfig::getByPage($pageType);
        $goodsLimit = $config['goods_limit'] ?: 5;
        $configSite = ShimmerLiveshopConfig::getByType(1);
        $examine = ExamineService::getConfig();

        $result = $config;
        $result['site_name'] = $configSite['site_name'];

        if ($config) {
            $categoryIds = [];
            if ($config['category_ids']) {
                $categoryIdsArr = json_decode($config['category_ids'], true);
                $sorts = array_column($categoryIdsArr, 'sort');

                array_multisort($sorts, SORT_DESC, $categoryIdsArr);
                $categoryIds = array_column($categoryIdsArr, 'id');
            }

            $result['categorys'] = self::formatCategoryIds($categoryIds);

            unset($result['category_ids']);

            $result['ad_pic'] = wgTomedia($result['ad_pic']);

            foreach ((array)$result['vip_banner'] as $key => $row) {
                $result['vip_banner'][$key] = wgTomedia($row);
            }

            //新闻
            $result['news'] = [];
            if ($config['news']) {
                $news = explode("\n", $config['news']);
                foreach ($news as $index => $news) {
                    $newsArr = [];
                    if (strpos($news, ':') !== false) {
                        $newsArr = explode(':', $news);
                    }
                    if (strpos($news, '：') !== false) {
                        $newsArr = explode('：', $news);
                    }
                    if ($newsArr) {
                        $result['news'][] = [
                            'tip' => trim($newsArr[0]),
                            'text' => trim($newsArr[1]),
                        ];
                    } else {
                        $result['news'][] = [
                            'tip' => '',
                            'text' => trim($news),
                        ];
                    }
                }
            }

            //顶部标语
            $result['top_titles'] = [];
            if ($config['top_titles']) {
                $result['top_titles'] = explode("\n", $config['top_titles']);
            }

            //获取精商品列表
            if ($config['is_vip']) {
                $vipGoods = ShimmerLiveshopGoods::getVipGoods($goodsLimit);
                $result['vip_goods'] = self::formatGoodsData($vipGoods);
            }
            if ($config['is_new']) {
                $newGoods = ShimmerLiveshopGoods::getNewGoods($goodsLimit);
                $result['new_goods'] = self::formatGoodsData($newGoods);
            }
            if ($config['is_sales']) {
                $saleGoods = ShimmerLiveshopGoods::getSaleGoods($goodsLimit);
                $result['sale_goods'] = self::formatGoodsData($saleGoods);
            }
            if ($config['live_open_type']
            && !$examine['switch']
            ) {
                $result['lives'] = LiveService::getLivesByType($config['live_open_type']);
            }

            //获取自定义标签商品列表
            $tags = $config['tag_ids'];
            $tags = TagService::getTagsByIds($tags);
            $result['tags'] = [];

            if ($tags) {
                foreach ($tags as $k => $tag) {
                    $list = TagService::getTagRelations(1, $tag['id'], $goodsLimit);
                    if (!$list || $list->isEmpty()) {
                        unset($tags[$k]);
                        continue;
                    }

                    $tags[$k]['list'] = self::formatGoodsData($list);
                    $tags[$k]['banners'] = TagService::getTagbanners($tag['id']);
                }
                $result['tags'] = $tags;
            }
        }

        //显示关注公众号
        $result['follow_weixin'] = (int)$config['follow_weixin'];

        return $result;
    }

    /**
     * 获取热门关键字
     */
    public static function getGoodsSearchKeywords()
    {
        $result = [];

        $pageType = 'goods_search';
        $config = ShimmerLiveshopPageConfig::getByPage($pageType);
        $keywords = $config['keywords'];

        if ($keywords) {
            if (strpos($keywords, '，') !== false) {
                $result = explode('，', $keywords);
            } else {
                $result = explode(',', $keywords);
            }
        }

        return $result;
    }

    /**
     * 格式化页面商品返回数据
     * @param $goods
     */
    public static function formatGoodsData($goods)
    {
        $result = [];
        if(!$goods){
            return $result;
        }
        foreach ($goods as $_goods) {
            $data = [
                'id' => $_goods['id'],
                'title' => $_goods['title'],
                'thumb' => wgTomedia($_goods['thumb']),
                'productprice' => ($_goods['productprice']),
                'costprice' => ($_goods['costprice']),
                'marketprice' => ($_goods['marketprice']),
                'sale_num' => $_goods['sale_num'] + $_goods['ficti_num'],
                'stock' => $_goods['stock'],
                'unit' => $_goods['unit'],
                'state' => $_goods['state'],
                'stateon_at' => $_goods['stateon_at'],//上架
                'commission' => \App\Services\Goods\GoodsService::getGoodsCommission($_goods['id'],$_goods['marketprice'])
            ];
            //销售百分比
//            if(){
//
//            }

            $result[] = $data;
        }
        return $result;
    }

    /**
     * 格式化首页展示分类数据
     * @param $categoryIds
     */
    public static function formatCategoryIds($categoryIds)
    {
        $result = [];

        if ($categoryIds) {
            $categorys = ShimmerLiveshopGoodsCategory::getCategorysByIds($categoryIds);

            foreach ($categorys as $item) {
                $result[] = [
                    'id' => $item->id,
                    'name' => $item->name,
                    'home_pic' => wgTomedia($item->home_pic),
                ];
            }
        }

        return $result;
    }

}