<?php
namespace App\Modules\Liveapi\Services\Spread;
use App\Modules\Liveapi\Models\ShimmerLiveshopUser;
use App\Modules\Liveapi\Models\ShimmerLiveshopUserRelation;
use App\Modules\Liveapi\Services\BaseService;
use Illuminate\Support\Facades\DB;

class AgentService extends BaseService {

    /**
     * @return array
     * 一级分销信息
     */
    public static function getOneLevelData(){
        $orderBy = Request()->input('sort');
        if ($orderBy){
            list($column,$sort) = explode(' ',$orderBy);
        }else{
            $column = 'id';
            $sort = 'desc';
        }

        $uids = ShimmerLiveshopUserRelation::where('parent_uid',Request()->userInfo['id'])->pluck('uid')->toArray();
        $users =(new ShimmerLiveshopUser())->setTable('ur')->from('shimmer_liveshop_users as ur')
            ->withoutGlobalScope('uniacid')
            ->leftJoin('shimmer_liveshop_agents as ag','ur.id','=','ag.uid')
            ->leftJoin('shimmer_liveshop_commission_orders as co','ag.uid','=','co.uid')
            ->where('ur.uniacid',Request()->route('uniacid'))
            ->whereIn('ur.id',$uids)
            ->select(DB::raw(DB::getTablePrefix()."ag.uid as uid,from_unixtime(".DB::getTablePrefix()."ag.created_at,'%Y/%m/%d') as time,COUNT(".DB::getTablePrefix()."co.id) as orderCount,SUM(".DB::getTablePrefix()."co.commission) as commission_total,".DB::getTablePrefix()."ur.nickname as nickname,".DB::getTablePrefix()."ur.avatar as avatar,".DB::getTablePrefix()."ur.created_at as created_at,".DB::getTablePrefix()."ag.child_count as child_count,".DB::getTablePrefix()."ag.id as id"))
            ->orderBy($column,$sort)
            ->paginate(ShimmerLiveshopUserRelation::PAGE_SIZE);
        foreach ($users->items() as &$item) {
            $item->dateline = $item->created_at->format('Y-m-d');
            $item->orderCount = intval($item->orderCount);
            $item->commission_total = intval($item->commission_total);
        }
        $secTotal = ShimmerLiveshopUserRelation::where('grandpa_uid',Request()->userInfo['id'])->count();
        $thrTotal = ShimmerLiveshopUserRelation::where('grand_parent_uid',Request()->userInfo['id'])->count();
        return ['list'=>$users->items(),'firTotal'=>$users->count(),'secTotal'=>$secTotal,'thrTotal'=>$thrTotal,'teamCount'=>$users->count()];
    }

    /**
     * @return array
     * 二级分销信息
     */
    public static function getTwoLevelData(){
        $orderBy = Request()->input('sort');
        if ($orderBy){
            list($column,$sort) = explode(' ',$orderBy);
        }else{
            $column = 'id';
            $sort = 'desc';
        }

        $uids = ShimmerLiveshopUserRelation::where('grandpa_uid',Request()->userInfo['id'])->pluck('uid')->toArray();
        $users =(new ShimmerLiveshopUser())->setTable('ur')->from('shimmer_liveshop_users as ur')
            ->withoutGlobalScope('uniacid')
            ->leftJoin('shimmer_liveshop_commission_orders as co','ur.id','=','co.uid')
            ->leftJoin('shimmer_liveshop_agents as ag','ur.id','=','ag.uid')
            ->where('ur.uniacid',Request()->route('uniacid'))
            ->whereIn('ur.id',$uids)
            ->select(DB::raw(DB::getTablePrefix()."ag.uid as uid,from_unixtime(".DB::getTablePrefix()."ag.created_at,'%Y/%m/%d') as time,COUNT(".DB::getTablePrefix()."co.id) as orderCount,SUM(".DB::getTablePrefix()."co.commission) as commission_total,".DB::getTablePrefix()."ur.nickname as nickname,".DB::getTablePrefix()."ur.avatar as avatar,".DB::getTablePrefix()."ur.created_at as created_at,".DB::getTablePrefix()."ag.child_count as child_count,".DB::getTablePrefix()."co.id as id"))
            ->orderBy($column,$sort)
            ->paginate(ShimmerLiveshopUserRelation::PAGE_SIZE);
        foreach ($users->items() as &$item) {
            $item->dateline = $item->created_at->format('Y-m-d');
            $item->orderCount = intval($item->orderCount);
            $item->commission_total = intval($item->commission_total);
        }

        $firTotal = ShimmerLiveshopUserRelation::where('parent_uid',Request()->userInfo['id'])->count();
        $thrTotal = ShimmerLiveshopUserRelation::where('grand_parent_uid',Request()->userInfo['id'])->count();
        return ['list'=>$users->items(),'firTotal'=>$firTotal,'secTotal'=>$users->count(),'thrTotal'=>$thrTotal,'teamCount'=>$users->count()];
    }

    /**
     * @return array
     * 三级分销信息
     */
    public static function getThreeLevelData(){

        $orderBy = Request()->input('sort');
        if ($orderBy){
            list($column,$sort) = explode(' ',$orderBy);
        }else{
            $column = 'id';
            $sort = 'desc';
        }

        $uids = ShimmerLiveshopUserRelation::where('grand_parent_uid',Request()->userInfo['id'])->pluck('uid')->toArray();
        $users =(new ShimmerLiveshopUser())->setTable('ur')->from('shimmer_liveshop_users as ur')
            ->withoutGlobalScope('uniacid')
            ->leftJoin('shimmer_liveshop_commission_orders as co','ur.id','=','co.uid')
            ->leftJoin('shimmer_liveshop_agents as ag','ur.id','=','ag.uid')
            ->where('ur.uniacid',Request()->route('uniacid'))
            ->whereIn('ur.id',$uids)
            ->select(DB::raw(DB::getTablePrefix()."ag.uid as uid,from_unixtime(".DB::getTablePrefix()."ag.created_at,'%Y/%m/%d') as time,COUNT(".DB::getTablePrefix()."co.id) as orderCount,SUM(".DB::getTablePrefix()."co.commission) as commission_total,".DB::getTablePrefix()."ur.nickname as nickname,".DB::getTablePrefix()."ur.avatar as avatar,".DB::getTablePrefix()."ur.created_at as created_at,".DB::getTablePrefix()."ag.child_count as child_count,".DB::getTablePrefix()."co.id as id"))
            ->orderBy($column,$sort)
            ->paginate(ShimmerLiveshopUserRelation::PAGE_SIZE);
        foreach ($users->items() as &$item) {
            $item->dateline = $item->created_at->format('Y-m-d');
            $item->orderCount = intval($item->orderCount);
            $item->commission_total = intval($item->commission_total);
        }

        $firTotal = ShimmerLiveshopUserRelation::where('parent_uid',Request()->userInfo['id'])->count();
        $secTotal = ShimmerLiveshopUserRelation::where('grandpa_uid',Request()->userInfo['id'])->count();

        return ['list'=>$users->items(),'firTotal'=>$firTotal,'secTotal'=>$secTotal,'thrTotal'=>$users->count(),'teamCount'=>$users->count()];
    }
}