<?php
/**
 * 用户前台菜单
 * Created by PhpStorm.
 * User: lirui
 * Date: 2020/2/12
 * Time: 上午10:43
 */

namespace App\Modules\Liveapi\Services;

use App\Models\ShimmerLiveshopAnchorApply;
use App\Modules\Liveapi\Models\ShimmerLiveshopConfig;
use App\Modules\Liveapi\Services\Live\AnchorService;
use App\Modules\Liveapi\Services\Util\ExamineService;
use App\Services\PluginService;
use Shimmer_liveshop_plugin_merch\Models\ShimmerLiveshopMerchConfig;
use Shimmer_liveshop_plugin_merch\Models\ShimmerLiveshopMerchStore;
use Shimmer_liveshop_plugin_partner\Services\Partner\PartnerService;

class UserMenuService extends BaseService
{
    /**
     * 获取用户个人中心菜单
     * @param $user
     */
    public static function memberCenter($user)
    {
        $menus = collect();
        //多商户
        $menus = $menus->merge(self::getMerchMenus($user));
        //直播
        $menus = $menus->merge(self::getAnchorMenus($user));
        //合伙人
        $menus = $menus->merge(self::getPartnerMenus($user));
        //公共菜单
        $menus = $menus->merge(self::getCommonMenus($user));

        //排序分组
        $menus = $menus->groupBy('group')->sortByDesc('sort');

        return $menus;
    }

    /**
     * 获取公共菜单
     * @param $user
     */
    public static function getCommonMenus($user)
    {
        $menu = [];
        $group = '我的服务';//菜单分组
        $menu[] = [
            'name' => '收货地址',
            'type' => 'link',
            'group' => $group,
            'sort' => 100,
            'icon' => wgUrl('/static/wxapp/user/center/c66.png'),
            'wxapp_url' => '/membercenter/user_address_list/index',
            'account_url' => '',
        ];
        $menu[] = [
            'name' => '我的收藏',
            'type' => 'link',
            'group' => $group,
            'sort' => 100,
            'icon' => wgUrl('/static/wxapp/user/center/c55.png'),
            'wxapp_url' => '/membercenter/user_goods_collection/index',
            'account_url' => '',
        ];

        if (PluginService::isEnabled('merch')) {
            $menu[] = [
                'name' => '收藏店铺',
                'type' => 'link',
                'group' => $group,
                'sort' => 100,
                'icon' => wgUrl('/static/wxapp/plugin/merch/store/center/a88.png'),
                'wxapp_url' => '/merch/pages/user/collect_list/index',
                'account_url' => '',
            ];
        }

        $menu[] = [
            'name' => '联系客服',
            'type' => 'contact',
            'group' => $group,
            'sort' => 100,
            'icon' => wgUrl('/static/wxapp/user/center/c77.png'),
            'wxapp_url' => '',
            'account_url' => '',
        ];

        $menu[] = [
            'name' => '我的推荐',
            'type' => 'link',
            'group' => $group,
            'sort' => 100,
            'icon' => wgUrl('/static/wxapp/user/center/c44.png'),
            'wxapp_url' => '/membercenter/user-recommend/index',
            'account_url' => '',
        ];
        return $menu;
    }

    /**
     * 获取店铺菜单
     * $user
     */
    public static function getMerchMenus($user)
    {
        $menu = [];
        $group = '我的服务';//菜单分组

        if (!PluginService::isEnabled('merch')) {
            return $menu;
        }

        $baseConfig = ShimmerLiveshopMerchConfig::getByType('base');

        if ($baseConfig && $baseConfig['enable'] == 0) {
            return $menu;
        }

        //查看是否有店铺
        $store = ShimmerLiveshopMerchStore::uid($user->id)
            ->first();

        if ($store) {
            $menus[] = [
                'name' => '我的店铺',
                'type' => 'link',
                'group' => $group,
                'sort' => 100,
                'icon' => wgUrl('/static/wxapp/user/center/t66.png'),
                'wxapp_url' => '/merch/pages/store/center/index',
                'account_url' => '',
            ];
        } else {
            //是否开启前台申请
            $intoConfig = ShimmerLiveshopMerchConfig::getByType('into');
            if ($intoConfig && $intoConfig['open_apply'] == 0) {
                return $menu;
            }

            $menus[] = [
                'name' => '我要开店',
                'type' => 'link',
                'group' => $group,
                'sort' => 100,
                'icon' => wgUrl('/static/wxapp/user/center/c11.png'),
                'wxapp_url' => '/merch/pages/apply/index',
                'account_url' => '',
            ];
        }

        return $menus;
    }

    /**
     * @param $user
     * @return array
     * 我的合伙人
     */
    public static function getPartnerMenus($user){
        $menu = [];
        $group = "我的合伙人";//菜单分组

        if (!PluginService::isEnabled('partner')) {
            return $menu;
        }

        if (PartnerService::userIsPartner($user->id)){
            $result[] = [
                'name' => "我的团队",
                'type' => 'link',
                'group' => $group,
                'sort' => '145',
                'icon' => wgUrl('/static/wxapp/user/center/b11.png'),
                'wxapp_url' => '/partner/pages/partner-center/index',
                'account_url' => '',
            ];
        }
        $result[] = [
            'name' => "套餐购买",
            'type' => 'link',
            'group' => $group,
            'sort' => '145',
            'icon' => wgUrl('/static/wxapp/user/center/b11.png'),
            'wxapp_url' => '/partner/pages/partner-apply/index',
            'account_url' => '',
        ];

        return $result;
    }

    /**
     * 主播菜单
     * @param $user
     * @return array
     */
    public static function getAnchorMenus($user)
    {
        $result = [];

        $examine = ExamineService::getConfig();

        $group = "我的{$examine['keywords_live']}";//菜单分组

        $anchor = AnchorService::userIsAnchor($user->id);

        if (!$anchor) {
            $config = ShimmerLiveshopConfig::getByType(14);

            if ($config['apply_open'] == 1) {
                $result[] = [
                    'name' => "{$examine['keywords_live_anchor']}套餐",
                    'type' => 'link',
                    'group' => $group,
                    'sort' => '100',
                    'icon' => wgUrl('/static/wxapp/user/center/b11.png'),
                    'wxapp_url' => '/live/pages/anchor-apply/index',
                    'account_url' => '',
                ];
            }
        } else {
            $result[] = [
                'name' => "开始{$examine['keywords_live']}",
                'type' => 'link',
                'group' => $group,
                'sort' => '144',
                'icon' => wgUrl('/static/wxapp/user/center/z11.png'),
                'wxapp_url' => '/live/pages/live_add/index',
                'account_url' => '',
            ];
            $result[] = [
                'name' => "邀请{$examine['keywords_live_anchor']}",
                'type' => 'link',
                'group' => $group,
                'sort' => '145',
                'icon' => wgUrl('/static/wxapp/user/center/b11.png'),
                'wxapp_url' => '/live/pages/anchor-inviation/index',
                'account_url' => '',
            ];
            $result[] = [
                'name' => "{$examine['keywords_live']}预告",
                'type' => 'link',
                'group' => $group,
                'sort' => '145',
                'icon' => wgUrl('/static/wxapp/user/center/b33.png'),
                'wxapp_url' => '/live/pages/anchor-advance-notice/index',
                'account_url' => '',
            ];
            $result[] = [
                'name' => "{$examine['keywords_live']}商品",
                'type' => 'link',
                'group' => $group,
                'sort' => '145',
                'icon' => wgUrl('/static/wxapp/user/center/a33.png'),
                'wxapp_url' => '/live/pages/anchor-goods/index',
                'account_url' => '',
            ];
            $result[] = [
                'name' => "{$examine['keywords_live_anchor']}套餐",
                'type' => 'link',
                'group' => $group,
                'sort' => '100',
                'icon' => wgUrl('/static/wxapp/user/center/b11.png'),
                'wxapp_url' => '/live/pages/anchor-apply/index',
                'account_url' => '',
            ];
            $result[] = [
                'name' => "{$examine['keywords_live_anchor']}权益",
                'type' => 'link',
                'group' => $group,
                'sort' => '100',
                'icon' => wgUrl('/static/wxapp/user/center/b11.png'),
                'wxapp_url' => '/live/pages/anchor_equity/index',
                'account_url' => '',
            ];
        }

        return $result;
    }
}