<?php

namespace App\Modules\Liveapi\Services\Wechat;

use App\Models\ShimmerLiveshopStoreOrderGroup;
use App\Modules\Liveapi\Models\ShimmerLiveshopConfig;
use App\Models\ShimmerLiveshopPayRequestData;
use App\Models\ShimmerLiveshopStoreOrder;
use App\Models\ShimmerLiveshopUser;
use App\Modules\Liveapi\Models\ShimmerLiveshopAttachment;
use App\Modules\Liveapi\Models\ShimmerLiveshopUserFormId;
use App\Modules\Liveapi\Services\AttachmentService;
use App\Modules\Liveapi\Services\BaseService;
use App\Modules\Liveapi\Services\MiniProgramService;
use App\Modules\Liveapi\Services\OrderService;
use App\Modules\Liveapi\Services\UserService;
use EasyWeChat\Factory;
use We7\Table\Store\Order;

class WechatService extends BaseService
{
    public static function saveFormId($data)
    {
        if ($data['formId'] == 'the formId is a mock one') {
            return true;
        }
        $data = array(
            'uniacid' => Request()->route('uniacid'),
            'uid' => Request()->userInfo['id'],
            'form_id' => $data['formId'],
            'status' => 1,
            'stop_time' => strtotime('+7 day') - 1,
        );
        $model = new ShimmerLiveshopUserFormId();
        $model->fill($data)->save();
        return true;
    }

    /**
     * 保存搜集用户formid
     * @param $uid
     * @param $formId
     * @param $uniacid
     */
    public static function collectFormId($uid, $formId, $uniacid = '')
    {
        if ($formId == 'the formId is a mock one') {
            return true;
        }

        if (!$uid || !$formId) {
            return false;
        }

        $data = array(
            'uniacid' => $uniacid ? $uniacid : Request()->route('uniacid'),
            'uid' => $uid,
            'form_id' => $formId,
            'status' => 1,
            'stop_time' => strtotime('+7 day') - 3600,
        );

        $model = new ShimmerLiveshopUserFormId();

        return $model->fill($data)->save();
    }

    /**
     * @param $extra
     * @param $name
     * @param string $path
     * @return $this|ShimmerLiveshopAttachment
     * @throws \EasyWeChat\Kernel\Exceptions\InvalidArgumentException
     * @throws \EasyWeChat\Kernel\Exceptions\RuntimeException
     * 生成小程序二维码
     */
    public static function getCode($extra, $name, $path = '')
    {
        if (is_array($extra)) {
            $scene = "";
            foreach ($extra as $k => $v) {
                $scene .= "{$k}={$v}&";
            }
            $scene = substr($scene, 0, -1);
        } else {
            $scene = "data={$extra}";
        }
        $app = MiniProgramService::miniprogram();
        $response = $app->app_code->getUnlimit($scene, [
            'page' => $path,
            'width' => 280,
        ]);
        if (is_array($response) && isset($response['errcode'])) {
            return response()->json([
                'errno' => 400,
                'data' => [],
                'message' => '二维码生成错误:' . $response['errmsg']
            ], 200, ['charset' => 'utf-8'])->send();
        }
        $filepath = ATTACHMENT_ROOT . 'shimmer_liveshop/shareCode/page';

        if (!is_dir($filepath)) {
            mkdir($filepath, 0777, true);
        }
        if ($response instanceof \EasyWeChat\Kernel\Http\StreamResponse) {
            $filename = $response->saveAs($filepath, $name);
        }
        $info = get_headers(wgTomedia('shimmer_liveshop/shareCode/page/' . $name), true);

        $attach["code"] = 200;
        $attach["name"] = $name;
        $attach["dir"] = 'shimmer_liveshop/shareCode/page/' . $name;
        $attach['size'] = $info['Content-Length'];
        $attach['type'] = $info['Content-Type'];
        $attach["image_type"] = 1;
        $attach['thumb_path'] = $attach['dir'];
        $data = AttachmentService::store($attach);
        return $data;
    }

    /**
     * 小程序支付参数
     * @param $orderId
     * @param string $attach 附件字符串
     * @param string $isGroup 是否是组合订单
     * @return mixed
     */
    public static function jsPay($orderId, $goodsName = '', $attach = 'product', $isGroup = false)
    {
        $app = MiniProgramService::payment();
        $jssdk = $app->jssdk;

        $prepay = self::createWxOrder($orderId, $goodsName, $attach, $isGroup);

        return $jssdk->sdkConfig($prepay['prepay_id']); // 返回数组
    }

    /**
     * 微信统一下单
     * @param $orderId
     * @param $goodsName 商品名称
     * @param $attach
     * @param $isGroup
     * @param string $attach
     */
    public static function createWxOrder($orderId, $goodsName = '', $attach = 'product', $isGroup = false)
    {
        if (is_string($orderId)) {
            if ($isGroup) {
                $orderInfo = ShimmerLiveshopStoreOrderGroup::getByOrderId($orderId);
            } else {
                $orderInfo = ShimmerLiveshopStoreOrder::getByOrderId($orderId);
            }
        } else {
            $orderInfo = $orderId;
        }
        if (!$orderInfo || !isset($orderInfo['paid'])) {
            exception('支付订单不存在!');
        }
        if ($orderInfo['paid']) {
            exception('订单已支付!');
        }
        if ($orderInfo['pay_price'] <= 0) {
            exception('该支付无需支付!');
        }

        $user = ShimmerLiveshopUser::getByUid($orderInfo['uid']);
        $site = ShimmerLiveshopConfig::getByType(1);

        $openid = $user['wxapp_openid'];

        $app = MiniProgramService::payment();

        $cartInfos = OrderService::getCartInfosByOid($orderInfo->id);

        if (!$goodsName) {
            $goodsName = '';
            foreach ($cartInfos as $cartInfo) {
                $goodsName .= $cartInfo->cart_info['productInfo']['store_name'] . ',';
            }
            $goodsName = trim($goodsName, ',');
            $goodsName = substr($goodsName, 0, 3000);
        }

        //生成支付订单
        $isContract = false;
        $siteName = (string)$site['site_name'];
        $request = [
            'body' => $siteName ?: '购买商品',
            'detail' => $goodsName,
            'out_trade_no' => $orderInfo->order_id,
            'total_fee' => $orderInfo['pay_price'] * 100,
            'spbill_create_ip' => '', // 可选，如不传该参数，SDK 将会自动获取相应 IP 地址
            'trade_type' => 'JSAPI', // 请对应换成你的支付方式对应的值类型
            'attach' => $attach,
        ];
        $payment = ShimmerLiveshopConfig::getByType(9);
        $accountType = $payment['account_type'];

        if ($accountType == 1) {
            $request['sub_openid'] = $openid;
        } else {
            $request['openid'] = $openid;
        }

        $result = $app->order->unify($request, $isContract);

        if ($result['return_code'] == 'FAIL') {
            ShimmerLiveshopPayRequestData::record($orderInfo->order_id, 'payment.order.unify', 1, $request, $result, 0);
            exception("订单创建失败:{$result['return_msg']}");
        }
        if ($result['result_code'] == 'FAIL') {
            ShimmerLiveshopPayRequestData::record($orderInfo->order_id, 'payment.order.unify', 1, $request, $result, 0);
            exception("订单创建失败:{$result['err_code_des']}");
        }

        ShimmerLiveshopPayRequestData::record($orderInfo->order_id, 'payment.order.unify', 1, $request, $result, 1);
        return $result;
    }
}