<?php

namespace App\Providers;

use Illuminate\Foundation\Support\Providers\EventServiceProvider as ServiceProvider;

class EventServiceProvider extends ServiceProvider
{
    /**
     * The event listener mappings for the application.
     *
     * @var array
     */
    protected $listen = [
        'App\Events\Event' => [
            'App\Listeners\EventListener',
        ],
        'App\Events\CommissionRelation' => [
            'App\Listeners\CommissionRelationListener',//绑定分销关系
        ],
        'App\Events\CommissionOrder' => [
            //'App\Listeners\CommissionOrderListener',//创建分销订单
        ],
        'App\Events\CommissionAgent' => [
            'App\Listeners\CommissionAgentListener',//分销商申请
        ],
        'App\Events\SendCommission' => [
            'App\Listeners\SendCommissionListener',//发送佣金
        ],
        'App\Events\LiveEvent' => [
            'App\Listeners\IntoLiveListener',//今日直播间
        ],
        //创建订单
        'App\Events\OrderCreated' => [
            'App\Listeners\Live\LiveOrderWaitPayMsg',//直播间下订单通知
        ],
        //订单支付成功
        'App\Events\OrderPaySuccess' => [
            'App\Listeners\OrderPaySuccessUpdateListener',//更新状态
            'App\Listeners\OrderPaySuccessNoticeListener',//发送通知

            'App\Listeners\Commission\PayUserRelationListener',//付费绑定关系
            'App\Listeners\Commission\SelfBuy\SelfBuyOrderListener',//商品自购省钱
            'App\Listeners\CommissionOrderListener',//分销订单（不包含主播入驻）
            'App\Listeners\Commission\AnchorSettledOrderListener',//主播入驻分销
            'App\Listeners\Anchor\AnchorGoodsCommissionListener',//主播商品提成
            'App\Listeners\Commission\Partner\PartnerSettledOrderListener',//合伙人入驻分销
            'App\Listeners\Gift\AnchorGiftSettledOrderListener',//主播入驻分销
            'App\Listeners\Live\LiveOrderSuccessPayMsg',//直播间下订单通知
        ],
        //订单发货
        'App\Events\OrderDelivery' => [
            'App\Listeners\OrderDeliveryNoticeListener',//发送通知
        ],
        //订单退款
        'App\Events\OrderRefund' => [
            'App\Listeners\OrderRefundUpdateListener',//退款更新
            'App\Listeners\OrderRefundNoticeListener',//发送通知
            'App\Listeners\Commission\CommissionRefundListener',//发送通知
        ],
        //订单确认收货
        'App\Events\OrderTakeed' => [
            'App\Listeners\OrderTakedNoticeListener',//发送通知
            'App\Listeners\CommissionOrderSettlementListener',//佣金结算
        ],
        //订单商品评价完成(全部商品)
        'App\Events\OrderCommented' => [
        ],
        //用户取消/删除订单
        'App\Events\OrderRemoved' => [
            'App\Listeners\OrderRemoveRegressionListener',//返库存/积分
        ],
        //用户再次下单
        'App\Events\OrderCreateAgain' => [
        ],
        //拒绝退款
        'App\Events\OrderRejectRefund' => [
            'App\Listeners\OrderRejectRefundNoticeListener',//拒绝退款通知
        ],
        //插件启用
        'App\Events\Plugin\PluginEnable' => [
        ],
        //插件禁用
        'App\Events\Plugin\PluginDisable' => [
        ],
        //删除插件
        'App\Events\Plugin\PluginDelete' => [
        ],
    ];

    /**
     * Register any events for your application.
     *
     * @return void
     */
    public function boot()
    {
        parent::boot();

        //
    }
}
