<?php
/**
 * 余额积分
 * Created by PhpStorm.
 * User: lirui
 * Date: 2020/2/5
 * Time: 上午11:34
 */

namespace App\Services;


use App\Models\ShimmerLiveshopCommissionOrder;
use App\Models\ShimmerLiveshopCreditLog;
use App\Models\ShimmerLiveshopRechargeLog;
use App\Models\ShimmerLiveshopStoreAsset;
use App\Models\ShimmerLiveshopUser;
use App\Models\ShimmerLiveshopUserBill;
use App\Modules\Live\Models\ShimmerLiveshopConfig;
use App\Modules\Liveapi\Models\ShimmerLiveshopStoreOrder;
use Illuminate\Support\Facades\DB;

class BalanceService extends BaseService
{

    /**
     * 订单退款扣佣金
     * @param ShimmerLiveshopRechargeLog $rechargeLog
     */
    public static function refundOrderCommission(ShimmerLiveshopCommissionOrder $commissionOrder)
    {
        if ($commissionOrder->status != 2 || $commissionOrder->has_withdraw != 1) {
            return self::error(-1, '未提现不能退款！');
        }

        $user = ShimmerLiveshopUser::find($commissionOrder->uid);
        $balance = $user->balance;

        if ($commissionOrder->commission) {
            $balance -= $commissionOrder->commission;
        }

        $user->balance = $balance;
        $user->save();

        $num = $commissionOrder->commission;
        $presentcredit = $user->balance;
        $givePresentcredit = $user->give_balance;
        $text = '余额';

        $text = "退款退{$text}";

        $commissionOrder->status = -1;
        $commissionOrder->save();
        $adminUser = AppService::$user;

        //插入余额明细
        self::addCreditLog(51, $commissionOrder->uid,
            1,
            -$num,
            $presentcredit,
            $text,
            0,
            0,
            $adminUser['uid'],
            $givePresentcredit
        );

        //记录用户账单
        ShimmerLiveshopUserBill::expend(
            '订单退款退佣金扣余额', $user->id, 'balance', 'refund_commission', $num, $commissionOrder->id, $presentcredit, $text
        );

        return true;
    }

    /**
     * 充值退款
     * @param ShimmerLiveshopRechargeLog $rechargeLog
     */
    public static function refundRecharge(ShimmerLiveshopRechargeLog $rechargeLog)
    {
        if ($rechargeLog->state != 1) {
            return self::error(-1, '未充值成功不能退款！');
        }

        $user = ShimmerLiveshopUser::find($rechargeLog->uid);
        $credit = $user->credit;
        $balance = $user->balance;
        $giveBalance = $user->give_balance;

        if ($rechargeLog->recharge_credit) {
            $credit -= $rechargeLog->recharge_credit;
        }
        if ($rechargeLog->recharge_money) {
            $balance -= $rechargeLog->recharge_money;
        }
        if ($rechargeLog->give_money) {
            $giveBalance -= $rechargeLog->give_money;
        }
        if ($rechargeLog->give_credit) {
            $credit -= $rechargeLog->give_credit;
        }

        $balance = max(0, $balance);
        $giveBalance = max(0, $giveBalance);
        $credit = max(0, $credit);

        $user->balance = $balance;
        $user->give_balance = $giveBalance;
        $user->credit = $credit;
        $user->save();

        $givePresentcredit = 0;
        if ($rechargeLog->type == 1) {
            $num = $rechargeLog->recharge_money;
            $presentcredit = $user->balance;
            $givePresentcredit = $user->give_balance;
            $text = '余额';
        } elseif ($rechargeLog->type == 2) {
            $num = $rechargeLog->recharge_credit;
            $presentcredit = $user->credit;
            $text = '积分';
        }

        $text = "退款退{$text}";

        $rechargeLog->state = 2;
        $rechargeLog->save();
        $adminUser = AppService::$user;
        //插入余额明细
        self::addCreditLog(51, $rechargeLog->uid,
            $rechargeLog->type,
            -$num,
            $presentcredit,
            $text,
            -$rechargeLog->give_money,
            -$rechargeLog->give_credit,
            $adminUser['uid'],
            $givePresentcredit
        );

        //记录用户账单
        ShimmerLiveshopUserBill::expend(
            '余额充值充值退款扣余额', $user->id, 'balance', 'refund_recharge', $num, $rechargeLog->id, $presentcredit, $text
        );

        return true;
    }

    /**
     * 插入余额积分明细表
     * @param $uid 用户
     * @param $type 1余额 2积分
     * @param $num 数量
     * @param $presentcredit 当前数量
     * @param $remark 备注
     * @param $giveMoney 赠送金额
     * @param $giveCredit 赠送积分
     * @param $operator 后台操作员id
     * @param $givePresentcredit 当前赠送数量
     * @param $orderId 订单id
     */
    public static function addCreditLog($logType, $uid, $type, $num, $presentcredit, $remark = '', $giveMoney = 0, $giveCredit = 0, $operator = 0, $givePresentcredit = 0, $orderId = 0)
    {
        $data = [
            'uniacid' => (int)AppService::$uniacid,
            'log_type' => (int)$logType,
            'uid' => (int)$uid,
            'type' => (int)$type,
            'order_id' => (int)$orderId,
            'num' => (string)$num,
            'give_money' => (string)$giveMoney,
            'give_credit' => (string)$giveCredit,
            'presentcredit' => (string)$presentcredit,
            'give_presentcredit' => (string)$givePresentcredit,
            'remark' => (string)$remark,
            'operator' => (int)$operator,
        ];

        $log = new ShimmerLiveshopCreditLog();
        $log->fill($data)->save();

        //记录用户账单
        switch ($type) {
            case 1:
                $category = 'balance';
                break;
            case 2:
                $category = 'credit';
                break;
        }
        if ($num > 0) {
            $pm = 1;
        } else {
            $pm = 0;
        }

        $title = ShimmerLiveshopCreditLog::$type[$type] . ShimmerLiveshopCreditLog::$logType[$logType];
        //记录用户账单
        ShimmerLiveshopUserBill::expend(
            $title, $uid, $category, 'credit_log_type_' . $logType, $num, $log->id, $presentcredit, $remark, 1, $pm
        );

    }

    /**
     * 系统赠送余额
     * @param $user
     * @param $rechargeMoney
     * @param $giveMoney
     * @param $giveCredit
     * @param $from
     * @param string $remark
     * @param int $operator
     * @return bool|mixed
     */
    public static function systemGiveBalance($user, $rechargeMoney, $giveMoney, $giveCredit, $from, $remark = '', $operator = 0){
        DB::beginTransaction();
        if (!$user || !$rechargeMoney) {
            return self::error(-1, '充值余额参数错误');
        }

        $credit = $user->credit;
        $balance = $user->balance;
        $giveBalance = $user->give_balance;

        if ($giveCredit) {
            $credit += $giveCredit;
        }
        if ($rechargeMoney) {
            $balance += $rechargeMoney;
        }
        if ($giveMoney) {
            $giveBalance += $giveMoney;
        }

        $balance = max(0, $balance);
        $giveBalance = max(0, $giveBalance);
        $credit = max(0, $credit);

        $user->balance = $balance;
        $user->give_balance = $giveBalance;
        $user->credit = $credit;
        $ret = $user->saveOrFail();

        if (!$ret) {
            DB::rollBack();
            return self::error(-1, '更新用户充值数据失败');
        }

        //插入明细日志
        self::addCreditLog($from, $user->id,
            1,
            $rechargeMoney,
            $user->balance,
            $remark,
            $giveMoney,
            $giveCredit,
            $operator,
            $user->give_balance
        );

        DB::commit();
        return true;
    }

    /**
     * 充值余额
     * @param $user 用户
     * @param $orderId 订单id
     * @param $rechargeMoney 充值金额
     * @param $giveMoney 赠送金额
     * @param $giveCredit 赠送积分
     * @param $from 事件
     * @param $remark 备注
     * @param $operator 后台操作员
     */
    public static function rechargeBalance($user, $orderId, $rechargeMoney, $giveMoney, $giveCredit, $from, $remark = '', $operator = 0)
    {
        DB::beginTransaction();
        if (!$user || !$rechargeMoney) {
            return self::error(-1, '充值余额参数错误');
        }

        $credit = $user->credit;
        $balance = $user->balance;
        $giveBalance = $user->give_balance;

        if ($giveCredit) {
            $credit += $giveCredit;
        }
        if ($rechargeMoney) {
            $balance += $rechargeMoney;
        }
        if ($giveMoney) {
            $giveBalance += $giveMoney;
        }

        $balance = max(0, $balance);
        $giveBalance = max(0, $giveBalance);
        $credit = max(0, $credit);

        $user->balance = $balance;
        $user->give_balance = $giveBalance;
        $user->credit = $credit;
        $ret = $user->saveOrFail();

        if (!$ret) {
            DB::rollBack();
            return self::error(-1, '更新用户充值数据失败');
        }

        //插入充值日志
        $rechargeLog = new ShimmerLiveshopRechargeLog();
        $data = [
            'uniacid' => AppService::$uniacid,
            'uid' => $user->id,
            'order_id' => $orderId,
            'recharge_money' => $rechargeMoney,
            'give_money' => $giveMoney,
            'give_credit' => $giveCredit,
            'type' => 1,
            'from' => $from,
            'state' => 1,
            'remark' => $remark,
            'operator' => $operator,
        ];
        $rechargeLog->fill($data)->saveOrFail();

        //插入明细日志
        self::addCreditLog($from, $user->id,
            1,
            $rechargeMoney,
            $user->balance,
            $remark,
            $giveMoney,
            $giveCredit,
            $operator,
            $user->give_balance
        );

        DB::commit();
        return true;
    }

    /**
     * 充值积分
     * @param $user
     * @param $orderId
     * @param $rechargeCredit
     * @param $giveCredit
     * @param $from
     * @param $remark
     * @param $operator
     */
    public static function rechargeCredit($user, $orderId, $rechargeCredit, $giveCredit, $from, $remark = '', $operator = 0)
    {
        if (!$user || !$rechargeCredit) {
            return self::error(-1, '充值积分参数错误');
        }

        DB::beginTransaction();


        $credit = $user->credit;

        if ($rechargeCredit) {
            $credit += $rechargeCredit;
        }
        if ($giveCredit) {
            $credit += $giveCredit;
        }

        $credit = max(0, $credit);
        $user->credit = $credit;

        $ret = $user->saveOrFail();

        if (!$ret) {
            DB::rollBack();
            return self::error(-1, '更新用户充值数据失败');
        }

        //插入充值日志
        $rechargeLog = new ShimmerLiveshopRechargeLog();
        $data = [
            'uniacid' => AppService::$uniacid,
            'uid' => $user->id,
            'order_id' => $orderId,
            'recharge_credit' => $rechargeCredit,
            'give_credit' => $giveCredit,
            'type' => 2,
            'from' => $from,
            'state' => 1,
            'remark' => $remark,
            'operator' => $operator,
        ];
        $rechargeLog->fill($data)->saveOrFail();

        //插入明细日志
        self::addCreditLog($from, $user->id, 2, $rechargeCredit, $user->credit, $remark, 0, $giveCredit, $operator, 0);

        DB::commit();
        return true;
    }

    /**
     * 创建用户充值订单
     * @param $user 用户信息
     * @param $money 充值金额
     */
    public static function createRechargeOrder($user, $money)
    {
        if ($money <= 0) {
            return self::error(-1, '充值金额错误');
        }

        $orderInfo = [
            'uniacid' => AppService::$uniacid,
            'uid' => $user['id'],
            'type' => 5,
            'order_id' => OrderService::getNewOrderId(),
            'total_price' => $money,
            'pay_price' => $money,
            'paid' => 0,
            'is_channel' => 1,
            'add_time' => time(),
        ];

        $key = md5($orderInfo['uid'] . '_5_' . $orderInfo['order_id']);
        $orderInfo['unique'] = $key;

        $order = new ShimmerLiveshopStoreOrder();
        $saveRet = $order->fill($orderInfo)->saveOrFail();
        $rechargeGive = self::getRechargeGive($money);

        //插入充值日志
        $rechargeLog = new ShimmerLiveshopRechargeLog();

        $data = [
            'uniacid' => AppService::$uniacid,
            'uid' => $user->id,
            'order_id' => $order->id,
            'recharge_money' => $money,
            'give_money' => $rechargeGive['give_money'],
            'give_credit' => $rechargeGive['give_credit'],
            'type' => 1,
            'from' => 1,
            'state' => 0,
        ];
        $rechargeLog->fill($data)->saveOrFail();

        if (!$saveRet) {
            return self::error(-1, '订单生成失败5!');
        }

        OrderService::log($order['id'], 'cache_key_create_order', '订单生成');

        return $order;
    }

    /**
     * 获取充值余额赠送
     * @param $money
     */
    public static function getRechargeGive($money)
    {
        $result = [
            'give_credit' => 0,//赠送积分
            'give_money' => 0,//赠送余额
        ];

        $config = ShimmerLiveshopConfig::getByType(18);

        if (!$config) {
            return $result;
        }

        foreach ($config as $item) {
            if ($item['enough'] == $money) {
                $result['give_money'] = $item['give'];
            }
        }

        return $result;
    }

    /**
     * 余额充值成功订单回调
     * @param $order
     */
    public static function rechargeBalanceSuccessNotify($order)
    {
        DB::beginTransaction();

        $rechargeLog = ShimmerLiveshopRechargeLog::where('order_id', $order->id)->first();

        if (!$rechargeLog) {
            exception('充值订单不存在');
        }

        $user = ShimmerLiveshopUser::find($rechargeLog->uid);

        if (!$user) {
            exception('充值用户不存在');
        }

        $credit = $user->credit;
        $balance = $user->balance;
        $giveBalance = $user->give_balance;

        if ($rechargeLog->give_credit) {
            $credit += $rechargeLog->give_credit;
        }
        if ($rechargeLog->recharge_money) {
            $balance += $rechargeLog->recharge_money;
        }
        if ($rechargeLog->give_money) {
            $giveBalance += $rechargeLog->give_money;
        }

        $balance = max(0, $balance);
        $giveBalance = max(0, $giveBalance);
        $credit = max(0, $credit);

        $user->balance = $balance;
        $user->give_balance = $giveBalance;
        $user->credit = $credit;
        $user->recharge_money = $user->recharge_money + $rechargeLog->recharge_money;

        $ret = $user->saveOrFail();

        if (!$ret) {
            DB::rollBack();
            return self::error(-1, '更新用户充值数据失败');
        }

        $rechargeLog->state = 1;
        $rechargeLog->save();

        //插入明细日志
        self::addCreditLog($rechargeLog->from, $user->id,
            1,
            $rechargeLog->recharge_money,
            $user->balance,
            '用户线上充值',
            $rechargeLog->give_money,
            $rechargeLog->give_credit,
            0,
            $user->give_balance
        );

        //插入资产明细
        $remark = '';
        if ($rechargeLog->give_money > 0) {
            $remark = '赠送余额' . $rechargeLog->give_money;
        }
        if ($rechargeLog->give_credit > 0) {
            $remark = '赠送积分' . $rechargeLog->give_credit;
        }
        ShimmerLiveshopStoreAsset::addRecord(ShimmerLiveshopStoreAsset::BALANCE_RECHARGE, 0, $user->id, $user->realname, $user->mobile, $order->id, $rechargeLog->recharge_money, 1, 1, $remark);

        DB::commit();
        return true;
    }
}