<?php
/**
 * 快递service
 * Created by PhpStorm.
 * User: lirui
 * Date: 2019/11/19
 * Time: 下午1:26
 */

namespace App\Services;

use App\Lib\Bemrc\HttpService;
use App\Models\ShimmerLiveshopExpress;
use App\Modules\Liveapi\Models\ShimmerLiveshopConfig;
use App\Modules\Liveapi\Models\ShimmerLiveshopStoreOrder;
use App\Services\BaseService;
use Illuminate\Support\Facades\Cache;

class ExpressService extends BaseService
{
    /**
     * 停车场api接口地址
     * @var array
     */
    protected static $api = [
        'query' => 'https://wuliu.market.alicloudapi.com/kdi'
    ];

    /**
     * 修改物流状态
     * @param $id
     * @param $isShow
     */
    public static function changeIsShow($id, $isShow)
    {
        if (!$id) {
            return self::error(-1, '参数错误');
        }

        return ShimmerLiveshopExpress::where('id', $id)->update(['is_show' => $isShow]);
    }

    /**
     * 获取正在使用的快递公司
     */
    public static function getUseExpress()
    {
        return ShimmerLiveshopExpress::where('is_show', 1)
            ->orderBy('sort', 'desc')
            ->get();
    }

    /**
     * 获取物流详情
     * @param $no 物流单号
     * @param string $type
     * @return bool|mixed
     */
    public static function query($no, $type = '')
    {
        $config = ShimmerLiveshopConfig::getByType(10);
        if (!$config || !$config['express_appsecret']) {
            return self::error(-1, '请先配置快递查询秘钥');
        }

        $cacheName = 'express_' . $no;
        $result = Cache::get($cacheName);

        if ($result) {
            return $result;
        }

        $appCode = $config['express_appsecret'];

        $res = HttpService::getRequest(self::$api['query'], compact('no', 'typ3e'), ['Authorization:APPCODE ' . $appCode]);

        if ($res) {
            $result = json_decode($res, true) ?: false;

            Cache::put($cacheName, $result, 1800);
            return $result;
        }

        return self::error(-1, '快递查询接口请求失败');
    }

}