<?php

namespace App\Services;

/**
 * 插件服务类
 * Class AppService
 */

use App\Models\ModulesPlugin;
use App\Models\ShimmerLiveshopPlugin;
use Illuminate\Support\Facades\Cache;

class PluginService extends \App\Services\BaseService
{
    public static $pluginPre = 'shimmer_liveshop_plugin_';
    public static $pluginEnableds = [];
    public static $errorPlugins = [];//启动错误的插件
    //插件统计
    public static $plugins = [
        'merch' => '多商户',
    ];

    /**
     * 判断是否装插件
     * @param $name
     */
    public static function isEnabled($slug)
    {
        $result = false;

        if ($slug instanceof ShimmerLiveshopPlugin) {
            $slug = $slug->slug;
        }

        if (isset(self::$pluginEnableds[$slug])) {
            $result = self::$pluginEnableds[$slug];
        } else {
            $result = ShimmerLiveshopPlugin::isEnabled($slug);
            self::$pluginEnableds[$slug] = $result;
        }

        if (isset(self::$errorPlugins[$slug])) {
            return false;
        }

        return $result;
    }

    /**
     * 启动错误的插件
     * @param $slug
     */
    public static function setErrorPlugins($plugin)
    {
        self::$errorPlugins[$plugin->slug] = $plugin;
    }

    public static function getPluginFullName($slug)
    {
        return self::$pluginPre . $slug;;
    }

    /**
     * 获取所有安装的插件
     */
    public static function getEnablesPlugins()
    {
        $cacheKey = 'shimmer_liveshop_plugins';

//        $plugins = Cache::get($cacheKey);
//
//        if ($plugins) {
//            return $plugins;
//        } else {
        $plugins = ShimmerLiveshopPlugin::enables();
//            Cache::put($cacheKey, $plugins, 86400);
//        }

        return $plugins;
    }

    /**
     * 获取插件命名空间
     * @param $slug
     */
    public static function getPluginNamespace($slug)
    {
        return ucwords(self::getPluginFullName($slug)) . '\\';
    }

    /**
     * 获取插件相对目录
     */
    public static function getBasePath()
    {
        return realpath(app_path() . '/../../');
    }

    /**
     * 获取插件路径
     * @param $name
     */
    public static function getPluginPath($slug)
    {
        return self::getBasePath() . '/' . self::getPluginFullName($slug);
    }
}