<?php
/**
 * 标签服务
 * Created by PhpStorm.
 * User: lirui
 * Date: 2019/8/15
 * Time: 下午4:57
 */

namespace App\Services;

use App\Models\ShimmerLiveshopGoods;
use App\Models\ShimmerLiveshopLideshow;
use App\Models\ShimmerLiveshopTag;

class TagService extends BaseService
{
    /**
     * 获取所有标签类型
     * @return array
     */
    public static function getTypes()
    {
        return ShimmerLiveshopTag::$types;
    }

    /**
     * 修改标签状态
     * @param $id
     * @param $isShow
     */
    public static function changeIsShow($id, $isShow)
    {
        if (!$id || !$isShow) {
            return self::error(-1, '参数错误');
        }

        return ShimmerLiveshopTag::where('id', $id)->update(['isshow' => $isShow]);
    }

    /**
     * 获取所有的标签，按分组
     */
    public static function getTags()
    {
        $result = [];

        $types = self::getTypes();

        foreach ($types as $typeIndex => $typeName) {
            $result[$typeIndex] = [
                'type' => $typeIndex,
                'name' => $typeName,
                'list' => self::getTagsByType($typeIndex),
            ];
        }
        return $result;
    }

    /**
     * 通过类型获取所有的标签列表
     * @param $type
     */
    public static function getTagsByType($type)
    {
        if (!$type) {
            return [];
        }

        return ShimmerLiveshopTag::where('type', $type)
            ->where('isshow', 1)
            ->orderBy('sort', 'desc')
            ->get();
    }

    /**
     * 通过id获取标签列表
     * @param $ids
     */
    public static function getTagsByIds($ids)
    {
        if (!$ids) {
            return [];
        }

        if (!is_array($ids)) {
            $ids = [$ids];
        }

        return ShimmerLiveshopTag::whereIn('id', $ids)
            ->orderBy('sort', 'desc')
            ->get()
            ->toArray();
    }

    /**
     * 获取标签关系对象列表
     * @param $relationType
     * @param $tagId
     */
    public static function getTagRelations($relationType, $tagId, $limit = 0)
    {
        $result = [];

        if (!$relationType || !$tagId) {
            return [];
        }

        switch ($relationType) {
            case ShimmerLiveshopTag::GOODS_TAG:
                $result = ShimmerLiveshopGoods::getTagGoods($tagId, $limit);
                break;
            case ShimmerLiveshopTag::LIVE_TAG:
                break;
        }

        return $result;
    }

    /**
     * 获取标签轮播图列表
     * @param $tagId
     */
    public static function getTagBanners($tagId)
    {
        if (!$tagId) {
            return [];
        }

        return ShimmerLiveshopLideshow::getByTagId($tagId);
    }


}