<?php

namespace App;

use App\Models\BaseModel;
use App\Modules\Admin\Models\Role;
use App\Modules\Admin\Models\UserRole;
use Illuminate\Auth\Authenticatable;
use Illuminate\Foundation\Auth\Access\Authorizable;
use Illuminate\Contracts\Auth\Authenticatable as AuthenticatableContract;
use Illuminate\Contracts\Auth\Access\Authorizable as AuthorizableContract;

class User extends BaseModel implements
    AuthenticatableContract,
    AuthorizableContract {
    use Authenticatable, Authorizable;

    protected $table = 'users';
    protected $primaryKey = "uid";
    public $timestamps = false;

    protected $fillable = [
        'uid', 'owner_uid', 'groupid', 'founder_groupid', 'username', 'type', 'status', 'joindate', 'joinip', 'lastvisit', 'lastip', 'lasttime', 'lastip','remark','starttime','endtime','register_type','openid','welcome_link','notice_setting','salt'
    ];

    public function __construct(array $attributes = [])
    {
        parent::__construct($attributes);
        $this->rememberTokenName = '';
    }

    public function getAuthPassword(){
        return ['password'=>$this->password,'salt'=>$this->salt];
    }

    public function getJoindateTextAttribute(){
        return date("Y-m-d",$this->joindate);
    }

    public function getEndtimeTextAttribute(){
        if ($this->endtime){
            return date("Y-m-d H:i:s",$this->endtime);
        }else{
            return '永久有效';
        }
    }

    public function getRoleNameAttribute(){
        if ($this->founder_groupid==1){
            return '管理员';
        }elseif(isset($this->userRole->role_id)){
            return Role::where('id',$this->userRole->role_id)->value('name');
        }else{
            return '未分配';
        }
    }

    public function userRole(){
        return $this->hasOne(UserRole::class,'user_id','uid');
    }
}
