<?php
/**
 * 多商户申请
 * Created by PhpStorm.
 * User: lirui
 * Date: 2020/1/26
 * Time: 上午10:20
 */


namespace Shimmer_liveshop_plugin_merch\Http\Controllers\Api\Common;

use App\Events\OrderCreated;
use App\Models\ShimmerLiveshopStoreOrder;
use App\Models\ShimmerLiveshopTemplate;
use App\Modules\Liveapi\Services\Wechat\WechatService;
use App\Services\AppService;
use App\Services\DiyFormService;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Shimmer_liveshop_plugin_merch\Http\Controllers\Api\Controller;
use Shimmer_liveshop_plugin_merch\Models\ShimmerLiveshopMerchConfig;
use Shimmer_liveshop_plugin_merch\Models\ShimmerLiveshopMerchStore;
use Shimmer_liveshop_plugin_merch\Models\ShimmerLiveshopMerchStoreApply;
use Shimmer_liveshop_plugin_merch\Services\OrderService;
use Shimmer_liveshop_plugin_merch\Services\StoreService;

class ApplyController extends Controller
{
    /**
     * 获取申请信息
     */
    public function intro()
    {
        $intoConfig = ShimmerLiveshopMerchConfig::getByType('into');
        $intoConfig['btn_enable'] = false;//按钮是否可点击
        $intoConfig['can_apply'] = false;//是否可点击申请
        $intoConfig['meet_condition'] = false;//满足申请条件

        if ($intoConfig && $intoConfig['open_apply'] == 0) {
            return $this->error(-1, '申请入驻已关闭!');
        }

        $store = ShimmerLiveshopMerchStore::uid($this->uid)
            ->first();

        if ($store) {
            $intoConfig['meet_condition'] = true;
            $intoConfig['status_text'] = '已有店铺，无需申请';
            return $this->error(-1, '已有店铺，无需申请', $intoConfig);
        }

        $lastApply = ShimmerLiveshopMerchStoreApply::uid($this->uid)
            ->where('status', '!=', 2)
            ->orderBy('id', 'desc')
            ->first();

        if ($lastApply && $lastApply->status == 0) {
            $intoConfig['meet_condition'] = true;
            $intoConfig['status_text'] = '请耐心等待申请结果';
            return $this->success($intoConfig);
        }

        if ($intoConfig['apply_limit'] == 1) {
            $intoConfig['can_apply'] = true;
            $intoConfig['btn_enable'] = true;
            $intoConfig['status_text'] = '申请店铺';
        }

        if ($intoConfig['apply_limit'] == 2) {
            $intoConfig['status_text'] = "支付{$intoConfig['apply_limit_pay_money']}元开通店铺";
            $intoConfig['btn_enable'] = true;
            if ($lastApply && $lastApply->pay_money > 0) {
                $intoConfig['can_apply'] = true;
                $intoConfig['meet_condition'] = true;
                $intoConfig['status_text'] = '申请店铺';
            }
        }

        if ($intoConfig['apply_limit'] == 3) {
            $leftLimitMoney = $intoConfig['apply_limit_buy_monty'] - $this->userInfo['pay_money'];

            $intoConfig['pay_money'] = $this->userInfo['pay_money'];
            $intoConfig['apply_limit_left_buy_monty'] = $leftLimitMoney;
            $intoConfig['btn_enable'] = true;

            if ($leftLimitMoney > 0) {
                $intoConfig['status_text'] = "消费金额满{$intoConfig['apply_limit_buy_monty']}元，还需购买{$leftLimitMoney}元";
            } else {
                $intoConfig['meet_condition'] = true;
                $intoConfig['can_apply'] = true;
                $intoConfig['status_text'] = '申请店铺';
            }
        }

        //获取diyform表单信息
        if ($intoConfig['is_apply_diyform'] && $intoConfig['apply_diyform'] > 0) {
            $intoConfig['diyform_fields'] = DiyFormService::getFormById($intoConfig['apply_diyform']);
        }

        //获取模板消息
        $templateId = ShimmerLiveshopTemplate::getTemplateIdByKey(1482);
        $intoConfig['tmplIds'] = [$templateId];

        return $this->success($intoConfig);
    }

    /**
     * 申请支付
     */
    public function pay()
    {
        DB::beginTransaction();

        $intoConfig = ShimmerLiveshopMerchConfig::getByType('into');

        $store = StoreService::getUserStore($this->uid);

        if ($store) {
            return $this->error(-1, '已有店铺，无需申请!');
        }
        if ($intoConfig['apply_limit'] != 2) {
            return $this->error(-1, '申请条件无需支付!');
        }
        if ($intoConfig['apply_limit_pay_money'] <= 0) {
            return $this->error(-1, '申请条件支付金额错误!');
        }

        $lastApply = ShimmerLiveshopMerchStoreApply::uid($this->uid)
            ->where('status', '!=', 2)
            ->orderBy('id', 'desc')
            ->first();

        if ($lastApply && $lastApply->pay_money > 0) {
            return $this->error(-1, '已支付!');
        }

        try {
            $order = OrderService::createApplyPayOrder($this->userInfo);
        } catch (\Exception $e) {
            DB::rollBack();
            return $this->error(-1, '系统错误:' . $e->getMessage());
        }

        if (is_error($order)) {
            DB::rollBack();
            return $this->error(-1, $order['message']);
        }

        $orderId = $order['order_id'];

        $info = compact('orderId', 'key');

        if ($orderId) {
            //订单创建成功事件
            event(new OrderCreated($order));
            try {
                $jsConfig = WechatService::jsPay($orderId, '申请店铺', 'apply_shop'); //创建订单jspay

                $info['jsConfig'] = $jsConfig;
                $data = [
                    'status' => 'WECHAT_PAY',
                    'result' => $info,
                ];
                DB::commit();
                return $this->success($data, '订单创建成功');
            } catch (\Exception $e) {
                DB::rollBack();
                $data = [
                    'status' => 'PAY_ERROR',
                    'result' => [],
                ];
                return $this->success($data, $e->getMessage());
            }
        } else {
            DB::rollBack();
            return $this->error(-1, '订单生成失败2!');
        }
    }

    /**
     * 申请店铺提交
     * @param Request $request
     */
    public function submit(Request $request)
    {
        $diyform = $request->input('diyform');

        DB::beginTransaction();

        $apply = ShimmerLiveshopMerchStoreApply::uid($this->uid)
            ->where('status', '!=', 2)
            ->orderBy('id', 'desc')
            ->first();

        if (!$apply) {
            $apply = new ShimmerLiveshopMerchStoreApply();
        }

        $intoConfig = ShimmerLiveshopMerchConfig::getByType('into');

        if ($intoConfig['apply_limit'] == 2
            && $apply['pay_money'] <= 0
        ) {
            return $this->error(-1, '请重新提交审核付费开通!');
        }

        if ($intoConfig['apply_limit'] == 3
            && $apply['apply_limit_buy_monty'] > $this->userInfo['pay_money']
        ) {
            return $this->error(-1, '未满足申请消费条件!');
        }

        $apply->uniacid = AppService::$uniacid;
        $apply->uid = $this->uid;
        if ($intoConfig['is_apply_diyform'] == 1 && $intoConfig['apply_diyform'] > 0) {
            $apply->diyform_form_id = $intoConfig['apply_diyform'];
        }

        $apply->apply_limit = $intoConfig['apply_limit'];
        if ($intoConfig['apply_limit'] == 3) {
            $apply->consume_money = $this->userInfo['pay_money'];
        }

        //是否需要审核
        if ($intoConfig['need_check'] == 1) {
            $apply->status = 0;
        } else {
            $apply->status = 1;
            $apply->jointime = time();
            $ret = $apply->save();
            if (!$ret) {
                DB::rollBack();
                return $this->error(-1, '申请失败，系统错误!');
            }

            $ret = StoreService::applyAccess($apply);

            if (is_error($ret)) {
                DB::rollBack();
                return $this->error(-1, $ret['message']);
            }
        }

        $ret = $apply->save();

        //保存表单数据
        if ($intoConfig['is_apply_diyform'] && $intoConfig['apply_diyform'] > 0) {
            $diyformData = DiyFormService::saveDiyformData($intoConfig['apply_diyform'], $this->uid, $diyform);
            if (is_error($diyformData)) {
                DB::rollBack();
                return $this->error(-1, $ret['message']);
            }

            $apply->diyform_form_id = $diyformData->form_id;
            $apply->diyform_data_id = $diyformData->id;
            $apply->save();
        }

        if ($ret) {
            DB::commit();
            return $this->success(-1, '申请成功！');
        } else {
            DB::rollBack();
            return $this->error(-1, '申请失败，系统错误!');
        }
    }
}