<?php
/**
 * 商品
 * Created by PhpStorm.
 * User: lirui
 * Date: 2020/2/17
 * Time: 上午12:26
 */

namespace Shimmer_liveshop_plugin_merch\Http\Controllers\Api\Store;

use App\Models\ShimmerLiveshopGoodsCategory;
use App\Models\ShimmerLiveshopGoodsCategoryXref;
use App\Modules\Liveapi\Models\ShimmerLiveshopGoods;
use App\Models\ShimmerLiveshopLiveGoods;
use App\Modules\Liveapi\Services\Live\AnchorService;
use App\Modules\Liveapi\Services\PageService;
use App\Services\AppService;
use App\Services\GlobalRoleService;
use App\Services\GoodsService;
use App\Services\ImageService;
use Illuminate\Http\Request;
use Shimmer_liveshop_plugin_merch\Http\Controllers\Api\StoreController;

class GoodsController extends StoreController
{

    /**
     * 商品列表
     * @param Request $request
     */
    public function getList(Request $request)
    {
        $data = [];
        $type = $request->input('type', 1);
        switch ($type) {
            case 1:
                $data = $this->getSaleingGoods();
                break;
            case 2:
                $data = $this->getLiveGoods();
                break;
            case 3:
                $data = $this->getStorageGoods();
                break;
        }

        $data = PageService::formatGoodsData($data);

        if (in_array($type, [1, 3])) {
            //判断是否在直播库
            $live = AnchorService::getUserLive($this->uid);

            foreach ($data as $key => $item) {
                $data[$key]['is_live_goods'] = ShimmerLiveshopLiveGoods::
                    where('live_id', $live->id)
                        ->where('goods_id', $item['id'])
                        ->count() > 0;
            }
        }

        return $this->success($data);
    }

    /**
     * 发布商品
     * @param Request $request
     */
    public function store(Request $request)
    {
        $data['hasoption'] = 0;//多规格
        $data['title'] = trim($request->input('title'));
        $data['subtitle'] = trim($request->input('subtitle'));
        $data['category_id'] = (int)$request->input('category_id');
        $data['store_category_id'] = (int)$request->input('store_category_id');
        $data['thumbs'] = $request->input('thumbs');
        $data['video'] = $request->input('video');
        $data['unit'] = trim($request->input('unit'));
        $data['marketprice'] = floatval($request->input('marketprice'));
        $data['productprice'] = floatval($request->input('productprice'));
        $data['expressprice'] = floatval($request->input('expressprice'));
        $data['stock'] = floatval($request->input('stock'));
        $data['is_free_freight'] = (int)$request->input('is_free_freight');
        $data['description'] = (string)($request->input('description'));
        $data['state'] = (int)$request->input('state');

        $ret = $this->checkSavePost($data);

        if (is_error($ret)) {
            return $this->error(-1, $ret['msg']);
        }

        $goods = $this->saveBaseGoods($data);

        $this->saveCategory($goods, $data['category_id']);
        $this->saveStoreCategory($goods, $data['store_category_id']);

        if ($data['hasoption'] == 1) {
            $this->saveOption($goods, $data);
        }

        $this->success('', '商品发布成功');
    }

    /**
     * 保存商品规格
     * @param $goods
     * @param $data
     */
    private function saveOption($goods, $data)
    {

    }

    /**
     * 保存商品分类
     * @param $goods
     * @param $categoryId
     */
    private function saveCategory($goods, $categoryId)
    {
        if (!$goods || !$categoryId) {
            return false;
        }

        $categoryIds = [$categoryId];

        GoodsService::saveCategorys($goods, $categoryIds);
    }

    /**
     * 保存店铺商品分类
     * @param $goods
     * @param $storeCategoryId
     */
    private function saveStoreCategory($goods, $storeCategoryId)
    {
        if (!$goods || !$storeCategoryId) {
            return false;
        }

        $storeCategoryId = [$storeCategoryId];

        \Shimmer_liveshop_plugin_merch\Services\GoodsService::saveCategorys($goods, $storeCategoryId);
    }

    /**
     * 保存商品基础数据
     * @param $data
     */
    private function saveBaseGoods($data, $goods = null)
    {
        if (!$goods) {
            $goods = new ShimmerLiveshopGoods();
        }

        $goodsData = [];

        $goodsData['uniacid'] = AppService::$uniacid;
        $goodsData['role_type'] = GlobalRoleService::MERCH;
        $goodsData['role_id'] = $this->store->id;

        $goodsData['title'] = $data['title'];
        $goodsData['subtitle'] = $data['subtitle'];
        $goodsData['category_id'] = $data['category_id'];
        $goodsData['unit'] = $data['unit'];
        $goodsData['marketprice'] = $data['marketprice'];
        $goodsData['productprice'] = $data['productprice'];
        $goodsData['expressprice'] = $data['expressprice'];
        $goodsData['stock'] = $data['stock'];
        $goodsData['is_free_freight'] = $data['is_free_freight'];
        $goodsData['description'] = $data['description'];
        $goodsData['state'] = $data['state'];

        if ($data['thumbs']) {
            foreach ($data['thumbs'] as $key => $thumb) {
                if ($key == 0) {
                    $goodsData['thumb'] = (string)$thumb['url'];
                }
                $goodsData['thumbs'][] = (string)$thumb['url'];
            }
        }

        if ($goodsData['thumbs']) {
            $goodsData['thumbs'] = implode(',', $goodsData['thumbs']);
        }
        if ($data['video']) {
            $goodsData['video'] = (string)$data['video'][0]['url'];
        }

        $goodsData['local_thumb'] = ImageService::ossImgToLocal($goodsData['thumb']);

        if ($goodsData['state'] == 1) {
            $goodsData['stateon_at'] = date('Y-m-d H:i:s');
        }

        $ret = $goods->fill($goodsData)->saveOrFail();

        if (!$ret) {
            return $this->error(-1, '保存失败');
        }

        return $goods;
    }

    /**
     * 商品保存参数检测
     * @param $data
     */
    private function checkSavePost($data)
    {
        if (!$data['title']) {
            return $this->error(-1, '请填写商品名称');
        }
        if (!$data['category_id']) {
            return $this->error(-1, '请选择商品类目');
        }
        if (!$data['store_category_id']) {
            return $this->error(-1, '请选择店铺分类');
        }
        if (count($data['thumbs']) == 0) {
            return $this->error(-1, '最少上传一张商品图片');
        }
        if ($data['marketprice'] <= 0) {
            return $this->error(-1, '商品价格填写错误');
        }
        if ($data['stock'] <= 0) {
            return $this->error(-1, '请填写商品库存');
        }
    }

    /**
     * 编辑商品
     * @param Request $request
     */
    public function update(Request $request)
    {
        $id = $request->route('id');

        $goods = $this->checkGoodsId($id);

        $data['hasoption'] = 0;//多规格
        $data['title'] = trim($request->input('title'));
        $data['subtitle'] = trim($request->input('subtitle'));
        $data['category_id'] = (int)$request->input('category_id');
        $data['store_category_id'] = (int)$request->input('store_category_id');
        $data['thumbs'] = $request->input('thumbs');
        $data['video'] = $request->input('video');
        $data['unit'] = trim($request->input('unit'));
        $data['marketprice'] = floatval($request->input('marketprice'));
        $data['productprice'] = floatval($request->input('productprice'));
        $data['expressprice'] = floatval($request->input('expressprice'));
        $data['stock'] = floatval($request->input('stock'));
        $data['is_free_freight'] = (int)$request->input('is_free_freight');
        $data['description'] = (string)($request->input('description'));
        $data['state'] = (int)$request->input('state');

        $this->checkSavePost($data);
        $goods = $this->saveBaseGoods($data, $goods);

        $this->saveCategory($goods, $data['category_id']);
        $this->saveStoreCategory($goods, $data['store_category_id']);

        if ($data['hasoption'] == 1) {
            $this->saveOption($goods, $data);
        }

        $this->success('', '商品修改成功');
    }

    /**
     * 修改商品上下架状态
     * @param Request $request
     * @return array|string
     * @throws \Throwable
     */
    public function upState(Request $request)
    {
        $id = $request->route('id');
        $state = $request->input('state');

        $goods = $this->checkGoodsId($id);

        $goods->state = $state;

        if ($state == 1) {
            $goods->stateon_at = date('Y-m-d H:i:s"');
        }

        $ret = $goods->saveOrFail();

        if ($ret) {
            return $this->success();
        } else {
            return $this->error(-1, '保存失败');
        }
    }

    /**
     * 商品加入直播间/从直播间删除
     * @param Request $request
     */
    public function liveGoodsUpdate(Request $request)
    {
        $id = $request->route('id');

        $live = AnchorService::getUserLive($this->uid);

        if (!$live) {
            return $this->error(-1, '直播间不存在55');
        }

        $goods = ShimmerLiveshopGoods::where('id', $id)
            ->where('role_id', $this->store->id)
            ->where('role_type', GlobalRoleService::MERCH)
            ->first();

        if (!$goods) {
            return $this->error(-1, '商品不存在');
        }


        $liveGoods = ShimmerLiveshopLiveGoods::where('live_id', $live->id)
            ->where('goods_id', $id)->first();

        if ($liveGoods) {
            $ret = $liveGoods->delete();
        } else {
            $liveGoods = new ShimmerLiveshopLiveGoods();
            $data = [
                'uniacid' => AppService::$uniacid,
                'live_id' => $live->id,
                'goods_id' => $id,
            ];

            $ret = $liveGoods->fill($data)->saveOrFail();
        }

        if ($ret) {
            return $this->success();
        } else {
            return $this->error(-1, '保存失败');
        }
    }

    /**
     * 获取商户直播商品列表
     */
    private function getLiveGoods()
    {
        $live = AnchorService::getUserLive($this->uid);
        if (!$live) {
            return [];
        }

        $prefix = config('database.connections.mysql.prefix');

        $list = ShimmerLiveshopGoods::leftJoin('shimmer_liveshop_live_goods', 'shimmer_liveshop_live_goods.goods_id', 'shimmer_liveshop_goods.id')
            ->selectRaw("{$prefix}shimmer_liveshop_goods.*")
            ->withoutGlobalScope('uniacid')
            ->where('shimmer_liveshop_goods.uniacid', AppService::$uniacid)
            ->where('shimmer_liveshop_live_goods.live_id', $live->id)
            ->orderBy('shimmer_liveshop_goods.sale_num', 'desc')
            ->orderBy('shimmer_liveshop_goods.id', 'desc')
            ->get();

        return $list;
    }

    /**
     * 获取仓库商品列表
     */
    private function getStorageGoods()
    {
        $request = request();

        $page = $request->input('page');
        $limit = $request->input('limit', 10);

        $offset = ($page - 1) * $limit;

        $list = ShimmerLiveshopGoods::where('role_id', $this->store->id)
            ->where('role_type', GlobalRoleService::MERCH)
            ->where('state', 0)
            ->orderBy('sale_num', 'desc')
            ->orderBy('id', 'desc')
            ->offset($offset)
            ->limit($limit)
            ->get();

        return $list;
    }

    /**
     * 获取出售中商品列表
     */
    private function getSaleingGoods()
    {
        $request = request();

        $page = $request->input('page');
        $limit = $request->input('limit', 10);

        $offset = ($page - 1) * $limit;

        $list = ShimmerLiveshopGoods::where('role_id', $this->store->id)
            ->where('role_type', GlobalRoleService::MERCH)
            ->where('state', 1)
            ->orderBy('sale_num', 'desc')
            ->orderBy('id', 'desc')
            ->offset($offset)
            ->limit($limit)
            ->get();

        return $list;
    }

    /**
     * 删除商品
     * @param Request $request
     */
    public function delete(Request $request)
    {
        $id = $request->route('id');

        if (!$id) {
            return $this->error(-1, '参数错误');
        }

        $goods = $this->checkGoodsId($id);

        $ret = $goods->delete();

        if ($ret) {
            return $this->success();
        } else {
            return $this->error(-1, '删除失败');
        }
    }

    /**
     * 通过商品id检测商品
     * @param $goodsId
     */
    private function checkGoodsId($goodsId)
    {
        if (!$goodsId) {
            apiException('参数错误');
        }

        $goods = ShimmerLiveshopGoods::where('id', $goodsId)
            ->where('role_id', $this->store->id)
            ->where('role_type', GlobalRoleService::MERCH)
            ->first();

        if (!$goods) {
            apiException('商品不存在');
        }

        return $goods;
    }

    /**
     * 获取商品详情
     * @param Request $request
     */
    public function info(Request $request)
    {
        $id = $request->route('id');

        $goods = $this->checkGoodsId($id);

        //获取分类
        $category = ShimmerLiveshopGoodsCategory::find($goods->category_id);
        $goods->category_name = $category->name;

        //获取店铺分类
        $storeCategory = \Shimmer_liveshop_plugin_merch\Services\GoodsService::getGoodsCategory($id);
        if ($storeCategory) {
            $goods->store_category_id = $storeCategory->id;
            $goods->store_category_name = $storeCategory->name;
        }

        $this->success($goods);
    }
}