<?php

/**
 * Created by PhpStorm.
 * User: lirui
 * Date: 2020/2/23
 * Time: 下午7:55
 */

namespace Shimmer_liveshop_plugin_merch\Http\Controllers\Api\User;

use App\Lib\Bemrc\UtilService;
use App\Modules\Liveapi\Services\Live\AnchorService;
use Illuminate\Http\Request;
use Shimmer_liveshop_plugin_merch\Http\Controllers\Api\Controller;
use Shimmer_liveshop_plugin_merch\Models\ShimmerLiveshopMerchStore;
use Shimmer_liveshop_plugin_merch\Models\ShimmerLiveshopMerchStoreRelation;
use Shimmer_liveshop_plugin_merch\Models\ShimmerLiveshopMerchStoreVisit;
use Shimmer_liveshop_plugin_merch\Services\StoreService;
use Shimmer_liveshop_plugin_merch\Services\StoreStatService;
use Shimmer_liveshop_plugin_merch\Services\UserService;

class StoreController extends Controller
{
    /**
     * 店铺详情
     * @param Request $request
     */
    public function detail(Request $request)
    {
        $id = $request->route('store_id');

        if (!$id) {
            return $this->error(-1, '参数错误');
        }

        $store = ShimmerLiveshopMerchStore::find($id);

        if (!$store) {
            return $this->error(-1, '店铺不存在！');
        }

        $store = $store->formatData();

        $store->is_anchor = AnchorService::userIsAnchor($this->uid);
        $store->is_collect = ShimmerLiveshopMerchStoreRelation::hasRelation($store->id, $this->uid, 'collect');

        $store->visit_user_num = StoreStatService::getVisitUserNum($store);
        $store->viewers = StoreService::getViewers($store, 5);
        $store->live = AnchorService::getUserLive($this->uid);

        $this->success($store);
    }

    /**
     * 店铺访问统计
     * @param Request $request
     */
    public function visit(Request $request)
    {
        $id = $request->route('store_id');

        if (!$id) {
            return $this->error(-1, '参数错误');
        }

        ShimmerLiveshopMerchStoreVisit::addVisit($id, $this->uid);

        $this->success();
    }

    /**
     * 收藏 | 取消店铺
     * @param Request $request
     */
    public function collect(Request $request)
    {
        $id = $request->route('store_id');

        if (!$id) {
            return $this->error(-1, '参数错误');
        }

        ShimmerLiveshopMerchStoreRelation::upRelation($id, $this->uid, 'collect');

        $this->success();
    }

    /**
     * 用户收藏的店铺列表
     * @param Request $request
     */
    public function collectList(Request $request)
    {
        list($page, $limit) = UtilService::getMore([
            ['page', 0],
            ['limit', 0]
        ], $request, true);

        if (!$limit) {
            $limit = 5;
        }

        $data = UserService::getUserCollectStores($this->uid, (int)$page, (int)$limit);

        return $this->success($data);
    }
}