<?php

/**
 * 订单确认收货，店铺账户更新
 * Created by PhpStorm.
 * User: lirui
 * Date: 2020/2/21
 * Time: 1:13 PM
 */

namespace Shimmer_liveshop_plugin_merch\Listeners\Order;

use App\Events\OrderPaySuccess;
use App\Events\OrderTakeed;
use App\Services\GlobalRoleService;
use Illuminate\Support\Facades\Log;
use Shimmer_liveshop_plugin_merch\Models\ShimmerLiveshopMerchStoreBill;
use Shimmer_liveshop_plugin_merch\Services\StoreService;

class OrderTakeedAccount
{
    public $event;
    public $order;
    public $attach;
    public $payType;

    /**
     * Handle the event.
     *
     * @param  OrderPaySuccess $event
     * @return void
     */
    public function handle(OrderTakeed $event)
    {
        $this->order = $event->order;
        $this->attach = $event->attach;
        $this->payType = $event->payType;

        if ($this->order->role_type
            != GlobalRoleService::MERCH
            || $this->order->role_id <= 0
        ) {
            return;
        }

        $role = GlobalRoleService::getRole($this->order->role_type, $this->order->role_id);

        if (!$role) {
            Log::error('店铺不存在888');
            return;
        }

        $bill = ShimmerLiveshopMerchStoreBill::where('order_id', $this->order->id)
            ->where('category', 1)->first();

        if (!$bill) {
            Log::error('确认收货未找到商户账单');
            return;
        }

        //获取待结算转入余额金额
        $moneyGroup = StoreService::getTakeedMoney($bill->money, $this->order, $role);

        if ($moneyGroup['money'] <= 0) {
            Log::error('分销佣金超出账单金额');
            return;
        }

        $role->pending_money = $role->pending_money - $bill->money;
        $role->balance_money = $role->balance_money + $moneyGroup['money'];
        $role->save();

        //记录账单
        ShimmerLiveshopMerchStoreBill::addRecord(2, $this->order->uid, $this->order->id, $role->id, $moneyGroup['money'], 2, 3, 0, $moneyGroup['commissionDeduct']);
    }
}
