<?php

namespace Shimmer_liveshop_plugin_merch\Models;

use App\Models\BaseUniacidModel;
use App\Services\AppService;

class ShimmerLiveshopMerchStoreRelation extends BaseUniacidModel
{
    protected $guarded = [];
    public $dateFormat = 'U';

    public static $types = [
        'collect' => '收藏',
        'like' => '点赞',
    ];

    /**
     * 判断用户和店铺是否存在关系
     * @param $storeId
     * @param $uid
     * @param string $type
     * @param string $category
     */
    public static function hasRelation($storeId, $uid, $type)
    {
        if (!$storeId || !$type || !$uid) {
            return false;
        }

        return self::where('store_id', $storeId)
                ->where('uid', $uid)
                ->where('type', $type)->count() > 0;
    }

    /**
     * 更新关系
     * @param $storeId
     * @param $uid
     * @param $type
     */
    public static function upRelation($storeId, $uid, $type)
    {
        $relation = self::where('store_id', $storeId)
            ->where('uid', $uid)
            ->where('type', $type)->first();

        if ($relation) {
            return $relation->delete();
        } else {
            return self::storeRelation($storeId, $uid, $type);
        }
    }

    /**
     * 添加点赞 收藏
     * @param $storeId
     * @param $uid
     * @param $relationType
     * @param string $category
     * @return bool
     */
    public static function storeRelation($storeId, $uid, $relationType)
    {
        if (!$storeId) {
            return false;
        }

        $relationType = strtolower($relationType);

        $data = ['uniacid' => AppService::$uniacid, 'uid' => $uid, 'store_id' => $storeId, 'type' => $relationType];

        if (self::be($data)) {
            return true;
        }

        $model = new self;

        $ret = $model->fill($data)->save();

        return $ret;
    }

    /**
     * 取消 点赞 收藏
     * @param $storeId
     * @param $uid
     * @param $relationType
     * @param string $category
     * @return bool
     */
    public static function unStoreRelation($storeId, $uid, $relationType)
    {
        if (!$storeId) {
            return false;
        }

        $relationType = strtolower($relationType);

        $ret = self::where('uid', $uid)
            ->where('store_id', $storeId)
            ->where('type', $relationType)->delete();

        return $ret;
    }

}