<?php

namespace Shimmer_liveshop_plugin_merch\Providers;

use Illuminate\Contracts\Filesystem\Filesystem;
use Illuminate\Filesystem\FilesystemAdapter;
use Illuminate\Foundation\Support\Providers\RouteServiceProvider as ServiceProvider;
use Illuminate\Support\Facades\Route;
use Shimmer_liveshop_plugin_merch\Services\PluginService;

class RouteServiceProvider extends ServiceProvider
{
    /**
     * This namespace is applied to your controller routes.
     *
     * In addition, it is set as the URL generator's root namespace.
     *
     * @var string
     */
    protected $namespace = 'aaaa';

    protected $plugin;

    /**
     * Define your route model bindings, pattern filters, etc.
     *
     * @return void
     */
    public function boot()
    {
        $this->setNamespace();
        parent::boot();
    }

    /**
     * Define the routes for the module.
     *
     * @return void
     */
    public function map()
    {
        $this->mapWebRoutes();

        $this->mapApiRoutes();

        //
    }

    /**
     * 设置命名空间
     */
    protected function setNamespace()
    {
        $this->plugin = PluginService::getPlugin();
        $this->namespace = $this->plugin->namespace . "Http\Controllers";
    }

    /**
     * Define the "web" routes for the module.
     *
     * These routes all receive session state, CSRF protection, etc.
     *
     * @return void
     */
    protected function mapWebRoutes()
    {
        Route::group([
            'middleware' => ['web', 'live.auth'],
            'prefix' => '{uniacid}/plugin/' . $this->plugin->slug . '/',
            'namespace' => $this->namespace . '\Web',
            'as' => 'web_plugin_' . $this->plugin->slug . '.',
        ], function ($router) {
            $file = $this->app->make(\Illuminate\Filesystem\Filesystem::class);
            $path = plugin_path($this->plugin->slug, 'Routes/web');
            $routeFiles = $file->glob($path . '/*.php');

            foreach ($routeFiles as $routeFile) {
                require $routeFile;
            }
        });
    }

    /**
     * Define the "api" routes for the module.
     *
     * These routes are typically stateless.
     *
     * @return void
     */
    protected function mapApiRoutes()
    {
        Route::group([
            'middleware' => ['api', 'liveapi.cors'],
            'namespace' => $this->namespace . '\Api',
            'as' => 'api_plugin_' . $this->plugin->slug . '.',
            'prefix' => 'api/{uniacid}/{type}/plugin/' . $this->plugin->slug . '/',
        ], function ($router) {
            $file = $this->app->make(\Illuminate\Filesystem\Filesystem::class);
            $path = plugin_path($this->plugin->slug, 'Routes/api');
            $routeFiles = $file->glob($path . '/*.php');

            foreach ($routeFiles as $routeFile) {
                require $routeFile;
            }
        });
    }
}
