@extends('live::layouts.master')
@push('stylesheets')
    <link href="{{ URL::asset('/static/web/css/plugins/sweetalert/sweetalert.css')}}?v=33" rel="stylesheet">
@endpush
@section('content')
    <div class="row">
        <div class="col-lg-12">
            <div class="ibox">
                <div class="ibox-content">
                    <form class="layui-form" action="">
                        <div class="layui-inline">
                            <label class="layui-form-label">店铺名称:</label>
                            <div class="layui-input-inline">
                                <input type="text" name="query[store_name]" lay-verify="" placeholder="店铺搜索"
                                       value="{{$query['store_name']}}" autocomplete="off" class="layui-input">
                            </div>
                        </div>
                        <div class="layui-inline">
                            <div class="layui-inline">
                                <label class="layui-form-label">提现方式:</label>
                                <div class="layui-input-inline">
                                    <select name="query[type]" lay-verify="">
                                        <option value="">全部</option>
                                        @foreach($queryData['type'] as $index=>$item)
                                            <option value="{{$index}}" {{ isNotNullAndNotEmptyString($query['type']) && $query['type']==$index ? 'selected': ''}}>{{$item}}</option>
                                        @endforeach
                                    </select>
                                </div>
                            </div>
                        </div>
                        <div class="layui-inline">
                            <div class="layui-inline">
                                <label class="layui-form-label">打款方式:</label>
                                <div class="layui-input-inline">
                                    <select name="query[type]" lay-verify="">
                                        <option value="">全部</option>
                                        @foreach($queryData['pay_type'] as $index=>$item)
                                            <option value="{{$index}}" {{ isNotNullAndNotEmptyString($query['pay_type']) && $query['pay_type']==$index ? 'selected': ''}}>{{$item}}</option>
                                        @endforeach
                                    </select>
                                </div>
                            </div>
                        </div>
                        <div class="layui-inline">
                            <label class="layui-form-label">申请时间:</label>
                            <div class="layui-input-inline" style="width: 150px;">
                                <input type="text" class="layui-input  start_time" value="{{$query['start_time']}}"
                                       name="query[start_time]"
                                       readonly>
                            </div>
                            -
                            <div class="layui-input-inline" style="width: 150px;">
                                <input type="text" class="layui-input end_time" value="{{$query['end_time']}}"
                                       name="query[end_time]"
                                       readonly>
                            </div>
                        </div>
                        <div class="layui-inline" style="padding-bottom: 5px">
                            <div class="layui-input-inline">
                                <input type="hidden" name="page_num" value="{{$currentPageNum}}">
                                <button name="submit" class="btn btn-success" type="submit">搜索</button>
                            </div>
                        </div>
                    </form>

                    @include('live::common.toolbar', ['pager' => $pager])

                    <form class="layui-form form-horizontal" id="ajaxform">
                        <table class="layui-table">
                            <thead>
                            <tr>
                                <th>ID</th>
                                <th>店铺</th>
                                <th>粉丝</th>
                                <th>会员</th>
                                <th>提现金额</th>
                                <th>提现方式</th>
                                <th>申请时间</th>
                                <th>驳回原因</th>
                                <th>状态</th>
                            </tr>
                            </thead>
                            <tbody>
                            @foreach($pager as $item)
                                <tr>
                                    <td>{{$item->id}}</td>
                                    <td>{{$item->store->name}}</td>
                                    <td>
                                        <img src="{{$item->user->avatar}}"
                                             class="rounded-circle img-sm img_pop_show"/><br/>
                                        {{$item->user->nickname}}
                                    </td>
                                    <td>
                                        {{$item->user->realname}}<br/>
                                        {{$item->user->mobile}}
                                    </td>
                                    <td>
                                        ￥{{$item->balance}}
                                        @if($item->commission>0)
                                            <br/>
                                            <span class="text-danger">手续费{{$item->commission}}</span>
                                        @endif
                                        <br />
                                        <span class="text-success">实际打款:{{$item->balance - $item->commission}}</span>
                                    </td>
                                    <td>
                                        @if($item->type=='bank')
                                            线下银行卡打款<br/>
                                            姓名:{{$item->name}}<br>
                                            银行卡号:{{$item->cardnum}}<br>
                                            所属银行:{{$item->bankname}}<br>
                                            银行开户地址:{{$item->bank_address}}<br>
                                        @elseif($item->type=='alipay')
                                            线下支付宝打款<br/>
                                            支付宝:{{$item->alipay_code}}<br>
                                        @elseif($item->type=='weixin')
                                            线下微信打款<br/>
                                            昵称:{{$item->user->nickname}}<br>
                                            微信号:{{$item->weixin}}<br>
                                        @endif
                                    </td>
                                    <td>{{$item->created_at}}</td>
                                    <td>{{$item->fail_msg}}</td>
                                    <td>
                                        @if($item->status==0)
                                            <span class="label label-info">待打款</span>
                                        @elseif($item->status==1)
                                            <span class="label label-success">打款成功</span>
                                        @elseif($item->status==-2)
                                            <span class="label label-danger">打款失败</span><br/>
                                            失败原因: {{$item->fail_msg}}<br/>
                                            失败时间: @php echo date('Y-m-d H:i:s',$item->fail_time); @endphp
                                        @elseif($item->status==-1)
                                            <span class="label label-warning">驳回申请</span>
                                        @endif
                                    </td>
                                </tr>
                            @endforeach
                            </tbody>
                        </table>
                    </form>
                    @include('live::common.pagination', ['pager' => $pager])
                </div>
            </div>
        </div>
    </div>
@endsection

@section('script')
    </script>
@endsection