<?php
/**
 * 订单service
 * Created by PhpStorm.
 * User: lirui
 * Date: 2020/1/28
 * Time: 下午6:33
 */

namespace Shimmer_liveshop_plugin_merch\Services;


use App\Models\ShimmerLiveshopGoodsReply;
use App\Models\ShimmerLiveshopStoreOrder;
use App\Models\ShimmerLiveshopStoreOrderChangeLog;
use App\Modules\Liveapi\Models\ShimmerLiveshopConfig;
use App\Services\AppService;
use Shimmer_liveshop_plugin_merch\Models\ShimmerLiveshopMerchConfig;
use Shimmer_liveshop_plugin_merch\Models\ShimmerLiveshopMerchStore;
use Shimmer_liveshop_plugin_merch\Models\ShimmerLiveshopMerchStoreApply;

class OrderService extends \App\Modules\Liveapi\Services\OrderService
{
    /**
     * 生成申请支付订单
     * @param $userInfo
     * @return mixed
     */
    public static function createApplyPayOrder($userInfo)
    {
        $intoConfig = ShimmerLiveshopMerchConfig::getByType('into');
        $lastApply = ShimmerLiveshopMerchStoreApply::uid($userInfo['id'])
            ->where('status', '!=', 2)
            ->orderBy('id', 'desc')
            ->first();

        $store = ShimmerLiveshopMerchStore::uid($userInfo['id'])
            ->first();

        if ($store) {
            return self::error(-1, '已有店铺，无需申请!');
        }

        if ($lastApply && $lastApply->status == 0
            && $lastApply->order_id > 0
        ) {
            $order = ShimmerLiveshopStoreOrder::find($lastApply->order_id);
            return $order;
        }

        $orderInfo = [
            'uniacid' => AppService::$uniacid,
            'uid' => $userInfo['id'],
            'real_name' => $userInfo['realname'],
            'user_phone' => $userInfo['mobile'],
            'type' => 4,
            'order_id' => self::getNewOrderId(),
            'total_price' => $intoConfig['apply_limit_pay_money'],
            'pay_price' => $intoConfig['apply_limit_pay_money'],
            'paid' => 0,
            'is_channel' => 1,
            'add_time' => time(),
        ];

        $key = md5($orderInfo['uid'] . '_4_' . $orderInfo['order_id']);
        $orderInfo['unique'] = $key;

        $order = new ShimmerLiveshopStoreOrder();
        $saveRet = $order->fill($orderInfo)->saveOrFail();

        if ($lastApply) {
            $lastApply->order_id = $order->id;
            $lastApply->save();
        } else {
            $applyInfo = [
                'uniacid' => AppService::$uniacid,
                'uid' => $userInfo['id'],
                'order_id' => $order->id,
                'apply_limit' => $intoConfig['apply_limit'],
                'status' => -1,
            ];
            $apply = new ShimmerLiveshopMerchStoreApply();
            $apply->fill($applyInfo)->save();
        }

        if (!$saveRet) {
            return self::error(-1, '订单生成失败1!');
        }

        self::log($order['id'], 'cache_key_create_order', '订单生成');

        return $order;
    }

    /**
     * 创建诚信商家认证订单
     * @param $userInfo
     */
    public static function createApproveIntegrityPayOrder($userInfo)
    {
        $config = ShimmerLiveshopMerchConfig::getByType('base');
        $configIntegrity = $config['integrity'];

        $store = StoreService::getUserStore($userInfo['id']);

        if ($store['integrity_order_id']) {
            return $order = ShimmerLiveshopStoreOrder::find($store['integrity_order_id']);
        }

        $orderInfo = [
            'uniacid' => AppService::$uniacid,
            'uid' => $userInfo['id'],
            'real_name' => $userInfo['realname'],
            'user_phone' => $userInfo['mobile'],
            'type' => 6,
            'order_id' => self::getNewOrderId(),
            'total_price' => $configIntegrity['money'],
            'pay_price' => $configIntegrity['money'],
            'paid' => 0,
            'is_channel' => 1,
            'add_time' => time(),
        ];

        $key = md5($orderInfo['uid'] . '_6_' . $orderInfo['order_id']);
        $orderInfo['unique'] = $key;

        $order = new ShimmerLiveshopStoreOrder();
        $saveRet = $order->fill($orderInfo)->saveOrFail();

        if (!$saveRet) {
            return self::error(-1, '订单生成失败77!');
        }

        $store->integrity_order_id = $order->id;
        $store->save();

        self::log($order['id'], 'cache_key_create_order', '订单生成');

        return $order;
    }

    /**
     * 订单详情数据格式化
     * @param $order
     * @param bool $detail 是否包含商品数据
     * @param bool $hasUserinfo 是否获取用户信息
     * @return mixed
     */
    public static function tidyOrder($order, $detail = false, $hasUserinfo = false)
    {
        if ($detail && $order['id']) {
            $cartInfos = self::getCartInfosByOid($order['id']);

            $carts = [];
            foreach ($cartInfos as $k => $cartInfo) {
                $cart = $cartInfo->cart_info;
                $cart['unique'] = $cartInfo->unique;
                //是否评价字段
                $cart['is_reply'] = ShimmerLiveshopGoodsReply::getOrderGoodsReplayCount($order['id'], $cartInfo['unique']);
                $carts[] = $cart;
            }
            $order['cartInfo'] = $carts;
        }

        $status = [];
        $shopConfig = ShimmerLiveshopConfig::getByType(13);
        $baseConfig = ShimmerLiveshopConfig::getByType(1);
        $createTime = $order['created_at']->timestamp;

        if (!$order['paid'] && $order['pay_type'] == 'offline' && !$order['status'] >= 2) {
            $status['_type'] = 9;
            $status['_title'] = '线下付款';
            $status['_msg'] = '等待处理';
            $status['_class'] = 'nobuy';
        } else if (!$order['paid']) {
            $status['_type'] = 0;
            $status['_title'] = '等待买家付款';
            //系统预设取消订单时间段
            //获取配置
            $time = bcadd($createTime, (int)$shopConfig['order_cancel_time'] * 3600, 0);
            $status['_msg'] = '请在' . date('m-d H:i:s', $time) . '前付款，超时订单将自动关闭';
            $status['_class'] = 'nobuy';
        } else if ($order['refund_status'] == 1) {
            $status['_type'] = -1;
            $status['_title'] = '申请退款中';
            $status['_msg'] = '';
            $status['_class'] = 'state-sqtk';
        } else if ($order['refund_status'] == 2) {
            $status['_type'] = -2;
            $status['_title'] = '已退款';
            $status['_msg'] = '';
            $status['_class'] = 'state-sqtk';
        } else if (!$order['status']) {
            $status['_type'] = 1;
            $status['_title'] = '待发货';
            $status['_msg'] = '';
            $status['_class'] = 'state-nfh';
        } else if ($order['status'] == 1) {
            if ($order['delivery_type'] == 'send') {//TODO 送货
                $status['_type'] = 2;
                $status['_title'] = '待收货';
                $status['_msg'] = '';
                $status['_class'] = 'state-ysh';
            } else {//TODO  发货
                $status['_type'] = 2;
                $status['_title'] = '待收货';
                $status['_msg'] = '';
                $status['_class'] = 'state-ysh';
            }
        } else if ($order['status'] == 2) {
            $status['_type'] = 3;
            $status['_title'] = '待评价';
            $status['_msg'] = '已收货,快去评价一下吧';
            $status['_class'] = 'state-ypj';
        } else if ($order['status'] == 3) {
            $status['_type'] = 4;
            $status['_title'] = '交易完成';
            $status['_msg'] = '订单已完成';
            $status['_class'] = 'state-ytk';
        }

        if ($order['is_del'] == 1) {
            $status['_type'] = -99;
            $status['_title'] = '交易关闭';
            $status['_msg'] = '';
        }

        if (isset($order['pay_type'])) {
            $status['_payType'] = isset(self::$payType[$order['pay_type']]) ? self::$payType[$order['pay_type']] : '其他方式';
        }

        if (isset($order['delivery_type'])) {
            $status['_deliveryType'] = isset(self::$deliveryType[$order['delivery_type']]) ? self::$deliveryType[$order['delivery_type']] : '其他方式';
        }

        $order['_status'] = $status;

        if (isset($order['pay_time']) && $order['pay_time']) {
            $order['_pay_time'] = date('Y-m-d H:i:s', $order['pay_time']);
        } else {
            $order['_pay_time'] = $order['created_at']->format('Y-m-d H:i:s');
        }

        $order['_add_time'] = $order['created_at']->format('Y-m-d H:i:s');
        $order['status_pic'] = '';

        $order['offlinePayStatus'] = 2;

        if ($hasUserinfo) {
            $order['user_info'] = $order->user;
        }

        //快递方式
        $deliveryType = ShimmerLiveshopStoreOrder::$deliveryTypes[$order['delivery_type']];
        $order['goods_price'] = $order['pay_price'] - $order['pay_postage'];

        if (!$deliveryType) {
            $deliveryType = '快递发货';
        }

        $order->add_time_y = $order['created_at']->format('Y-m-d');
        $order->add_time_h = $order['created_at']->format('H:i:s');

        $order['_delivery_type'] = $deliveryType;
        $order['consumer_hotline'] = (string)$baseConfig['consumer_hotline'];


        //获取退款信息
        if ($order['refund_status'] > 0) {
            $order['refund_reason_time'] = date('Y-m-d H:i:s', $order['refund_reason_time']);
        }

        return $order;
    }

}