<?php
/**
 * 店铺
 * Created by PhpStorm.
 * User: lirui
 * Date: 2020/1/28
 * Time: 下午9:46
 */

namespace Shimmer_liveshop_plugin_merch\Services;


use App\Models\ShimmerLiveshopStoreAsset;
use App\Models\ShimmerLiveshopTemplate;
use App\Models\ShimmerLiveshopUser;
use App\Modules\Liveapi\Services\Spread\CommissionService;
use App\Services\AppService;
use App\Services\BaseService;
use App\Modules\Liveapi\Services\MiniProgramService;
use Shimmer_liveshop_plugin_merch\Models\ShimmerLiveshopMerchConfig;
use Shimmer_liveshop_plugin_merch\Models\ShimmerLiveshopMerchStore;
use Shimmer_liveshop_plugin_merch\Models\ShimmerLiveshopMerchStoreApply;
use Shimmer_liveshop_plugin_merch\Models\ShimmerLiveshopMerchStoreVisit;

class StoreService extends BaseService
{
    public static $stores = [];

    /**
     * 店铺申请通过
     * @param ShimmerLiveshopCashApply $apply
     */
    public static function applyAccess(ShimmerLiveshopMerchStoreApply $apply)
    {
        $store = ShimmerLiveshopMerchStore::getByUid($apply->uid);
        $intoConfig = ShimmerLiveshopMerchConfig::getByType('into');

        if ($store) {
            return self::error(-1, '已有店铺，无需申请');
        }

        $store = new ShimmerLiveshopMerchStore();
        //店铺信息
        $storeData = [
            'uniacid' => AppService::$uniacid,
            'uid' => $apply->uid,
            'apply_id' => $apply->id,
            'applytime' => $apply->created_at->timestamp,
            'status' => 0,
        ];
        //店铺有效期
        if ($intoConfig['valid_type'] == 2
            && $intoConfig['valid_month'] > 0
        ) {
            $storeData['accounttime'] = time() + ($intoConfig['valid_month'] * 86400 * 30);
        }

        $ret = $store->fill($storeData)->saveOrFail();

        if (!$ret) {
            return self::error(-1, '系统错误，店铺保存失败！');
        }

        $apply->jointime = time();
        $apply->status = 1;
        $apply->save();

        //发送通知
        $app = MiniProgramService::miniprogram();
        $templateId = ShimmerLiveshopTemplate::getTemplateIdByKey(1482);
        $user = ShimmerLiveshopUser::find($apply->uid);

        $data = [
            'template_id' => $templateId,
            'touser' => $user->wxapp_openid,
            'page' => '/common/pages/user/user',
            'data' => [
                'thing2' => [
                    'value' => '申请店铺审核通知',
                ],
                'phrase3' => [
                    'value' => '审核通过',
                ],
                'thing5' => [
                    'value' => '您的店铺已经审核通过，快去完善店铺信息吧',
                ],
                'date8' => [
                    'value' => date('Y-m-d H:i:s')
                ]
            ],
        ];

        $app->subscribe_message->send($data);
        return $store;
    }

    /**
     * 审核拒绝
     * @param ShimmerLiveshopMerchStoreApply $apply
     * @param $reason 拒绝原因
     */
    public static function applyReject(ShimmerLiveshopMerchStoreApply $apply, $reason)
    {
        if (!$reason) {
            return self::error(-1, '请填写拒绝原因');
        }

        $apply->reason = $reason;
        $apply->status = 2;
        $apply->save();
        $order = $apply->order;

        //退款
        if ($order) {
            $app = MiniProgramService::payment();
            $refundNo = \App\Services\OrderService::getNewRefundOrderId();

            $result = $app->refund->byOutTradeNumber($order->order_id, $refundNo, $order->pay_price * 100, $order->pay_price * 100, [
                'refund_desc' => '店铺申请驳回:' . $reason
            ]);
            //插入资产流水表
            $remark = '驳回原因:' . $reason;
            ShimmerLiveshopStoreAsset::addRecord(ShimmerLiveshopStoreAsset::MERCH_APPLY_REJECT, $order['mer_id'], $order['uid'], $order['real_name'], $order['user_phone'], $order['id'], $order['pay_price'], 2, 1, $remark);
        }

        //发送通知
        $app = MiniProgramService::miniprogram();
        $templateId = ShimmerLiveshopTemplate::getTemplateIdByKey(1482);
        $user = ShimmerLiveshopUser::find($apply->uid);

        $data = [
            'template_id' => $templateId,
            'touser' => $user->wxapp_openid,
            'page' => '/common/pages/user/user',
            'data' => [
                'thing2' => [
                    'value' => '申请店铺审核驳回',
                ],
                'phrase3' => [
                    'value' => '审核驳回',
                ],
                'thing5' => [
                    'value' => '驳回原因:' . $reason,
                ],
                'date8' => [
                    'value' => date('Y-m-d H:i:s')
                ]
            ],
        ];

        $app->subscribe_message->send($data);
    }

    /**
     * 诚信商家认证成功
     * @param $order
     */
    public static function integrityApprovePaySuccess($order)
    {
        $store = StoreService::getUserStore($order->uid);
        $store->is_integrity = 1;
        $store->save();

        $user = $order->user;

        ShimmerLiveshopStoreAsset::addRecord(ShimmerLiveshopStoreAsset::MERCH_APPROVE_INTEGRITY, $store->id, $order->id, $user->realname, $user->mobile, $order->id, $order->pay_money, 1, 1, '');
    }

    /**
     * 获取用户是否有店铺
     * @param $uid
     */
    public static function getUserStore($uid)
    {
        if (!$uid) {
            return false;
        }

        return ShimmerLiveshopMerchStore::uid($uid)->first();
    }

    /**
     * 检车店铺
     * @param $store
     */
    public static function checkStore($store)
    {
        if (!$store) {
            apiException('该的店铺不存在或已被删除');
        }
        if ($store->status == 2) {
            apiException('该的店铺已被暂停');
        }
        if ($store->accounttime && $store->accounttime < time()) {
            apiException('该的店铺服务已到期');
        }
    }

    /**
     * 获取诚信商家文本
     */
    public static function getIntegrityText()
    {
        $integrityText = '诚信商家';

        $config = ShimmerLiveshopMerchConfig::getByType('base');

        if ($config['integrity']['integrity_text']) {
            $integrityText = $config['integrity']['integrity_text'];
        }

        return $integrityText;
    }

    /**
     * 通过id获取店铺
     * @param $id
     */
    public static function getStoreById($id)
    {
        if (!$id) {
            return false;
        }

        if (isset(self::$stores[$id])) {
            return self::$stores[$id];
        } else {
            self::$stores[$id] = ShimmerLiveshopMerchStore::find($id);
            return self::$stores[$id];
        }
    }

    /**
     * 获取店铺可收入待结算金额
     * @param $money
     * @param $role
     */
    public static function getIncomeMoney($money, $role)
    {
        $result = [
            'money' => 0,
            'platformDeduct' => 0,
        ];;

        if (!$money || !$role) {
            return $result;
        }

        //获取商户税率
        $baseConfig = ShimmerLiveshopMerchConfig::getByType('base');
        $settlementRate = $baseConfig['settlement_rate'];

        if ($settlementRate <= 0) {
            $result['money'] = $money;
            return $result;
        }

        $result['platformDeduct'] = $money * ($settlementRate / 100);
        $result['money'] = $money - $result['platformDeduct'];

        return $result;
    }

    /**
     * 获取待结算可转入余额金额
     * @param $money
     * @param $order
     * @param $role
     */
    public static function getTakeedMoney($money, $order, $role)
    {
        $result = [
            'money' => 0,
            'commissionDeduct' => 0,
        ];

        if (!$money || !$order) {
            return $result;
        }

        //获取商户税率
        $baseConfig = ShimmerLiveshopMerchConfig::getByType('base');
        $isDeductcommission = $baseConfig['is_deduct_commission'];

        if (!$isDeductcommission) {
            $result['money'] = $money;
            return $result;
        }

        $result['commissionDeduct'] = (float)CommissionService::getCommissionByOrderId($order->id);

        $result['money'] = $money - $result['commissionDeduct'];

        return $result;
    }

    /**
     * 获取最新访问用户
     * @param $store
     */
    public static function getViewers($store, $limit = 10)
    {
        $uids = ShimmerLiveshopMerchStoreVisit::where('store_id', $store->id)
            ->where('type', 'view')
            ->groupBy('uid')
            ->limit($limit)
            ->get()
            ->pluck('uid');

        return ShimmerLiveshopUser::whereIn('id', $uids)->get();
    }
}